/*
 * Created on Jul 22, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine;

import java.util.HashMap;

import diagnose.graphicengine.elements.Document;
import diagnose.graphicengine.elements.system.Selection;
import diagnose.graphicengine.libraries.Librarian;

/**
 * Contains abilities of an manager to invoke events
 */
public class Abilities {

    private HashMap ability = new HashMap();
    
    public Abilities(Document document, Selection selection, Selection clipboard, 
            UndoRedoStack undoRedoStack, Librarian librarian) {
        super();
        
        setAbility("undo", undoRedoStack.canUndo());
        setAbility("redo", undoRedoStack.canRedo());
        setAbility("groupElements", (selection != null) && (selection.getElementsCount() > 1));
        setAbility("splitElements", selection.hasSplitableElements());
        setAbility("alignSelection", ((selection != null) && (selection.getElementsCount() > 1)));
        setAbility("saveIntoLibrary", librarian.getLibCount() > 0);
        setAbility("sendInAnyDirection", (selection != null) && (selection.getElementsCount() > 0) 
                && (selection.getElementsCount() < document.getElementsCount()));
        setAbility("selectAll", document.getElementsCount() > 0);
        setAbility("paste", (clipboard != null) && (clipboard.getElementsCount() > 0));
        setAbility("copy", (selection != null) && (selection.getElementsCount() > 0));
        setAbility("cut", (selection != null) && (selection.getElementsCount() > 0));
        setAbility("delete", (selection != null) && (selection.getElementsCount() > 0));
        setAbility("selectNone", (selection != null) && (selection.getElementsCount() > 0));
        setAbility("showPropertyDialog", (selection != null) && (selection.getElementsCount() > 0)
                && (selection.getProperties().getPropertyDescriptor().getItemsCount() > 0));
        
    }
    
    public boolean isAble(String feature) {
        return ((Boolean)ability.get(feature)).booleanValue();
    }
    
    private void setAbility(String feature, boolean value) {
        ability.put(feature, new Boolean(value));
    }
}
