/*
 * Created on Jul 22, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine;

import java.io.File;
import java.io.IOException;

import org.jdom.JDOMException;

import diagnose.graphicengine.elements.ActiveElement;
import diagnose.graphicengine.elements.Document;
import diagnose.graphicengine.elements.LibraryElement;
import diagnose.graphicengine.elements.PointBasedElement;
import diagnose.graphicengine.elements.RectBasedElement;
import diagnose.graphicengine.elements.implementations.ArrowElement;
import diagnose.graphicengine.elements.implementations.GroupElement;
import diagnose.graphicengine.elements.implementations.LineElement;
import diagnose.graphicengine.elements.implementations.OvalElement;
import diagnose.graphicengine.elements.implementations.PolyLineElement;
import diagnose.graphicengine.elements.implementations.PolygonElement;
import diagnose.graphicengine.elements.implementations.RectElement;
import diagnose.graphicengine.elements.implementations.TextElement;
import diagnose.graphicengine.elements.system.GuideLine;
import diagnose.graphicengine.elements.system.GuideLines;
import diagnose.graphicengine.libraries.Librarian;
import diagnose.graphicengine.libraries.Library;
import diagnose.utils.properties.BoolProperty;
import diagnose.utils.properties.ColorProperty;
import diagnose.utils.properties.DoubleProperty;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.MethodProperty;
import diagnose.utils.properties.PointProperty;
import diagnose.utils.properties.Properties;
import diagnose.utils.properties.PropertyDescription;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.properties.SelectPropertyDescription;
import diagnose.utils.properties.StringProperty;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;

/**
 * Default deserializing tool
 */
public final class DefaultDeserializer {

    private static Deserializer deserializer = new Deserializer();
    
    static {
        deserializer.registerElementClass(Document.class);       
        deserializer.registerElementClass(Properties.class);
        deserializer.registerElementClass(LineElement.class);
        deserializer.registerElementClass(PolyLineElement.class);
        deserializer.registerElementClass(LibraryElement.class);
        deserializer.registerElementClass(GroupElement.class);
        deserializer.registerElementClass(BoolProperty.class);
        deserializer.registerElementClass(IntProperty.class);
        deserializer.registerElementClass(PointProperty.class);
        deserializer.registerElementClass(DoubleProperty.class);
        deserializer.registerElementClass(StringProperty.class);
        deserializer.registerElementClass(MethodProperty.class);
        deserializer.registerElementClass(ColorProperty.class);
        deserializer.registerElementClass(Librarian.class);
        deserializer.registerElementClass(Library.class);
        deserializer.registerElementClass(OvalElement.class);
        deserializer.registerElementClass(RectElement.class);
        deserializer.registerElementClass(RectBasedElement.class);
        deserializer.registerElementClass(PointBasedElement.class);
        deserializer.registerElementClass(PolygonElement.class);
        deserializer.registerElementClass(TextElement.class);
        deserializer.registerElementClass(ActiveElement.class);
        deserializer.registerElementClass(PropertyDescription.class);
        deserializer.registerElementClass(PropertyDescriptor.class);
        deserializer.registerElementClass(SelectPropertyDescription.class);
        deserializer.registerElementClass(GuideLine.class);
        deserializer.registerElementClass(GuideLines.class);
        deserializer.registerElementClass(ArrowElement.class);
    }

    public static Document deserializeDocument(File file) throws JDOMException, IOException, DeserializationException, InstantiationException, IllegalAccessException {
        return (Document)deserializer.deserialeze(file);
    }
    
    public static Librarian deserializeLibrarian(File file) throws JDOMException, IOException, DeserializationException, InstantiationException, IllegalAccessException {
        return (Librarian)deserializer.deserialeze(file);
    }
    
    public static Library deserializeLibrary(File file) throws JDOMException, IOException, DeserializationException, InstantiationException, IllegalAccessException {
        return (Library)deserializer.deserialeze(file);
    }
}
