/*
 * Created on Feb 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JComponent;

import diagnose.graphicengine.devices.CanvasDevice;
import diagnose.utils.DPIUtils;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.Properties;
import diagnose.utils.serialization.XmlConstants;

/**
 * Swing component, implementing base features of drawing 
 */
public class GraphicArea extends JComponent {
    
    private static final long serialVersionUID = 7140848427605250640L;
    private Manager manager;
    private CanvasDevice device;

    public GraphicArea() {
        super();
        device = new CanvasDevice(this, null);
        setManager(new Manager(this));
        setDoubleBuffered(true);
    }
    
    public Manager getManager() {
        return manager;
    }
    
    public CanvasDevice getDevice() {
        return device;
    }
    
    public void setManager(Manager manager) {
        this.manager = manager;
    }
    

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        getDevice().setCanvas((Graphics2D)g);
        showGrid((Graphics2D)g);
        getManager().paint();   
    }
    
    protected void showGrid(Graphics2D g) {
        Properties properties = getManager().getDocumentProperties();
        if (!properties.getBooleanValue(XmlConstants.PROP_SHOW_GRID)) return;
        double gridSize = DPIUtils.milimetersToPixels(properties.getDoubleValue(XmlConstants.PROP_GRID_SIZE)) * getManager().getZoom();
        Point padding = DPIUtils.milimetersToPixels(getManager().getPadding());
        g.setColor(Color.GRAY);
        for (double x = padding.x % gridSize; x < this.getWidth(); x += gridSize) {
            for (double y = padding.y % gridSize; y < this.getHeight(); y += gridSize) {
                g.drawLine((int)x, (int)y, (int)x, (int)y);
            }
            
        }
    }        
}
