/*
 * Created on Feb 11, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine;

import java.util.ArrayList;

import diagnose.graphicengine.events.PersistentEvent;
import diagnose.utils.Lang;
import diagnose.utils.Log;

/**
 * Manages undo/redo actions
 */
public class UndoRedoStack {
    
    private int capacity = 50;
    
    private ArrayList undoStack = new ArrayList(50);
    private ArrayList redoStack = new ArrayList(50);
    
    public UndoRedoStack() {
        super();
    }        
    
    public int getCapacity() {
        return capacity;
    }
    
    public void setCapacity(int capacity) {
        this.capacity = capacity;
    }
    
    public void registerEvent(PersistentEvent event) {
        if (undoStack.size() > capacity) undoStack.remove(0);
        undoStack.add(event);
        if (redoStack.size() > 0) redoStack.clear();
    }
    
    public void undo() {
        if (undoStack.size() == 0) {
            Log.warn(Lang.get("log.emptyundostack"));
            return;
        }
        int top = undoStack.size() - 1;
        PersistentEvent evt = (PersistentEvent)undoStack.get(top);
        undoStack.remove(top);
        redoStack.add(evt);
        evt.undo();
    }
    
    public void redo() {
        if (redoStack.size() == 0) {
            Log.warn(Lang.get("log.emptyredostack"));
            return;
        }
        int top = redoStack.size() - 1;
        PersistentEvent evt = (PersistentEvent)redoStack.get(top);
        redoStack.remove(top);
        undoStack.add(evt);
        evt.redo();
    }
    
    public boolean canUndo() {
        return undoStack.size() > 0;
    }
    
    public boolean canRedo() {
        return redoStack.size() > 0;
    }
}
