/*
 * Created on Apr 7, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.contexts;

import java.awt.event.MouseEvent;
import java.util.ArrayList;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.utils.CursorFactory;
import diagnose.utils.DPIUtils;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.Properties;
import diagnose.utils.properties.Property;

/**
 * This context is activated during creation of new elements
 */
public class ElementCreationContext extends EventContext {

    private CommonElement newElement;
    private boolean canShowElement;
    
    public ElementCreationContext(Manager manager, AbstractDevice device, CommonElement newElement) {
        super(manager, device);
        this.newElement = newElement;
        getManager().clearSelectionEvent();
        
        applyDefaultValues(newElement, manager.getDefaultProperties());
        
        newElement.startCreation();
        canShowElement = false;
    }
    
    public void activateContext() {
        getManager().setCursor(CursorFactory.getInsertPointCursor());
    }

    public CommonElement getNewElement() {
        return newElement;
    }
    
    public void mouseClicked(MouseEvent e) {
        
        super.mouseClicked(e);
        
        if ((e.getClickCount() == 2) || (e.getButton() == MouseEvent.BUTTON3)) {
            cancelContext();
            return;
        }
        
        Point target = viewCorrection(DPIUtils.pixelsToMilimeters(e.getPoint()));
        
        Point snapCorrection = getSnappingCorrection(target);
        if (snapCorrection != null)
            target.translate(snapCorrection.x, snapCorrection.y);
        
        int state = newElement.setLastControlPointAsFinal(target);
        if (state == CommonElement.CREATION_FINISHED) {
            getManager().addElement(newElement);
            getManager().setDefaultContext();
        } else getManager().repaint();
        
        
    }
    
    public void mouseMoved(MouseEvent e) {
        super.mouseMoved(e);
        
        Point target = viewCorrection(DPIUtils.pixelsToMilimeters(e.getPoint()));
        
        Point snapCorrection = getSnappingCorrection(target);
        if (snapCorrection != null) target.translate(snapCorrection.x, snapCorrection.y);
        
        newElement.setLastControlPoint(target);
        canShowElement = true;
        getManager().repaint();
    }
    
    public void mouseDragged(MouseEvent e) {
        mouseMoved(e);
    }
    
    public void cancelContext() {

        int state = newElement.cancelCreation();
        if (state == CommonElement.ELEMENT_VIABLE)
            getManager().addElement(newElement);
        getManager().setDefaultContext();
        getManager().repaint();
    }
    
    
    public void paint(AbstractDevice device) {
        super.paint(device);
        if (canShowElement) newElement.paint(device);
    }
    
    /**
     * Copy default values into element properties
     */
    private static void applyDefaultValues(CommonElement element, Properties defaultProperties) {
        ArrayList defaults = defaultProperties.getPropertiesNames();
        for (int i = 0; i < defaults.size(); i++) {
            String defaultProp = (String)defaults.get(i);
            if (element.getProperties().hasProperty(defaultProp)) {
                Property prop = (Property)defaultProperties.getProperty(defaultProp).clone();
                element.getProperties().getProperty(defaultProp).setValue(prop.getValue());
            }
        }
    }

    public String getContextInfo() {
        return Lang.get("context.creatingnewelement");
    }
}