/*
 * Created on Apr 9, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.contexts;

import java.awt.event.MouseEvent;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.utils.CursorFactory;
import diagnose.utils.DPIUtils;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;

/**
 * Padding context which changes padding using mouse
 */
public class PaddingContext extends EventContext {

    private Point selectedAt;
    
    public PaddingContext(Manager manager, AbstractDevice device) {
        super(manager, device);
    }
    
    public void activateContext() {
        getManager().setCursor(CursorFactory.getCursor("hand_cur"));
    }
    
    public void cancelContext() {
        getManager().setDefaultContext();
    }
    
    public void mousePressed(MouseEvent e) {
        super.mousePressed(e);
        selectedAt = DPIUtils.pixelsToMilimeters(new Point(e.getPoint()));
    }
    
    public void mouseDragged(MouseEvent e) {
        super.mouseDragged(e);
        Point p = DPIUtils.pixelsToMilimeters(new Point(e.getPoint()));
        Point padding = getManager().getPadding();
        Point direction = new Point(p.x - selectedAt.x, p.y - selectedAt.y);
        padding.translate(direction);
        getManager().setPadding(padding);
        getManager().invokeViewChange();
        selectedAt.translate(direction);
        getManager().repaint();
    }
    
    public String getContextInfo() {
        return Lang.get("context.padding");
    }
}
