/*
 * Created on Jul 11, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.contexts;

import java.awt.Color;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.utils.DPIUtils;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;

/**
 * Shared function between classes that requires
 * rectangular selection like default selection,
 * zoom, etc.
 */
public abstract class SelectingContext extends EventContext {

    /**
     * Contains area representing selection, if null selection is not supported
     */
    protected Rect selectionArea;
    
    /**
     * @param manager
     * @param device
     */
    public SelectingContext(Manager manager, AbstractDevice device) {
        super(manager, device);
    }
    
    public void paint(AbstractDevice device) {
        super.paint(device);
        if (selectionArea != null) {
            device.setDefaultValues();
            device.setColor(Color.BLACK);
            
            double zoom = getManager().getZoom();
            Point p1 = (Point)selectionArea.getPosition().clone();
            Point p2 = (Point)selectionArea.getOppositePoint().clone();
            p1.multiply(zoom);
            p2.multiply(zoom);
            Rect visibleRect = new Rect(p1, p2);

            device.setLineWidth(DPIUtils.pixelsToMilimeters(1));
            device.setLineStyle(AbstractDevice.DASH_LINE);
            device.rect(visibleRect, AbstractDevice.USE_PADDING);
        }
    }
}
