/*
 * Created on Jun 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.contexts;

import java.awt.event.MouseEvent;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.utils.CursorFactory;
import diagnose.utils.DPIUtils;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;

/**
 * Context which enables zoom to specified
 */
public class ZoomContext extends SelectingContext {
    
    private static final int LEFT_BTN = 0x01;
    private static final int RIGHT_BTN = 0x02;
    
    private Point selectedAt;
    
    private int pressedButton;
    
    /**
     * @param manager
     * @param device
     */
    public ZoomContext(Manager manager, AbstractDevice device) {
        super(manager, device);
    }
    
    public void activateContext() {
        getManager().setCursor(CursorFactory.getCursor("magnif_cur"));
    }
    
    public void mousePressed(MouseEvent e) {
        super.mousePressed(e);
        selectedAt = viewCorrection(DPIUtils.pixelsToMilimeters(e.getPoint()));     
        pressedButton = ((e.getButton() == MouseEvent.BUTTON1) ? LEFT_BTN : RIGHT_BTN); 
    }
    
    public void mouseDragged(MouseEvent e) {
        super.mouseDragged(e);
        Point ml = viewCorrection(DPIUtils.pixelsToMilimeters(e.getPoint()));
        selectionArea = new Rect(selectedAt, ml);
        getManager().repaint();        
    }

    public void mouseReleased(MouseEvent e) {
        
        super.mouseReleased(e);
        
        if (selectionArea == null) return;
        Point p1 = (Point)selectionArea.getPosition().clone();
        
        if (selectionArea == null) {
            double zoomChange = (pressedButton == LEFT_BTN ? 0.1 : -0.1);
            getManager().setZoom(getManager().getZoom() + zoomChange);
        } else {
        
            p1.multiply(-1 * getManager().getZoom());
            getManager().setPadding(p1);
                
            double xZoom = Math.abs(getDevice().getVisibleWidth() / selectionArea.getWidth());
            double yZoom = Math.abs(getDevice().getVisibleHeight() / selectionArea.getHeight());
            getManager().setZoom(Math.min(xZoom, yZoom));
        }
        
        selectedAt = null;
        selectionArea = null;
        
        getManager().invokeViewChange();
        getManager().repaint();
    }
    
    
    public String getContextInfo() {
        return Lang.get("context.zoom");
    }
}