/*
 * Created on Feb 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements;

import java.util.ArrayList;

import org.jdom.Element;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.system.GuideLine;
import diagnose.graphicengine.elements.system.GuideLines;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;
import diagnose.utils.properties.BoolProperty;
import diagnose.utils.properties.DoubleProperty;
import diagnose.utils.properties.PointProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class Document extends ComposedElement {
    
    private GuideLines guideLines = new GuideLines();
    private static PropertyDescriptor defaultPropertyDescriptor = new PropertyDescriptor();
    
    
    static {
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_ZOOM, Lang.get("property.zoom"), Lang.get("category.view"), 10);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_SNAP_TO_GUIDELINES, Lang.get("property.snapguideline"), Lang.get("category.view"), 10);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_SNAP_TO_OTHER_ELEMENT, Lang.get("property.snapelements"), Lang.get("category.view"), 10);
        //defaultPropertyDescriptor.setDescription(XmlConstants.PROP_PADDING, XmlConstants.PROP_PADDING, Lang.get("category.view"), 11);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_SNAP_TO_GRID, Lang.get("property.snaptogrid"), Lang.get("category.grid"), 20);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_GRID_SIZE, Lang.get("property.gridsize"), Lang.get("category.grid"), 21);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_SHOW_GRID, Lang.get("property.showgrid"), Lang.get("category.grid"), 22);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_SNAP_TECH, Lang.get("property.snaptech"), Lang.get("category.view"), 12);
    }
    
    public Document() {
        getProperties().setProperty(new DoubleProperty(XmlConstants.PROP_ZOOM, 1));
        getProperties().setProperty(new PointProperty(XmlConstants.PROP_PADDING, new Point(0, 0)));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SNAP_TO_GRID, true));
        getProperties().setProperty(new DoubleProperty(XmlConstants.PROP_GRID_SIZE, 5));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SHOW_GRID, true));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SHOW_GUIDELINES, true));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SNAP_TO_GUIDELINES, true));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SNAP_TO_OTHER_ELEMENT, true));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_SNAP_TECH, true));
        setPropertyDescriptor(defaultPropertyDescriptor);
    }
    
    /**
     * @return all elements that are positioned in rectangle <code>r</code> 
     */
    public ArrayList getElementsInRect(Rect r) {
        ArrayList result = new ArrayList();
        for (int i = 0; i < elements.size(); i++) {
            VisualElement elt = (VisualElement)elements.get(i);
            if ((elt.getRectBounds() != null) && 
                    (r.contains(elt.getRectBounds()))) result.add(elt);
        }
        
        return result;   
    }
    
    public void paint(AbstractDevice device) {
        super.paint(device);
        if (getProperties().getBooleanValue(XmlConstants.PROP_SHOW_GUIDELINES)) guideLines.paint(device);
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_DOCUMENT;
    }
    
    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        VisualElement result = super.receivesMouseEvent(p, device);
        if (result != null) return result; 
        return guideLines.receivesMouseEvent(p, device);        
    }

    public void addGuideLine(GuideLine guideLine) {
        guideLines.add(guideLine);
    }
    
    public void removeGuideLine(GuideLine guideLine) {
        guideLines.remove(guideLine);
    }
    
    public boolean hasGuideLine(GuideLine guideLine) {
        return guideLines.hasGuideLine(guideLine);
    }

    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        super.deserialize(elt, deserializer);
        guideLines = (GuideLines)deserializer.deserialize(elt.getChild(XmlConstants.ELEMENT_GUIDE_LINES));
    }

    public Element serialize() {
        Element result = super.serialize();
        result.addContent(guideLines.serialize());
        return result;
    }
    
    public GuideLines getGuideLines() {
        return guideLines;
    }
    
}