/*
 * Created on May 1, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements;

import java.util.ArrayList;

import org.jdom.Element;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.libraries.LibElementIdentifier;
import diagnose.graphicengine.relation.Relations;
import diagnose.graphicengine.relation.SubmissiveElement;
import diagnose.utils.Lang;
import diagnose.utils.Log;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;
import diagnose.utils.properties.MethodProperty;
import diagnose.utils.properties.Properties;
import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class LibraryElement extends CommonElement
                            implements SubmissiveElement {

    private CommonElement innerObject;
    
    public LibraryElement() {
        super();
    }
    
    public LibraryElement(CommonElement innerObject) {
        super();
        this.innerObject = innerObject;
        this.relations = innerObject.getRelations();
        setProperties(innerObject.getProperties());
    }
    
    public CommonElement getInnerObject() {
        return innerObject;
    }
    
    public void setInnerObject(CommonElement innerObject) {
        this.innerObject = innerObject;
        this.relations = innerObject.getRelations();
        setProperties(innerObject.getProperties());
    }
    
    public void startCreation() {
        if (innerObject != null) innerObject.startCreation();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#fixateLastControlPoint(diagnose.utils.geometry.Point)
     */
    public int fixateLastControlPoint(Point p) {
        return (innerObject == null ? -1 : innerObject.fixateLastControlPoint(p));
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#cancelCreation()
     */
    public int cancelCreation() {
        return (innerObject == null ? -1 : innerObject.cancelCreation());
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#getControlPointElements()
     */
    public ArrayList getControlPointElements() {
        return (innerObject == null ? null : innerObject.getControlPointElements());
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#getPropertryDescriptor()
     */
    public PropertyDescriptor getPropertryDescriptor() {
        if (innerObject != null) return innerObject.getPropertryDescriptor();
        return null;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#paint(diagnose.graphicengine.AbstractDevice)
     */
    public void paint(AbstractDevice device) {
        if (innerObject != null) innerObject.paint(device);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#receivesMouseEvent(diagnose.utils.geometry.Point, diagnose.graphicengine.AbstractDevice)
     */
    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        return (innerObject == null ? null : innerObject.receivesMouseEvent(p, device));
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#move(diagnose.utils.geometry.Point)
     */
    public void move(Point direction) {
        if (innerObject != null) innerObject.move(direction);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#scale(diagnose.utils.geometry.Point, diagnose.utils.geometry.Point)
     */
    public void scale(Point center, Point coef) {
        if (innerObject != null) innerObject.scale(center, coef);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#snaps(diagnose.utils.geometry.Point)
     */
    public Point snaps(Point p) {
        return (innerObject == null ? null : innerObject.snaps(p));
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#getSnapablePoints()
     */
    public ArrayList getSnapablePoints() {
        return (innerObject == null ? null : innerObject.getSnapablePoints());
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#getRectBounds()
     */
    public Rect getRectBounds() {
        return (innerObject == null ? null : innerObject.getRectBounds());
    }

    /* (non-Javadoc)
     * @see diagnose.utils.serialization.XMLSeriazable#getElementType()
     */
    public String getElementType() {
        return XmlConstants.ELEMENT_LIBRARY_ELEMENT;
    }
    
    public void setLastControlPoint(Point p) {
        if (innerObject != null) innerObject.setLastControlPoint(p);
    }
    
    public int setLastControlPointAsFinal(Point p) {
        return (innerObject == null ? -1 : innerObject.setLastControlPointAsFinal(p));
    }
    
    public Object clone() {
        LibraryElement result;
        try {
            result = (LibraryElement)this.getClass().newInstance();
        } catch (Exception e) {
            e.printStackTrace();
            Log.error(Lang.get("log.cloneerror"));
            return null;
        }
        result.setProperties((Properties)this.getProperties().clone());
        if (innerObject != null) {
	        result.innerObject = (CommonElement)innerObject.clone();
	        result.innerObject.setProperties(result.getProperties());
        }
        return result;
    }
    
    
    public Element serialize() {
        // temporaly removes methods
        Properties withoutMethods = (Properties)getProperties().clone();
        ArrayList names = withoutMethods.getPropertiesNames();
        for (int i = 0; i < names.size(); i++) {
            Property p = withoutMethods.getProperty((String)names.get(i));
            if (p instanceof MethodProperty) withoutMethods.removeProperty((String)names.get(i));
        }
        
        Properties oldProperties = getProperties();
        setProperties(withoutMethods);
        Element result = super.serialize();
        setProperties(oldProperties);
        return result;
    }

    public LibElementIdentifier getIdentifier() {
        return new LibElementIdentifier(
                properties.getStringValue(XmlConstants.PROP_LIBRARY),
                properties.getStringValue(XmlConstants.PROP_NAME));
    }

    protected void suggestRelation(Relations relations, SubmissiveElement submissiveElt, int id, Point position) {
        //this.relations = innerObject.getRelations();
        if (innerObject != null) {
            innerObject.suggestRelation(relations, submissiveElt, id, position);
        }
    }

    public void changeSubmissive(int id, Point newPosition) {
        if ((innerObject != null) && (innerObject instanceof SubmissiveElement))
            ((SubmissiveElement)innerObject).changeSubmissive(id, newPosition);
    }

    public void askDominant(CommonElement elt) {
        if ((innerObject != null) && (innerObject instanceof SubmissiveElement))
            ((SubmissiveElement)innerObject).askDominant(elt);
    }
    
    public void setEnabledRelation(boolean enabledRelation) {
        super.setEnabledRelation(enabledRelation);
        if (innerObject != null) innerObject.setEnabledRelation(enabledRelation);
    }
}
