/*
 * Created on Feb 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements;

import java.util.ArrayList;

import diagnose.graphicengine.elements.system.NormalControlPointElement;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Geometry;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class PointBasedElement extends ActiveElement {
    
    public PointBasedElement() {
        super();
    }
    
    public ArrayList getSnapablePoints() {
        
        if (canRunFunction(XmlConstants.METHOD_GET_SNAPABLE_POINTS)) return super.getSnapablePoints();
        
        ArrayList result = new ArrayList();
        for (int i = 0; i < this.getControlPointCount(); i++)
            result.add(this.getControlPoint(i));
        
        for (int i = 0; i < this.getControlPointCount(); i++) {
            Point p1 = getControlPoint(i);
            Point p2 = getControlPoint((i + 1) % getControlPointCount());
            result.add(new Point((p1.x + p2.x) / 2, (p1.y + p2.y) / 2));
        }
        
        return result;
    }
    
    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#move(diagnose.graphicengine.utils.Point)
     */
    public void move(Point direction) {
        
        if (canRunFunction(XmlConstants.METHOD_MOVE)) {
            super.move(direction);
            return;
        }
        
        int cnt = getControlPointCount();
        for (int i = 0; i < cnt; i++)
            getControlPoint(i).translate(direction.x, direction.y);
    }
    
    
    
    public void startCreation() {
        // removes all default control points
        for (int i = 0; i < getControlPointCount(); i++) {
            getProperties().removeProperty(XmlConstants.PROP_CP + i);
        }
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_CP_COUNT, 0));
        super.startCreation();
        addControlPoint(new Point(0, 0));
    }

    public ArrayList getControlPointElements() {
        
        if (canRunFunction(XmlConstants.METHOD_GET_CONTROL_POINT_ELEMENTS)) return super.getControlPointElements();
        
        ArrayList result = new ArrayList();
        int cnt = getControlPointCount();
        for (int i = 0; i < cnt; i++)
            result.add(new NormalControlPointElement(this, i));
        return result;
    }
    
    public void scale(Point center, Point coef) {
        
        if (canRunFunction(XmlConstants.METHOD_SCALE)) {
            super.scale(center, coef);
            return;
        }
        
        int cnt = getControlPointCount();
        for (int i = 0; i < cnt; i++) {
            Point p = (Point)getControlPoint(i).clone();
            Point q = Geometry.scale(p, center, coef);
            getControlPoint(i).x = q.x;
            getControlPoint(i).y = q.y;
        }
        
        if (isEnabledRelation()) getRelations().scale(center, coef);
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_POINT_BASED;
    }
    
    /**
     * Fills specified property descriptor with defualt description
     * for control points
     */
    protected void fillProperyDescriptorCp(PropertyDescriptor desc) {
        for (int i = 0; i < getControlPointCount(); i++) {
            desc.setDescription(XmlConstants.PROP_CP + i,
                    Lang.get("property.cp", Integer.toString(i + 1)),
                    Lang.get("category.position"), 10 + i);
        }
    }
}
