/*
 * Created on Feb 10, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements;

import java.util.ArrayList;

import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;

/**
 * Interface for all visual elements including system and other elements
 */
public interface VisualElement extends GraphicElement, Cloneable {
    
    /**
     * Moves object from position in direction specified 
     * by vector <code>direction</code> 
     */
    public void move(Point direction);
    
    
    /**
     * Scales elemen
     * @param coef, represents scaling factor in x, y axis
     */
    public void scale(Point center, Point coef);
    
    /**
     * @return point where point <code>p</code> can be snapped,
     * in case element doesn't support snapping returns null
     */
    public Point snaps(Point p);
    
    /**
     * @return ArrayList of points which are snapable to grid, guide lines, etc.
     */
    public ArrayList getSnapablePoints();
    
    /**
     * @return rectangle which bounds complete element, or null in case element 
     * representation has less then 2 dimensions (e.g. point)
     */
    public Rect getRectBounds();
}
