/*
 * Created on Aug 23, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.implementations;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.utils.DefaultArrays;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.properties.SelectPropertyDescription;
import diagnose.utils.serialization.XmlConstants;


/**
 *
 */
public class ArrowElement extends LineElement {
    
    public static final int NONE = 0;
    public static final int SIMPLE = 1;
    public static final int EMPTY = 2;
    public static final int FILL = 3;
    
    private static double HEAD_LENGTH= 3;

    private static PropertyDescriptor defaultPropertyDescriptor = new PropertyDescriptor();
    static {
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_CP_0, Lang.get("property.firstpoint"), Lang.get("category.position"), 10);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_CP_1, Lang.get("property.secondpoint"), Lang.get("category.position"), 11);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_COLOR, Lang.get("property.color"), Lang.get("category.style"), 20);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_LINE_WIDTH, Lang.get("property.linewidth"), Lang.get("category.style"), 21);
        
        SelectPropertyDescription lineStyleDesc = new SelectPropertyDescription(XmlConstants.PROP_LINE_STYLE, Lang.get("property.linestyle"), Lang.get("category.style"), 22);
        DefaultArrays.fillLineStyles(lineStyleDesc);
        defaultPropertyDescriptor.setDescription(lineStyleDesc);
        
        SelectPropertyDescription end1Desc = new SelectPropertyDescription(XmlConstants.PROP_END1, Lang.get("property.end1"), Lang.get("category.style"), 23);
        end1Desc.addItem(Integer.toString(NONE), "@/resources/arrownone.png");
        end1Desc.addItem(Integer.toString(SIMPLE), "@/resources/arrowsimpleleft.png");
        end1Desc.addItem(Integer.toString(EMPTY), "@/resources/arrowemptyleft.png");
        end1Desc.addItem(Integer.toString(FILL), "@/resources/arrowfillleft.png");
        defaultPropertyDescriptor.setDescription(end1Desc);
        
        SelectPropertyDescription end2Desc = new SelectPropertyDescription(XmlConstants.PROP_END2, Lang.get("property.end2"), Lang.get("category.style"), 24);
        end2Desc.addItem(Integer.toString(NONE), "@/resources/arrownone.png");
        end2Desc.addItem(Integer.toString(SIMPLE), "@/resources/arrowsimpleright.png");
        end2Desc.addItem(Integer.toString(EMPTY), "@/resources/arrowemptyright.png");
        end2Desc.addItem(Integer.toString(FILL), "@/resources/arrowfillright.png");
        defaultPropertyDescriptor.setDescription(end2Desc);
    }
    
    public ArrowElement() {
        super();
        
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_END1, NONE));
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_END2, SIMPLE));
        setPropertyDescriptor(defaultPropertyDescriptor);
    }
    
    public void paint(AbstractDevice device) {
        if (canRunFunction(XmlConstants.METHOD_PAINT)) {
            super.paint(device);
        } else {
            device.setDefaultValues();
            if (getControlPointCount() < 2) return;
            device.setColor(getProperties().getColorValue(XmlConstants.PROP_COLOR));
            
            
            int head1 = getProperties().getIntValue(XmlConstants.PROP_END1);
            int head2 = getProperties().getIntValue(XmlConstants.PROP_END2);
            
            Point lineEnd1 = getControlPoint(0);
            Point lineEnd2 = getControlPoint(1);
            
            if (head2 != NONE) {
                Point p1 = getControlPoint(0);
                Point p2 = getControlPoint(1);
                double length = p1.distance(p2);
                double alfa = 0;
                try {
                    if (p2.y > p1.y) alfa = (p2.y - p1.y) / length;
                    if (p2.y < p1.y) alfa = -(p1.y - p2.y) / length;
                    alfa = Math.asin(alfa);
                    if (p2.x < p1.x) alfa = Math.PI - alfa;
                    alfa += Math.PI;
                    double alfa3 = alfa - Math.PI / 6;
                    double alfa4 = alfa + Math.PI / 6;
                    Point p3 = new Point(Math.cos(alfa3) * HEAD_LENGTH, Math.sin(alfa3) * HEAD_LENGTH);
                    Point p4 = new Point(Math.cos(alfa4) * HEAD_LENGTH, Math.sin(alfa4) * HEAD_LENGTH);
                    p3.translate(p2);
                    p4.translate(p2);
                    if (head2 == SIMPLE) {
                        device.line(p3, p2, AbstractDevice.DEFAULT_DRAW);
                        device.line(p4, p2, AbstractDevice.DEFAULT_DRAW);
                    }
                    if ((head2 == FILL) || (head2 == EMPTY)) {
                        double [] x = { p2.x,  p3.x , p4.x };
                        double [] y = { p2.y,  p3.y , p4.y };
                        
                        if (head2 == FILL) device.fillPolygon(x, y, AbstractDevice.DEFAULT_DRAW);
                        else {
                            device.polygon(x, y, AbstractDevice.DEFAULT_DRAW);
                            double head_length2 = HEAD_LENGTH * Math.cos(Math.PI / 6);
                            Point p5 = new Point(Math.cos(alfa) * head_length2, Math.sin(alfa) * head_length2);
                            p5.translate(p2);
                            lineEnd2 = p5;
                        }
                        
                    }
                } catch (Exception e) {
                }
            }
            
            
            if (head1 != NONE) {
                Point p1 = getControlPoint(0);
                Point p2 = getControlPoint(1);
                double length = p1.distance(p2);
                double alfa = 0;
                try {
                    if (p1.y > p2.y) alfa = (p1.y - p2.y) / length;
                    if (p1.y < p2.y) alfa = -(p2.y - p1.y) / length;
                    alfa = Math.asin(alfa);
                    if (p1.x < p2.x) alfa = Math.PI - alfa;
                    alfa += Math.PI;
                    double alfa3 = alfa - Math.PI / 6;
                    double alfa4 = alfa + Math.PI / 6;
                    Point p3 = new Point(Math.cos(alfa3) * HEAD_LENGTH, Math.sin(alfa3) * HEAD_LENGTH);
                    Point p4 = new Point(Math.cos(alfa4) * HEAD_LENGTH, Math.sin(alfa4) * HEAD_LENGTH);
                    p3.translate(p1);
                    p4.translate(p1);
                    if (head1 == SIMPLE) {
                        device.line(p3, p1, AbstractDevice.DEFAULT_DRAW);
                        device.line(p4, p1, AbstractDevice.DEFAULT_DRAW);
                    }
                    if ((head1 == FILL) || (head1 == EMPTY)) {
                        double [] x = { p1.x,  p3.x , p4.x };
                        double [] y = { p1.y,  p3.y , p4.y };
                        
                        if (head1 == FILL) device.fillPolygon(x, y, AbstractDevice.DEFAULT_DRAW);
                        else {
                            device.polygon(x, y, AbstractDevice.DEFAULT_DRAW);
                            double head_length2 = HEAD_LENGTH * Math.cos(Math.PI / 6);
                            Point p5 = new Point(Math.cos(alfa) * head_length2, Math.sin(alfa) * head_length2);
                            p5.translate(p1);
                            lineEnd1 = p5;
                        }
                        
                    }
                } catch (Exception e) {
                }
            }
            
            device.setLineStyle(getProperties().getIntValue(XmlConstants.PROP_LINE_STYLE));
            device.setLineWidth(getProperties().getDoubleValue(XmlConstants.PROP_LINE_WIDTH));
            device.line(lineEnd1, lineEnd2, AbstractDevice.DEFAULT_DRAW);
        }
    }
    
    public String getElementType() {
        return XmlConstants.ELEMENT_ARROW;
    }

}
