/*
 * Created on Feb 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.implementations;

import java.awt.Color;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.PointBasedElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.graphicengine.relation.Relations;
import diagnose.graphicengine.relation.SubmissiveElement;
import diagnose.utils.DefaultArrays;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Abscissa;
import diagnose.utils.geometry.Geometry;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;
import diagnose.utils.properties.ColorProperty;
import diagnose.utils.properties.DoubleProperty;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.properties.SelectPropertyDescription;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class LineElement extends PointBasedElement {
    
    private static PropertyDescriptor defaultPropertyDescriptor = new PropertyDescriptor();
    static {
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_CP_0, Lang.get("property.firstpoint"), Lang.get("category.position"), 10);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_CP_1, Lang.get("property.secondpoint"), Lang.get("category.position"), 11);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_COLOR, Lang.get("property.color"), Lang.get("category.style"), 20);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_LINE_WIDTH, Lang.get("property.linewidth"), Lang.get("category.style"), 21);
        
        SelectPropertyDescription lineStyleDesc = new SelectPropertyDescription(XmlConstants.PROP_LINE_STYLE, Lang.get("property.linestyle"), Lang.get("category.style"), 22);
        DefaultArrays.fillLineStyles(lineStyleDesc);
        defaultPropertyDescriptor.setDescription(lineStyleDesc);
    }
    
    public LineElement() {
        super();
        getProperties().setProperty(new DoubleProperty(XmlConstants.PROP_LINE_WIDTH, 0.35));
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_LINE_STYLE, AbstractDevice.CLEAR_LINE));
        getProperties().setProperty(new ColorProperty(XmlConstants.PROP_COLOR, Color.BLACK));
        setPropertyDescriptor(defaultPropertyDescriptor);
    }
    
    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#fixateLastControlPoint(diagnose.utils.geometry.Point)
     */
    public int fixateLastControlPoint(Point p) {
        if (canRunFunction(XmlConstants.METHOD_FIXATE_LAST_CONTROL_POINT)) return super.fixateLastControlPoint(p);
        
        setLastControlPoint(p);
        if (getControlPointCount() == 2) return CREATION_FINISHED;
        return CREATION_CONTINUES;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#cancelCreation()
     */
    public int cancelCreation() {
        if (canRunFunction(XmlConstants.METHOD_CANCEL_CREATION)) return super.cancelCreation();
        return ELEMENT_NOT_VIABLE;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#paint()
     */
    public void paint(AbstractDevice device) {
        if (canRunFunction(XmlConstants.METHOD_PAINT)) {
            super.paint(device);
        } else {
            device.setDefaultValues();
            if (getControlPointCount() < 2) return;
            device.setColor(getProperties().getColorValue(XmlConstants.PROP_COLOR));
            device.setLineWidth(getProperties().getDoubleValue(XmlConstants.PROP_LINE_WIDTH));
            device.setLineStyle(getProperties().getIntValue(XmlConstants.PROP_LINE_STYLE));
            device.line(getControlPoint(0), getControlPoint(1), AbstractDevice.DEFAULT_DRAW);
        }
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#receivesMouseEvent(diagnose.graphicengine.utils.Point)
     */
    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        if (canRunFunction(XmlConstants.METHOD_RECEIVE_MOUSE_EVENT)) return super.receivesMouseEvent(p, device);
        return (Geometry.isOnLine(getControlPoint(0), getControlPoint(1), p, Manager.PRECISSNESS / device.getZoom()) ? this : null);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#snaps(diagnose.graphicengine.utils.Point, int)
     */
    public Point snaps(Point p) {
        if (canRunFunction(XmlConstants.METHOD_SNAPS)) return super.snaps(p);
        Abscissa line = new Abscissa(getControlPoint(0), getControlPoint(1));
        return line.getNearestPoint(p);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#getShapeRect()
     */
    public Rect getRectBounds() {
        if (canRunFunction(XmlConstants.METHOD_GET_RECT_BOUNDS)) return super.getRectBounds();
        
        if (getControlPointCount() > 1) {
            double lineWidthTolerance = getProperties().getDoubleValue(XmlConstants.PROP_LINE_WIDTH)  / 2;
            Point p1 = (Point)getControlPoint(0).clone();
            Point p2 = (Point)getControlPoint(1).clone();
            p1.translate(-lineWidthTolerance, -lineWidthTolerance);
            p2.translate(lineWidthTolerance, lineWidthTolerance);
            return new Rect(p1, p2);
        }
        return null;
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_LINE;
    }
    
    public void askDominant(CommonElement elt) {
        if (canRunFunction(XmlConstants.METHOD_ASK_DOMINANT)) {
            super.askDominant(elt);
            return;
        }
        
        for (int i = 0; i  < getControlPointCount(); i++) {
            if (elt != this) elt.suggestRelation(this, i, getControlPoint(i));
        }
    }

    public void changeSubmissive(int id, Point newPosition) {
        if (canRunFunction(XmlConstants.METHOD_CHANGE_SUBMISSIVE)) {
            super.changeSubmissive(id, newPosition);
            return;
        }
        setControlPoint(id, newPosition);
    }    
    
    public void suggestRelation(Relations relations, SubmissiveElement submissiveElt, int id, Point position) {
        if (canRunFunction(XmlConstants.METHOD_SUGGEST_RELATION)) super.suggestRelation(relations, submissiveElt, id, position);
        /*
        else {
            Point p1 = getControlPoint(0);
            Point p2 = getControlPoint(1);
            Point p = new Point((p1.x + p2.x) / 2, (p1.y + p2.y) / 2);
            if (p.distance(position) < SNAP_TOLERANCE) {
                relations.add(submissiveElt, id, position);
            }
        }
        */
    }
}
