/*
 * Created on Jun 3, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.implementations;

import java.awt.Color;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.RectBasedElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.graphicengine.relation.Relations;
import diagnose.graphicengine.relation.SubmissiveElement;
import diagnose.utils.DefaultArrays;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;
import diagnose.utils.properties.BoolProperty;
import diagnose.utils.properties.ColorProperty;
import diagnose.utils.properties.DoubleProperty;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.properties.SelectPropertyDescription;
import diagnose.utils.properties.StringProperty;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class OvalElement extends RectBasedElement {

    private static PropertyDescriptor defaultPropertyDescriptor = new PropertyDescriptor();
    static {
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_POSITION, Lang.get("property.position"), Lang.get("category.position"), 10);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_WIDTH, Lang.get("property.width"), Lang.get("category.position"), 11);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_HEIGHT, Lang.get("property.height"), Lang.get("category.position"), 12);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_COLOR, Lang.get("property.color"), Lang.get("category.style"), 20);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_TRANSPARENT, Lang.get("property.transparent"), Lang.get("category.style"), 21);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_BACKGROUND, Lang.get("property.background"), Lang.get("category.style"), 22);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_LINE_WIDTH, Lang.get("property.linewidth"), Lang.get("category.style"), 23);
        
        SelectPropertyDescription lineStyleDesc = new SelectPropertyDescription(XmlConstants.PROP_LINE_STYLE, Lang.get("property.linestyle"), Lang.get("category.style"), 24);
        DefaultArrays.fillLineStyles(lineStyleDesc);
        
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_TEXT, Lang.get("property.text"), Lang.get("category.text"), 15);
        
        
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_TEXTCOLOR, Lang.get("property.color"), Lang.get("category.textstyle"), 19);
        SelectPropertyDescription fontDesc = new SelectPropertyDescription(XmlConstants.PROP_FONT_FACE, Lang.get("property.font.face"), Lang.get("category.textstyle"), 21);
        DefaultArrays.fillFontNames(fontDesc);

        defaultPropertyDescriptor.setDescription(fontDesc);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_FONT_SIZE, Lang.get("property.font.size"), Lang.get("category.textstyle"), 32);

        SelectPropertyDescription alignDesc = new SelectPropertyDescription(XmlConstants.PROP_ALIGN, Lang.get("property.align"), Lang.get("category.textstyle"), 33);
        alignDesc.addItem(Integer.toString(AbstractDevice.ALIGN_LEFT), Lang.get("property.align.left"));
        alignDesc.addItem(Integer.toString(AbstractDevice.ALIGN_CENTER), Lang.get("property.align.center"));
        alignDesc.addItem(Integer.toString(AbstractDevice.ALIGN_RIGHT), Lang.get("property.align.right"));
        defaultPropertyDescriptor.setDescription(alignDesc);
        
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_BOLD, Lang.get("property.font.bold"), Lang.get("category.textstyle"), 35);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_ITALIC, Lang.get("property.font.italic"), Lang.get("category.textstyle"), 36);
        defaultPropertyDescriptor.setDescription(XmlConstants.PROP_UNDERLINE, Lang.get("property.font.underline"), Lang.get("category.textstyle"), 37);

        defaultPropertyDescriptor.setDescription(lineStyleDesc);
    }
    
    public OvalElement() {
        super();
        getProperties().setProperty(new ColorProperty(XmlConstants.PROP_BACKGROUND, Color.WHITE));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_TRANSPARENT, false));
        getProperties().setProperty(new DoubleProperty(XmlConstants.PROP_LINE_WIDTH, 0.35));
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_LINE_STYLE, AbstractDevice.CLEAR_LINE));
        getProperties().setProperty(new StringProperty(XmlConstants.PROP_TEXT, ""));
        getProperties().setProperty(new StringProperty(XmlConstants.PROP_FONT_FACE, Lang.get("application.defaultfont")));
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_ALIGN, AbstractDevice.ALIGN_LEFT));
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_FONT_SIZE, 10));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_BOLD, false));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_ITALIC, false));
        getProperties().setProperty(new BoolProperty(XmlConstants.PROP_UNDERLINE, false));
        getProperties().setProperty(new ColorProperty(XmlConstants.PROP_COLOR, Color.BLACK));
        getProperties().setProperty(new ColorProperty(XmlConstants.PROP_TEXTCOLOR, Color.BLACK));
        setPropertyDescriptor(defaultPropertyDescriptor);
        
    }

    public void paint(AbstractDevice device) {
        if (canRunFunction(XmlConstants.METHOD_PAINT)) {
            super.paint(device);
        } else {
            if (getControlPointCount() < 2) return;
            device.setDefaultValues();
            if (!getProperties().getBooleanValue(XmlConstants.PROP_TRANSPARENT)) {
                device.setColor(getProperties().getColorValue(XmlConstants.PROP_BACKGROUND));
                device.fillOval(getRectBounds(), AbstractDevice.DEFAULT_DRAW);    
            }
            
            device.setLineWidth(getProperties().getDoubleValue(XmlConstants.PROP_LINE_WIDTH));
            device.setColor(getProperties().getColorValue(XmlConstants.PROP_COLOR));
            device.setLineStyle(getProperties().getIntValue(XmlConstants.PROP_LINE_STYLE));
            device.oval(getRectBounds(), AbstractDevice.DEFAULT_DRAW);
            
            if (getProperties().hasProperty(XmlConstants.PROP_TEXT) 
                    && (getProperties().getStringValue(XmlConstants.PROP_TEXT).length() > 0)) {

                device.setColor(getProperties().getColorValue(XmlConstants.PROP_TEXTCOLOR));
                device.setFontSize(getProperties().getIntValue(XmlConstants.PROP_FONT_SIZE));
                device.setFontFamily(getProperties().getStringValue(XmlConstants.PROP_FONT_FACE));
                
                int style = AbstractDevice.FONT_NORMAL;
                if (getProperties().getBooleanValue(XmlConstants.PROP_BOLD)) style |= AbstractDevice.FONT_BOLD;
                if (getProperties().getBooleanValue(XmlConstants.PROP_ITALIC)) style |= AbstractDevice.FONT_ITALIC;
                if (getProperties().getBooleanValue(XmlConstants.PROP_UNDERLINE)) style |= AbstractDevice.FONT_UNDERLINE;
                
                device.setFontStyle(style);
                
                device.setTextAlign(getProperties().getIntValue(XmlConstants.PROP_ALIGN));
                Rect txtRect = getRectBounds();
                double oWidth = txtRect.getWidth();
                double oHeight = txtRect.getHeight();
                Point correction = new Point(txtRect.getWidth() * 0.15, txtRect.getHeight() * 0.15);
                txtRect.getPosition().translate(correction);
                txtRect.setWidth(oWidth - 2 * correction.x);
                txtRect.setHeight(oHeight - 2 * correction.y);
                device.text(getProperties().getStringValue(XmlConstants.PROP_TEXT), txtRect, AbstractDevice.DEFAULT_DRAW);
            }
        }
    }

    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        if (canRunFunction(XmlConstants.METHOD_RECEIVE_MOUSE_EVENT)) return super.receivesMouseEvent(p, device);
        Ellipse2D.Double ellipse = new Ellipse2D.Double(
                getPosition().x, getPosition().y, getWidth(), getHeight());
        return (ellipse.contains(p.x, p.y) ? this : null);
    }

    public Point snaps(Point p) {
        if (canRunFunction(XmlConstants.METHOD_SNAPS)) return super.snaps(p);
        ArrayList points = getEdgePoints();
        Point bestPoint = null;
        double bestDist = Double.MAX_VALUE;
        for (int i = 0; i < points.size(); i++) {
            Point actPoint = (Point)points.get(i);
            double actDist = actPoint.distance(p);
            if (actDist < bestDist) {
                bestDist = actDist;
                bestPoint = actPoint;
            }
        }
        return bestPoint;
    }

    public ArrayList getSnapablePoints() {
        
        if (canRunFunction(XmlConstants.METHOD_GET_SNAPABLE_POINTS)) return super.getSnapablePoints();
        
        ArrayList result = new ArrayList();
        result.addAll(getCornerPoints());
        result.addAll(getEdgePoints());
        return result;
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_ELLIPSE;
    }
    
    public void suggestRelation(Relations relations, SubmissiveElement submissiveElt, int id, Point position) {
        if (canRunFunction(XmlConstants.METHOD_SUGGEST_RELATION)) super.suggestRelation(relations, submissiveElt, id, position);
        else acceptSuggestionOnSnapPoints(relations, submissiveElt, id, position);
    }   
}
