/*
 * Created on Feb 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.system;

import java.awt.Color;
import java.util.ArrayList;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.utils.DPIUtils;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;

/**
 *
 */
public abstract class AbstractControlPointElement implements VisualElement {
    
    /**
     * Size of control point box
     */
    private static final double CP_SIZE = 1.25;
    
    /**
     * Holds information about objects, which is controled by this control point
     */
    private CommonElement controls; 

    public AbstractControlPointElement() {
        super();
    }
    
    public AbstractControlPointElement(CommonElement controls) {
        super();
        this.controls = controls;
    }
    
    public void paint(AbstractDevice device) {
        Point p = getControlPointPosition();
        double zoom = device.getZoom();
        
        Rect box = new Rect(
                (p.x * zoom - CP_SIZE), 
                (p.y * zoom - CP_SIZE), 
                CP_SIZE * 2, CP_SIZE * 2);
        
        device.setDefaultValues();
        device.setLineWidth(DPIUtils.pixelsToMilimeters(1));
        device.setLineStyle(AbstractDevice.CLEAR_LINE);
        
        device.setColor(Color.GREEN);
        device.fillRect(box, AbstractDevice.USE_PADDING);
        device.setColor(Color.BLACK);
        device.rect(box, AbstractDevice.USE_PADDING);
    }
    
    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        return receivesMouseEventForPoint(p, getControlPointPosition(), device);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#receivesMouseEvent(diagnose.graphicengine.utils.Point)
     */
    protected VisualElement receivesMouseEventForPoint(Point p, Point px, AbstractDevice device) {
        double zoomCorection = 1 / device.getZoom();
        if ((p.x > px.x - CP_SIZE * zoomCorection) && 
            (p.x < px.x + CP_SIZE * zoomCorection) &&   
            (p.y > px.y - CP_SIZE * zoomCorection) &&
            (p.y < px.y + CP_SIZE * zoomCorection)) return this;
        return null;
    }
    
    public CommonElement getControls() {
        return controls;
    }

    public void setControls(CommonElement controls) {
        this.controls = controls;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#snaps(diagnose.graphicengine.utils.Point, int)
     */
    public Point snaps(Point p) {
        return null;
    }
    
    
    public void scale(Point center, Point coef) {
        // ignore scaling
    }
    
    public Rect getRectBounds() {
        return null;
    }        
    
    public ArrayList getSnapablePoints() {
        ArrayList al = new ArrayList(); 
        al.add(getControlPointPosition());
        return al;
    }
    
    public abstract Point getControlPointPosition();
}
