/*
 * Created on Feb 7, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.system;

import java.util.ArrayList;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.GraphicElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.utils.geometry.Point;


/**
 * Holds all control points - only one instance of this sholud be present in application
 */
public final class ControlPoints implements GraphicElement {
    
    private ArrayList elements = new ArrayList();
    
    /**
     * Adds into list of all control points, all control points of specified element 
     */
    public void addControlPointsOfElement(CommonElement elt) {
        ArrayList elts = elt.getControlPointElements(); 
        if (elts != null) elements.addAll(elts);
    }
    
    /**
     * Removes from list of control points, controls point of a specified element
     */
    public void removeControlPointsOfElement(CommonElement elt) {
        for (int i = 0; i < elements.size(); i++) {
            AbstractControlPointElement cpelt = (AbstractControlPointElement)elements.get(i);
            if (cpelt.getControls() == elt) {
                elements.remove(cpelt);
                i--;
            }
        }
    }    
    
    /**
     * Removes all control points
     */
    public void clear() {
        elements.clear();
    }

    public void paint(AbstractDevice device) {
        for (int i = 0; i < elements.size(); i++)
            ((AbstractControlPointElement)elements.get(i)).paint(device);
    }

    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        for (int i = elements.size() - 1; i >= 0; i--) {
            VisualElement elt = (VisualElement)elements.get(i); 
            if (elt.receivesMouseEvent(p, device) != null) return elt;
        }
        return null;
    }
}
