/*
 * Created on Jul 9, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.system;

import java.awt.Color;
import java.util.ArrayList;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.graphicengine.relation.Relations;
import diagnose.graphicengine.relation.SubmissiveElement;
import diagnose.utils.DPIUtils;
import diagnose.utils.geometry.Geometry;
import diagnose.utils.geometry.Line;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;
import diagnose.utils.properties.IntProperty;
import diagnose.utils.properties.PointProperty;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class GuideLine extends CommonElement {

    public static final int VERTICAL    = 0x01;
    public static final int HORIZONTAL  = 0x02;
    
    private static PropertyDescriptor propertyDescriptor = new PropertyDescriptor();
    
    static {
    }
    
    public GuideLine() {
        super();
    }
    
    public GuideLine(int orientation, Point position) {
        super();
        getProperties().setProperty(new IntProperty(XmlConstants.PROP_ORIENTATION, orientation));
        getProperties().setProperty(new PointProperty(XmlConstants.PROP_CP_0, position));
        setControlPointCount(1);
    }

    public int fixateLastControlPoint(Point p) {
        setControlPoint(0, p);
        return CREATION_FINISHED;
    }

    public int cancelCreation() {
        return ELEMENT_VIABLE;
    }

    public ArrayList getControlPointElements() {
        return null;
    }

    public PropertyDescriptor getPropertryDescriptor() {
        return propertyDescriptor;
    }

    public void paint(AbstractDevice device) {
        if (getControlPointCount() < 1) return;
        device.setColor(Color.BLUE);
        device.setLineStyle(AbstractDevice.CLEAR_LINE);
        device.setLineWidth(DPIUtils.pixelsToMilimeters(1 / device.getZoom()));
        
        Point padding = device.getPadding();
        double zoom = device.getZoom();
        
        Point p = getControlPoint(0);
        if (getProperties().getIntValue(XmlConstants.PROP_ORIENTATION) == HORIZONTAL)
            device.line(-padding.x / zoom, p.y, (device.getVisibleWidth() - padding.x) / zoom, p.y, AbstractDevice.DEFAULT_DRAW);
        else 
            device.line(p.x, -padding.y / zoom, p.x, (device.getVisibleHeight() - padding.y) / zoom, AbstractDevice.DEFAULT_DRAW);
        
    }

    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        
        Point px = getControlPoint(0);
        Point padding = device.getPadding();
        double zoom = device.getZoom();
        
        if (getProperties().getIntValue(XmlConstants.PROP_ORIENTATION) == HORIZONTAL)
            return (Geometry.isOnLine(new Point(-padding.x / zoom, px.y), new Point((device.getVisibleWidth() - padding.x) / zoom, px.y), p, Manager.PRECISSNESS / device.getZoom()) ? this : null);
        else
            return (Geometry.isOnLine(new Point(px.x, -padding.y / zoom), new Point(px.x, (device.getVisibleHeight() - padding.y) / zoom), p, Manager.PRECISSNESS / device.getZoom() ) ? this : null);
    }

    public void move(Point direction) {
        getControlPoint(0).translate(direction);
    }

    public void scale(Point center, Point coef) {
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#snaps(diagnose.utils.geometry.Point)
     */
    public Point snaps(Point p) {
        Line line;
        
        Point px = getControlPoint(0);
        if (getProperties().getIntValue(XmlConstants.PROP_ORIENTATION) == HORIZONTAL)
            line = new Line(new Point(Manager.MINIMAL_PADDING_X, px.y), new Point(Manager.MAXIMAL_PADDING_X * 2, px.y));
        else
            line = new Line(new Point(px.x, Manager.MINIMAL_PADDING_Y), new Point(px.x, Manager.MAXIMAL_PADDING_Y * 2));
        
        return line.getNearestPoint(p);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.VisualElement#getSnapablePoints()
     */
    public ArrayList getSnapablePoints() {
        return new ArrayList();
    }

    public Rect getRectBounds() {
        return null;
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_GUIDE_LINE;
    }

    public void suggestRelation(Relations relations, SubmissiveElement submissiveElt, int id, Point position) {
        // ignore that
    }
}