/*
 * Created on Jul 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.system;

import java.util.ArrayList;

import org.jdom.Element;

import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.GraphicElement;
import diagnose.graphicengine.elements.VisualElement;
import diagnose.utils.geometry.Point;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.Serializer;
import diagnose.utils.serialization.XMLSeriazable;
import diagnose.utils.serialization.XmlConstants;

/**
 * Container for all guidelines
 */
public class GuideLines implements GraphicElement, XMLSeriazable {
    
    private ArrayList guideLines = new ArrayList();

    public GuideLines() {
        super();
    }
    
    public void add(GuideLine guideLine) {
        guideLines.add(guideLine);
    }
    
    public void remove(GuideLine guideLine) {
        guideLines.remove(guideLine);
    }
    
    public boolean hasGuideLine(GuideLine guideLine) {
        return guideLines.contains(guideLine);
    }

    public void paint(AbstractDevice device) {
        for (int i = 0; i < guideLines.size(); i++)
            ((GuideLine)guideLines.get(i)).paint(device);
    }

    public VisualElement receivesMouseEvent(Point p, AbstractDevice device) {
        for (int i = guideLines.size() - 1; i >= 0; i--) {
            GuideLine elt = (GuideLine)guideLines.get(i); 
            if (elt.receivesMouseEvent(p, device) != null) return elt;
        }
        return null;
    }
    
    public Point snaps(Point p) { // should be accessible using some interface
        Point result = null;
        for (int i = 0; i < guideLines.size(); i++) {
            Point px = ((GuideLine)guideLines.get(i)).snaps(p);
            if ((px != null) && (result == null)) result = px;
            else if ((px != null) && (result.distance(p) > px.distance(p))) result = px;
        }
        return result;
    }

    public Element serialize() {
        return Serializer.serializeCollection(getElementType(), guideLines);
    }

    public void deserialize(Element elt, Deserializer deserializer)
            throws DeserializationException {
        for (int i = 0; i < elt.getChildren().size(); i++)
            add((GuideLine)deserializer.deserialize((Element)elt.getChildren().get(i)));
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_GUIDE_LINES;
    }

}
