/*
 * Created on Feb 7, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.elements.system;

import java.util.ArrayList;

import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.ComposedElement;
import diagnose.graphicengine.elements.Document;
import diagnose.graphicengine.elements.implementations.GroupElement;
import diagnose.utils.properties.Properties;
import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyDescription;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.properties.PropertyReadListener;

/**
 *
 */
public class Selection extends ComposedElement {
    
    private ControlPoints controlPoints = new ControlPoints();
    
    /**
     * Clears all elements in selection
     */
    public void clear() {
        elements.clear();
        controlPoints.clear();
    }
    
    /**
     * @return true if selection is empty
     */
    public boolean isEmpty() {
        return (elements.size() == 0);
    }
    
    /**
     * @return true in case selection contains specified elt
     */
    public boolean hasElement(CommonElement elt) {
        return elements.contains(elt);
    }
    
    /**
     * Pastes selection into the other element (ie. document)
     */
    public void pasteInto(ComposedElement doc) {
        
        for (int i = 0; i < elements.size(); i++)
            doc.addElement(this.getElement(i));
    }
    
    /**
     * Removes selection from document
     */
    public void removeFromDocument(Document doc) {
        for (int i = 0; i < elements.size(); i++)
            doc.removeElement(this.getElement(i));
    }
    
    /**
     * @return true if selection contains splitable elements
     */
    public boolean hasSplitableElements() {
        for (int i = 0; i < getElementsCount(); i++)
            if (getElement(i) instanceof GroupElement) return true;
        return false;
    }

    public String getElementType() {
        return null;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.elements.CommonElement#getPropertryDescriptor()
     */
    public PropertyDescriptor getPropertryDescriptor() {
        return null;
    }

    public void addElement(CommonElement elt) {
        super.addElement(elt);
        controlPoints.addControlPointsOfElement(elt);
    }

    public Object clone() {
        Selection result = (Selection)super.clone();
        for (int i = 0; i < result.getElementsCount(); i++) {
            CommonElement elt = result.getElement(i);
            result.controlPoints.addControlPointsOfElement(elt);
        }
        return result;
    }

    public void removeElement(CommonElement elt) { 
        super.removeElement(elt);
        controlPoints.removeControlPointsOfElement(elt);
    }
    
    public ControlPoints getControlPoints() {
        return controlPoints;
    }

    ////////////////////// FUNCTIONS FOR GETTING PROPERTIES ////////////////////
    public Properties getProperties() {
        
        if ((elements == null) || (getElementsCount() == 0)) return null;
        
        Properties properties = new Properties();
        PropertyDescriptor descriptor = new PropertyDescriptor();
        properties.setPropertyDescriptor(descriptor);
        
        ArrayList propertiesNames = getElement(0).getProperties().getPropertiesNames();
        for (int i = 1; i < getElementsCount(); i++) {
            propertiesNames = intersection(propertiesNames,
                    getElement(i).getProperties().getPropertiesNames());
        }
        
        for (int i = 0; i < propertiesNames.size(); i++) {
            String name = (String)propertiesNames.get(i);
            ArrayList descriptions = getPropertyDescriptions(name);
            if ((descriptions.size() == 1) && (descriptions.get(0) != null)) {
                descriptor.setDescription((PropertyDescription)descriptions.get(0));
            }
        }
        
        for (int i = 0; i < propertiesNames.size(); i++) {
            String propertyName = (String)propertiesNames.get(i);
            ArrayList values = getPropertyValues(propertyName);
            
            if (values.size() == 1) {
                Property p = (Property)((Property)values.get(0)).clone();
                properties.setProperty(p);
            } else if (values.size() > 1) {
                Property p = (Property)((Property)values.get(0)).clone();
                
                p.setPropertyReadListener(new PropertyReadListener() {

                    public Object getValue() {
                        return null;
                    }});
                
                p.setPropertyChangedListener(null);
                p.setValue(null);
                properties.setProperty(p);
            }
        }
        
        return properties;
        
    }
    
    private ArrayList getPropertyDescriptions(String propertyName) {
        ArrayList descriptions = new ArrayList(2);
        descriptions.contains(null);
        for (int i = 0; i < getElementsCount(); i++) {
            CommonElement elt = getElement(i);
            PropertyDescription desc = elt.getProperties().getPropertyDescription(propertyName); 
            if (!descriptions.contains(desc)) descriptions.add(desc);
        }
                
        return descriptions;
    }
    
    private ArrayList getPropertyValues(String propertyName) {
        ArrayList properties = new ArrayList();
        for (int i = 0; i < getElementsCount(); i++) {
            CommonElement elt = getElement(i);
            Property prop = elt.getProperties().getProperty(propertyName);
            if (!properties.contains(prop)) properties.add(prop);
        }
                
        return properties;
    }
    
    private static ArrayList intersection(ArrayList a1, ArrayList a2) {
        ArrayList result = new ArrayList();
        for (int i = 0; i < a1.size(); i++) {
            for (int j = 0; j < a2.size(); j++) {
                if (a1.get(i).equals(a2.get(j))) {
                    result.add(a1.get(i));
                    break;
                }
            }
        }
        return result;
    }
}
