/*
 * Created on Jul 15, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.events;

import java.util.ArrayList;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.system.Selection;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.geometry.Rect;

/**
 * Moves element to the specified alignment line
 */
public class AlignEvent extends PersistentEvent {

    public static final int TOP     = 1;
    public static final int BOTTOM  = 2;
    public static final int RIGHT   = 3;
    public static final int LEFT    = 4;
    public static final int CENTER_X = 5;
    public static final int CENTER_Y = 6;
    
    private ArrayList movements = new ArrayList();
    private int direction;
    
    /**
     * @param document
     */
    public AlignEvent(Manager manager, Selection selection, int direction) {
        super(manager);
        this.direction = direction;
        
        Rect masterRect = selection.getRectBounds();
        for (int i = 0; i < selection.getElementsCount(); i++) {
            
            Point dir = new Point(0, 0);
            
            CommonElement elt = selection.getElement(i); 
            Rect eltRect = elt.getRectBounds();
            
            if (direction == TOP) dir = new Point(0, masterRect.getPosition().y - eltRect.getPosition().y);
            if (direction == CENTER_X) dir = new Point(0, (masterRect.getPosition().y + masterRect.getHeight() / 2) - (eltRect.getPosition().y + eltRect.getHeight() / 2));
            if (direction == BOTTOM) dir = new Point(0, masterRect.getOppositePoint().y - eltRect.getOppositePoint().y);
            if (direction == LEFT) dir = new Point(masterRect.getPosition().x - eltRect.getPosition().x, 0);
            if (direction == CENTER_Y) dir = new Point((masterRect.getPosition().x + masterRect.getWidth() / 2) - (eltRect.getPosition().x + eltRect.getWidth() / 2), 0);
            if (direction == RIGHT) dir = new Point(masterRect.getOppositePoint().x - eltRect.getOppositePoint().x, 0);
            
            movements.add(new Movement(elt, dir));
        }
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#perform()
     */
    public void perform() {
        for (int i = 0; i < movements.size(); i++)
            ((Movement)movements.get(i)).move();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#undo()
     */
    public void undo() {
        for (int i = 0; i < movements.size(); i++)
            ((Movement)movements.get(i)).undoMove();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#getDescription()
     */
    public String getDescription() {
        String comment;
        switch (direction) {
        case TOP:
            comment = Lang.get("event.alignment.top");
            break;
            
        case BOTTOM:
            comment = Lang.get("event.alignment.bottom");
            break;
            
        case LEFT:
            comment = Lang.get("event.alignment.left");
            break;
            
        case RIGHT:
            comment = Lang.get("event.alignment.right");
            break;
            
        case CENTER_X:
            comment = Lang.get("event.alignment.center.x");
            break;
            
        case CENTER_Y:
            comment = Lang.get("event.alignment.center.y");
            break;
            
        default:
            comment = Lang.get("event.alignment.unknown");
            break;
        }
        return comment;
    }
    
    private static class Movement {
        
        private CommonElement elt;
        private Point direction;
        
        public Movement(CommonElement elt, Point direction) {
            super();
            this.direction = direction;
            this.elt = elt;
        }
        
        public void move() {
            elt.move(direction);
        }
        
        public void undoMove() {
            Point reverseDirection = (Point)direction.clone();
            reverseDirection.multiply(-1);
            elt.move(reverseDirection);
        }
    }
}
