/*
 * Created on Feb 18, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.events;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.system.Selection;
import diagnose.utils.Lang;

/**
 * Pastes content of clipboard (selection) into document
 */
public class PasteSelectionEvent extends PersistentEvent {

    private Selection selection;
    
    public PasteSelectionEvent(Manager manager, Selection selection) {
        super(manager);
        this.selection = (Selection)selection.clone();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistantEvent#perform()
     */
    public void perform() {
        selection.pasteInto(getDocument());
        getManager().clearSelectionEvent();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistantEvent#undo()
     */
    public void undo() {
        selection.removeFromDocument(getDocument());
        
        for (int i = 0; i < selection.getElementsCount(); i++) {
            CommonElement elt = selection.getElement(i);
            if (getManager().hasSelectionElement(elt))
                getManager().removeElementFromSelectionEvent(elt);
        }
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistantEvent#getDescription()
     */
    public String getDescription() {
        return Lang.get("event.paste");
    }

}
