/*
 * Created on Jun 11, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.events;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.system.Selection;
import diagnose.utils.Lang;
import diagnose.utils.properties.PropertyChangeList;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class SendBackEvent extends PersistentEvent {
    
    private Selection selection;
    private PropertyChangeList changes = new PropertyChangeList();

    /**
     * @param document
     */
    public SendBackEvent(Manager manager, Selection selection) {
        super(manager);
        this.selection = selection;
        selection.reorderElements();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#perform()
     */
    public void perform() {
        int zindex = getDocument().getLeastZIndex();
        for (int i = selection.getElementsCount() - 1; i >= 0; i--) {
            changes.add(new PropertyChangeLog(
                    selection.getElement(i).getProperties().getProperty(XmlConstants.PROP_ZINDEX), 
                    new Integer(--zindex)));
        }
        changes.applyChanges();
        getDocument().reorderElements();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#undo()
     */
    public void undo() {
        changes.revertChanges();
        getDocument().reorderElements();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#getDescription()
     */
    public String getDescription() {
        return Lang.get("event.sendback");
    }
}
