/*
 * Created on Feb 19, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.events;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.CommonElement;
import diagnose.graphicengine.elements.implementations.GroupElement;
import diagnose.graphicengine.elements.system.Selection;
import diagnose.utils.Lang;

/**
 *
 */
public class SplitSelectionEvent extends PersistentEvent {

    private Selection newElts = new Selection();
    private Selection oldElts = new Selection();
    
    /**
     * @param document
     */
    public SplitSelectionEvent(Manager manager, Selection selection) {
        super(manager);
        
        for (int i = 0; i < selection.getElementsCount(); i++)
            if (selection.getElement(i) instanceof GroupElement) {
                GroupElement elt = (GroupElement)selection.getElement(i);
                for (int j = 0; j < elt.getElementsCount(); j++) {
                    newElts.addElement(elt.getElement(j));
                }
                oldElts.addElement(elt);
            }
        
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#perform()
     */
    public void perform() {

        oldElts.removeFromDocument(getDocument());
        for (int i = 0; i < oldElts.getElementsCount(); i++) {
            getManager().removeElementFromSelectionEvent(oldElts.getElement(i));
        }
        
        newElts.pasteInto(getDocument());
        for (int i = 0; i < newElts.getElementsCount(); i++) {
            getManager().addElementToSelectionEvent(newElts.getElement(i));
        }
        getManager().getRelationManager().updateRelations(newElts);
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#undo()
     */
    public void undo() {
        
        oldElts.pasteInto(getDocument());
        newElts.removeFromDocument(getDocument());
        for (int i = 0; i < newElts.getElementsCount(); i++) {
            CommonElement elt = newElts.getElement(i);
            if (getManager().hasSelectionElement(elt))
                getManager().removeElementFromSelectionEvent(elt);
        }
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.events.PersistentEvent#getDescription()
     */
    public String getDescription() {
        return Lang.get("event.split");
    }

}
