/*
 * Created on Apr 21, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.libraries;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;

import diagnose.graphicengine.elements.CommonElement;
import diagnose.utils.Lang;
import diagnose.utils.Log;
import diagnose.utils.properties.StringProperty;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.Serializer;
import diagnose.utils.serialization.XMLSeriazable;
import diagnose.utils.serialization.XmlConstants;

/**
 * Manages libraries
 */
public class Librarian implements XMLSeriazable {

    private ArrayList libraries = new ArrayList();

    public Librarian() {
        super();
    }
    
    public void putLibrary(Library lib) {
        int index = getLibraryIndex(lib.getName());
        if (index >= 0) {
            Log.info(Lang.get("log.oldlibraryremoved", lib.getName()));
            removeLibrary(lib.getName());
        }
        libraries.add(lib);
    }
    
    public int getLibCount() {
        return libraries.size();
    }
    
    public Library getLibrary(int i) {
        return (Library)libraries.get(i);
    }
    
    public Library getLibrary(String name) {
        int index = getLibraryIndex(name);
        if (index >= 0) return (Library)libraries.get(index);
        return null;
    }
    
    private int getLibraryIndex(String name) {
        for (int i = 0; i < libraries.size(); i++) {
            Library lib = (Library)libraries.get(i);
            if (lib.getProperties().getStringValue(XmlConstants.PROP_NAME).equals(name))
                return i;
        }
        return -1;
    }
    
    public void removeLibrary(String name) {
        libraries.remove(getLibraryIndex(name));
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.serialization.XMLSeriazable#serialize()
     */
    public Element serialize() {
        Element result = builder.element(getElementType());
        for (int i = 0; i < libraries.size(); i++) {
            Library l = (Library)libraries.get(i);
            Element lib = builder.element(XmlConstants.PROP_LIBRARY);            
            lib.setAttribute(XmlConstants.PROP_NAME, l.getProperties().getStringValue(XmlConstants.PROP_NAME));
            lib.setAttribute(XmlConstants.PROP_URL, l.getProperties().getStringValue(XmlConstants.PROP_URL));
            lib.setAttribute(XmlConstants.PROP_DESCRIPTION, l.getProperties().getStringValue(XmlConstants.PROP_DESCRIPTION));
            result.addContent(lib);
        }
        return result;
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.serialization.XMLSeriazable#deserialize(org.jdom.Element)
     */
    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        List l = elt.getChildren();
        for (int i = 0; i < l.size(); i++) {
            Element libElt = (Element)l.get(i);
            try {
                String libPath = libElt.getAttributeValue(XmlConstants.PROP_URL);
                String desc = libElt.getAttributeValue(XmlConstants.PROP_DESCRIPTION);
                Library lib = (Library)deserializer.deserialeze(new File(libPath));
                lib.setLibrarian(this);
                lib.getProperties().setProperty(new StringProperty(XmlConstants.PROP_URL, libPath));
                lib.getProperties().setProperty(new StringProperty(XmlConstants.PROP_DESCRIPTION, desc));
                putLibrary(lib);
            } catch (Exception e) {
                throw new DeserializationException(Lang.get("log.unabletoloadlibrary", libElt.getAttributeValue(XmlConstants.PROP_URL)));
            } 
        }
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.serialization.XMLSeriazable#getElementType()
     */
    public String getElementType() {
        return XmlConstants.ELEMENT_LIB_CONFIG;
    }
    
    public Object clone() {
        Librarian result = new Librarian();
        for (int i = 0; i < libraries.size(); i++) {
            Library lib =  (Library)((Library)libraries.get(i)).clone();
            lib.setLibrarian(result);
            result.libraries.add(lib);
        }
            
        return result;
    }
    
    public void saveLibraries() {
        for (int i = 0; i < libraries.size(); i++) {
            Library lib = (Library)libraries.get(i);
            String libPath = lib.getProperties().getStringValue(XmlConstants.PROP_URL);
            try {
                Serializer.serialize(lib, new File(libPath));
            } catch (IOException e) {
                Log.error(Lang.get("log.unabletosavelibrary", lib.getProperties().getStringValue(XmlConstants.PROP_NAME)));
            }
        }
    }
    
    /**
     * @return true if librarian has specified element in library
     */
    public boolean hasElement(LibElementIdentifier identifier) {
        Library lib = getLibrary(identifier.getLibraryName());
        if (lib == null) return false;
        
        CommonElement elt = lib.getElement(identifier.getElementName());
        if (elt == null) return false;
        return true;
    }
}
