/*
 * Created on Jul 13, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.libraries.scripting;

import java.util.ArrayList;
import java.util.HashMap;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.ImporterTopLevel;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

import diagnose.utils.Lang;
import diagnose.utils.Log;

/**
 * Scope pecific for each element
 */
public class ElementScope {
    
    private Context cx;
    private Scriptable scope;
    private String elementID;
    private int lineID = 1;
    
    private HashMap functions = new HashMap();
    
    /**
     * Creates global scope for each element
     * @param elementID - helps to identify objects 
     */
    public ElementScope(String elementID) {
        this.elementID = elementID;
        cx = Context.enter();
        cx.setLanguageVersion(Context.VERSION_1_6);
        //scope = cx.initStandardObjects();
        scope = new ImporterTopLevel(cx);
    }
    
    public void importPackage(String pkg) {
        cx.evaluateString(scope, "importPackage(" + pkg + ");", elementID + "#GlobalScope", lineID++, null);
    }
    
    /**
     * Creates function in global scope
     */
    public void compileFunction(String name, String body) {
        Function fn = cx.compileFunction(scope, body, elementID + "#" + name, 1, null);
        functions.put(name, fn);
    }
    
    /**
     * Calls named function with given parameters
     */
    public Object callFunctionEx(String name, ArrayList params) {
        Function fn = (Function)functions.get(name);
        if (fn == null) {
            Log.warn(Lang.get("log.unabletofindmethod", name, elementID)); 
            return null;
        }
        return fn.call(cx, scope, scope, params.toArray());
    }
    
    
    /**
     * Calls named function without parameters
     */
    public Object callFunction(String name) {
        Function fn = (Function)functions.get(name);
        if (fn == null) {
            Log.warn(Lang.get("log.unabletofindmethod", name, elementID));
            return null;
        }
        return fn.call(cx, scope, scope, null);
    }
    
    
    public Object callFunction(String name, Object param1) {
        ArrayList al = new ArrayList(1);
        al.add(param1);
        return callFunctionEx(name, al);
    }
    
    public Object callFunction(String name, Object param1, Object param2) {
        ArrayList al = new ArrayList(2);
        al.add(param1);
        al.add(param2);
        return callFunctionEx(name, al);
    }
    
    /**
     * Puts object into scope under its name
     */
    public void putObject(String name, Object obj) {
        Object wrappedOut = Context.javaToJS(obj, scope);
        ScriptableObject.putProperty(scope, name, wrappedOut);
    }
    
    /**
     * Converts internal internal representation of java objects
     * back to java objects
     */
    public static Object unwrap(Object obj) {
        return ((org.mozilla.javascript.NativeJavaObject)obj).unwrap();
    }
    
    public void evalString(String functions) {
        cx.evaluateString(scope, functions, elementID + "#GlobalScope", 1, null);
    }
}
