/*
 * Created on Jul 14, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.libraries.scripting;

import java.util.ArrayList;

import diagnose.utils.serialization.XmlConstants;

/**
 * Generates ElementScopes from specified methods
 * and values
 */
public class ElementScopeFactory {
    
    private ArrayList methods = new ArrayList();
    private ArrayList imports = new ArrayList();
    private ArrayList values = new ArrayList();
    
    private String elementID;
    
    public ElementScopeFactory(String elementID) {
        super();
        this.elementID = elementID;
    }
    
    public void importPackage(String pkg) {
        imports.add(pkg);
    }
    
    public void addMethod(String name, String body) {
        methods.add(new Method(name, body));
        
    }
    
    public void addValue(String name, Object value) {
        values.add(new Value(name, value));
    }
    
    public ElementScope createElementScope() {
        ElementScope result = new ElementScope(elementID);
        
        // imports standard packages
        for (int i = 0; i < imports.size(); i++)
            result.importPackage((String)imports.get(i));
        
        // set values
        for (int i = 0; i < values.size(); i++) {
            Value val = (Value)values.get(i);
            result.putObject(val.name, val.value);
        }
        
        // adds methods
        for (int i = 0; i < methods.size(); i++) {
            Method met = (Method)methods.get(i);
            if (met.name.equals(XmlConstants.METHOD_INIT)) continue;
            result.compileFunction(met.name, met.body);
        }
        
        return result;
    }
        
    ////////////////// STORAGE CLASSES ///////////////////
    
    private class Method {
        
        public String name;
        public String body;
        
        public Method(String name, String body) {
            super();
            this.body = body;
            this.name = name;
        }
    }
    
    private class Value {
        
        public String name;
        public Object value;
        
        public Value(String name, Object value) {
            super();
            this.name = name;
            this.value = value;
        }        
    }
}
