/*
 * Created on Aug 8, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.graphicengine.relation;

import java.util.ArrayList;

import diagnose.utils.geometry.Geometry;
import diagnose.utils.geometry.Point;

/**
 * Holds informations for each element about its relations
 */
public class Relations {

    private ArrayList relations = new ArrayList();
    
    public Relations() {
        super();
    }
    
    public void add(SubmissiveElement subElt, int id, Point position) {
        Relation newRel = new Relation(subElt, id, position); 
        if (!relations.contains(newRel)) relations.add(newRel);
    }
    
    public int size() {
        return relations.size();
    }
    
    public Relation get(int index) {
        return (Relation)relations.get(index);
    }
    
    public void clear() {
        relations.clear();
    }
    
    /**
     * Removes only relations to specfied element
     */
    public void clear(SubmissiveElement elt) {
        for (int i = 0; i < size(); i++)
            if (get(i).getSubmissiveElement().equals(elt)) relations.remove(i);
    }
    
    /**
     * Moves all points in one direction
     */
    public void move(Point direction) {
        for (int i = 0; i < relations.size(); i++) {
            Relation r = (Relation)relations.get(i);
            Point p = r.getPosition();
            p.translate(direction);
            r.getSubmissiveElement().changeSubmissive(r.getSubmissiveId(), p);
        }  
    }
    
    public void scale(Point center, Point coef) {
        for (int i = 0; i < relations.size(); i++) {
            Relation r = (Relation)relations.get(i);
            Point p = r.getPosition();
            
            p = (Point)Geometry.scale(p, center, coef).clone();
            r.getSubmissiveElement().changeSubmissive(r.getSubmissiveId(), p);
            r.setPosition(p);
        }
    }
}
