/*
 * Created on Aug 9, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.filechooser.FileFilter;

/**
 *
 */
public class DefaultFileFilter extends FileFilter {

    private HashMap extensions;
    private String description;
    
    /**
     * Filter that accepts any file
     */
    public DefaultFileFilter(String description) {
        super();
        this.description = description;
    }
    
    public DefaultFileFilter(String extension, String description) {
        super();
        extensions = new HashMap();
        extensions.put(extension.toLowerCase(), this);
        this.description = description;
    }
    
    public DefaultFileFilter(String [] extension, String description) {
        super();
        extensions = new HashMap();
        for (int i = 0; i < extension.length; i++)
            extensions.put(extension[i].toLowerCase(), this);
        this.description = description;
    }

    public boolean accept(File f) {
        if (f != null) {
            if (f.isDirectory()) return true;
            
            String ext = getExtension(f);
            if (extensions.get(ext) != null) return true;
        }
        return false;
    }

    public String getDescription() {
        if (extensions == null) return description + " (*.*)";
        
        String result = description + " (";
        Iterator exts = extensions.keySet().iterator();
        result += "*." + (String) exts.next();
        while (exts.hasNext()) {
            result += ", *." + (String) exts.next();
        }
        
        result += ")";
        return result;
    }
    
    private String getExtension(File f) {
        if(f != null) {
            String filename = f.getName();
            int i = filename.lastIndexOf('.');
            if ((i > 0) && (i < filename.length() - 1)) {
                return filename.substring(i+1).toLowerCase();
            }
        }
        return "";
    }
}
