/*
 * Created on Apr 22, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.libraries.LibElementIdentifier;
import diagnose.graphicengine.libraries.Librarian;
import diagnose.graphicengine.libraries.Library;
import diagnose.gui.properties.PropertyDialog;
import diagnose.utils.GUIUtils;
import diagnose.utils.IconCache;
import diagnose.utils.Lang;
import diagnose.utils.properties.Properties;
import diagnose.utils.properties.PropertyChangeList;
import diagnose.utils.properties.PropertyDescriptor;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class LibrariesForm extends JDialog {

    private static final long serialVersionUID = 3670299905653546206L;
    
    private JPanel buttonPanel;
    private JButton addLibraryItemBtn;
    private JButton removeLibraryBtn;
    private JButton propertiesBtn;
    private JButton removeItemBtn;
    private JButton okBtn;
    private JButton cancelBtn;
    
    private JTree libraryTree;
    private Manager manager;
    
    private static PropertyDescriptor libPropertyDescriptor = new PropertyDescriptor();
    
    static {
        libPropertyDescriptor.setDescription(XmlConstants.PROP_NAME, Lang.get("property.elementname"), Lang.get("category.description"), 1);
    }
    
    /** currently selected object in tree */
    private Object activeTreeObject;
    private DefaultMutableTreeNode activeNode;
    
    private DefaultMutableTreeNode rootNode;
    
    private Librarian librarian;
    private static File lastUsedPath = null;
    
    /**
     * @param owner
     * @throws java.awt.HeadlessException
     */
    public LibrariesForm(final JFrame owner, final Manager manager) throws HeadlessException {
        super(owner, true);
        
        this.getContentPane().setLayout(new BorderLayout());
        this.setTitle(Lang.get("dialog.libs.caption"));
        this.manager = manager;
        this.setModal(true);
        
        this.setSize(500, 300);
        librarian = (Librarian)manager.getLibrarian().clone();
        rootNode = createLibraryTree();
        libraryTree = new JTree(rootNode);
        libraryTree.setCellRenderer(new LibraryTreeCellRenderer());
        libraryTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        libraryTree.addTreeSelectionListener(new TreeSelectionListener () {

            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)libraryTree.getLastSelectedPathComponent();
                activeNode = node;
                if (node != null) activeTreeObject = node.getUserObject();
                else activeTreeObject = null;
                refreshForm();
            }
        });
        
        buttonPanel = new JPanel();
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(16, 16, 16, 16));
        GridLayout layout = new GridLayout(0, 1);
        layout.setHgap(8);
        layout.setVgap(8);
        buttonPanel.setLayout(layout);
        
        addLibraryItemBtn = new JButton(Lang.get("dialog.libs.addlibrary"));
        addLibraryItemBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(getLastUsedPath());
                fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                DefaultFileFilter dlf = new DefaultFileFilter("dlf", Lang.get("format.dlf"));
                fc.addChoosableFileFilter(dlf);
                int returnVal = fc.showOpenDialog(null);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File file = fc.getSelectedFile();
                    setLastUsedPath(file);
                   try {
                       Library lib = manager.loadLibrary(file.getAbsoluteFile());
                       
                       if (manager.getLibraries().contains(lib.getName())) {
                           
                           int result = JOptionPane.showConfirmDialog(getParent(), 
                                       Lang.get("msg.libraryinsystem", lib.getName()), 
                                       Lang.get("msg.caption.caution"),
                                       JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                           if (result == JOptionPane.YES_OPTION) {
                               
                               DefaultMutableTreeNode oldNode = getLibraryNode(lib.getName());
                               oldNode.removeFromParent();
                               manager.putLibrary(lib);
                               
                               rootNode.add(createLibrarySubTree(new LibName(lib.getName())));
                               ((DefaultTreeModel)libraryTree.getModel()).reload(rootNode);
                           }
                       } else {
                           manager.putLibrary(lib);
                           rootNode.add(createLibrarySubTree(new LibName(lib.getName())));
                           ((DefaultTreeModel)libraryTree.getModel()).reload(rootNode);
                       }
                   } catch (Exception ex) {
                       JOptionPane.showMessageDialog(null, Lang.get("dialog.libs.unabletoloadlib"), Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                       ex.printStackTrace();
                   }
                }
            } });
        
        removeLibraryBtn = new JButton(Lang.get("dialog.libs.removelibrary"));
        removeLibraryBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                manager.removeLibrary(((LibName)activeTreeObject).getName());
                DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)activeNode.getParent();
                activeNode.removeFromParent();
                ((DefaultTreeModel)libraryTree.getModel()).reload(parentNode);
            }});
        
        
        propertiesBtn = new JButton(Lang.get("dialog.libs.properties"));
        propertiesBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {              
                if (activeTreeObject != null) {
                    PropertyDialog dlg = null;
                    
                    if (activeTreeObject instanceof LibName) {
                        String libName = ((LibName)activeTreeObject).getName();
                        
                        Properties props = manager.getLibraryProperties(libName);
                        dlg = new PropertyDialog(getDialogSelf(), props);
                        dlg.runForm();
                        
                        PropertyChangeList changes = dlg.getChanges();
                        if (changes != null) changes.applyChanges();

                        activeNode.setUserObject(new LibName(props.getStringValue(XmlConstants.PROP_NAME)));
                        ((DefaultTreeModel)libraryTree.getModel()).reload(rootNode);
                    }
                    
                    if (activeTreeObject instanceof LibElement) {
                        String libName = ((LibElement)activeTreeObject).getLibraryName();
                        String eltName = ((LibElement)activeTreeObject).getElementName();
                        
                        Properties props = manager.getElementInLibraryProperties(libName, eltName);
                        PropertyDescriptor oldDesc = props.getPropertyDescriptor();
                        props.setPropertyDescriptor(libPropertyDescriptor);
                        dlg = new PropertyDialog(getDialogSelf(), props);
                        props.setPropertyDescriptor(oldDesc);
                        
                        dlg.runForm();
                        
                        PropertyChangeList changes = dlg.getChanges();
                        if (changes != null) changes.applyChanges();
                        
                        activeNode.setUserObject(new LibElement(libName, props.getStringValue(XmlConstants.PROP_NAME)));
                        ((DefaultTreeModel)libraryTree.getModel()).reload(rootNode);
                    }
                }
                
            }});
        
        removeItemBtn = new JButton(Lang.get("dialog.libs.removeitem"));
        removeItemBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                LibElement elt = (LibElement)activeTreeObject;
                manager.removeElementInLibrary(elt.getLibraryName(), elt.getElementName());
                DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)activeNode.getParent();
                activeNode.removeFromParent();
                ((DefaultTreeModel)libraryTree.getModel()).reload(parentNode);
            }});
        
        
        okBtn = new JButton(Lang.get("dialog.button.ok"));
        okBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                manager.saveLibraries();
                setVisible(false);
            }});
        
        cancelBtn = new JButton(Lang.get("dialog.button.cancel"));
        cancelBtn.addActionListener(new CancelActionListener());
        
        buttonPanel.add(addLibraryItemBtn);
        buttonPanel.add(removeLibraryBtn);
        buttonPanel.add(propertiesBtn);
        buttonPanel.add(removeItemBtn);
        buttonPanel.add(okBtn);
        buttonPanel.add(cancelBtn);
        
        JPanel libraryPanel = new JPanel(new BorderLayout());
        libraryPanel.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createEmptyBorder(16, 16, 16, 16),
                        BorderFactory.createLoweredBevelBorder()
                ));
        
        libraryPanel.add(libraryTree, BorderLayout.CENTER);
        
        this.getContentPane().add(new JScrollPane(libraryPanel), BorderLayout.CENTER);
        this.getContentPane().add(buttonPanel, BorderLayout.EAST);
        
        this.setResizable(false);
        
        refreshForm();
        
        getRootPane().setDefaultButton(okBtn);
        GUIUtils.centerWindow(this);
        GUIUtils.setEscapeListener(this, new CancelActionListener());
        //this.pack();
    }
    
    private DefaultMutableTreeNode getLibraryNode(String libName) {
        for (int i = 0; i < rootNode.getChildCount(); i++) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)rootNode.getChildAt(i);
            if (node.getUserObject().toString().equals(libName)) return node;
        }
        
        return null;
    }
    
    protected JDialog getDialogSelf() {
        return this;
    }

    /**
     * Refreshes buttons in form
     */
    protected void refreshForm() {
        removeItemBtn.setEnabled(activeTreeObject instanceof LibElement);
        removeLibraryBtn.setEnabled(activeTreeObject instanceof LibName);
        propertiesBtn.setEnabled((activeTreeObject instanceof LibName) 
                || (activeTreeObject instanceof LibElement));
    }

    /**
     * Creates tree of libraries 
     */
    private DefaultMutableTreeNode createLibraryTree() {
        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(Lang.get("dialog.libs.rootnode"));
        ArrayList libraries = manager.getLibraries();
        for (int i = 0; i < libraries.size(); i++) {
            String libName = (String)libraries.get(i);    
            rootNode.add(createLibrarySubTree(new LibName(libName)));
        }
        
        return rootNode;
    }
    
    private DefaultMutableTreeNode createLibrarySubTree(LibName libName) {
        DefaultMutableTreeNode node = new DefaultMutableTreeNode(libName);
        
        ArrayList elements = manager.getElementsInLibrary(libName.getName());
        for (int j = 0; j < elements.size(); j++) {
            String eltName = (String)elements.get(j);
            DefaultMutableTreeNode subNode = new DefaultMutableTreeNode(new LibElement(libName.getName(), eltName));
            node.add(subNode);
        }
        
        return node;
    }
    
    private final class CancelActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            // returns to prev state
            manager.setLibrarian(librarian);
            setVisible(false);
        }
        
    }
    
    
    private class LibraryTreeCellRenderer extends JLabel implements TreeCellRenderer {

        private static final long serialVersionUID = -1649031641162092122L;
        
        protected boolean selected;
        protected boolean focused;
        
        public LibraryTreeCellRenderer() {
            super();
        }
        
        public LibraryTreeCellRenderer(Object value) {
            super(value.toString());
        }
        
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {            
            setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
            
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
            setText(node.getUserObject().toString());
            if (node.getUserObject() instanceof LibName)setIcon(IconCache.getSystemIcon("library"));
            else if (node.getUserObject() instanceof LibElement) setIcon(IconCache.getSystemIcon("element"));
            else setIcon(IconCache.getSystemIcon("libraries"));
            
            this.selected = selected;
            this.focused = hasFocus;
            return this;
        }
        
        
        public void paint(Graphics g) {
            Icon ico = getIcon();
            int offset = (ico.getIconWidth() + getIconTextGap());
            if (selected) {
                g.setColor(new Color(184, 207, 229));
                g.fillRect(offset, 0, getWidth() - 1 - offset, getHeight() - 1);
                if (focused) {
                    g.setColor(g.getColor().darker());
                    g.drawRect(offset, 0, getWidth() - 1 - offset, getHeight() - 1);
                }
            } else {
                g.setColor(getParent().getBackground());
                g.fillRect(offset, 0, getWidth() - 1 - offset, getHeight() - 1);
            }
            
            
            super.paint(g);
        }
    }

    private static class LibElement extends LibElementIdentifier {
        
        public LibElement(String libraryName, String elementName) {
            super(libraryName, elementName);
        }
        
        public String toString() {
            return getElementName();
        }
    }

    private static class LibName {
        
        String name;
        
        public LibName(String name) {
            super();
            this.name = name;
        }
        
        public String getName() {
            return name;
        }
        
        public void setName(String name) {
            this.name = name;
        }
        
        public String toString() {
            return name;
        }
    }

    public static File getLastUsedPath() {
        return lastUsedPath;
    }

    public static void setLastUsedPath(File lastUsedPath) {
        if (lastUsedPath.isFile()) lastUsedPath = new File(lastUsedPath.getAbsolutePath());
        LibrariesForm.lastUsedPath = lastUsedPath;
    }
}
