/*
 * Created on Feb 6, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;

import diagnose.graphicengine.Abilities;
import diagnose.graphicengine.GraphicArea;
import diagnose.graphicengine.Manager;
import diagnose.graphicengine.UIInvoker;
import diagnose.graphicengine.devices.AbstractDevice;
import diagnose.graphicengine.elements.implementations.ArrowElement;
import diagnose.graphicengine.elements.implementations.LineElement;
import diagnose.graphicengine.elements.implementations.OvalElement;
import diagnose.graphicengine.elements.implementations.PolyLineElement;
import diagnose.graphicengine.elements.implementations.PolygonElement;
import diagnose.graphicengine.elements.implementations.RectElement;
import diagnose.graphicengine.elements.implementations.TextElement;
import diagnose.graphicengine.events.AlignEvent;
import diagnose.gui.menu.EditMenuActionFactory;
import diagnose.gui.menu.FileMenuActionFactory;
import diagnose.gui.menu.FormatMenuActionFactory;
import diagnose.gui.menu.InsertMenuActionFactory;
import diagnose.gui.menu.MenuFactory;
import diagnose.gui.menu.ViewMenuActionFactory;
import diagnose.gui.properties.PropertyDialog;
import diagnose.utils.DefaultArrays;
import diagnose.utils.GUIUtils;
import diagnose.utils.IconCache;
import diagnose.utils.Lang;
import diagnose.utils.Log;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.Properties;
import diagnose.utils.serialization.XmlConstants;

/**
 * Main form
 */
public class MainForm extends JFrame {
    
    private static final long serialVersionUID = 7857989487538439505L;
    private static final int DECIMAL_CORRECTION = 100;
    
    public static int activeWindows = 0;
    
    private File lastUsedPath = null;

    public static void main(String[] args) {
        if (args.length == 0) {
            try {
                SplashScreen.show();
                MainForm mfrm = new MainForm();
                mfrm.run();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                SplashScreen.hide();
            }
        } else {
            activeWindows++; // virtual window opening others
            for (int i = 0; i < args.length; i++) { 
                MainForm mfrm;
                try {
                    SplashScreen.show();
                    mfrm = new MainForm(args[i]);
                    mfrm.run();
                } catch (Exception e) {
                    Log.error(Lang.get("msg.unabletoopen", args[i]));
                } finally {
                    SplashScreen.hide();
                }
            }
            activeWindows--;
        }
    }
    
    private static final int CTRL   = ActionEvent.CTRL_MASK;
    private static final int ALT    = ActionEvent.ALT_MASK;
    private static final int SHIFT  = ActionEvent.SHIFT_MASK;
    
    private final GraphicArea areaBlah;
    private JPanel mainPanel;
    private JScrollBar vertScrollBar;
    private JScrollBar horizScrollBar;
    private Ruler horizRuler;
    private Ruler vertRuler;
    
    private JButton fgColorBtn;
    private JButton bgColorBtn;
    private JMenuBar mainMenu;
    private JMenu insertMenu;
    private JMenu toolsMenu;
    
    private JMenuItem saveIntoLibraryMenuItem;
    
    private JMenuItem copyMenuItem;
    private JMenuItem undoMenuItem;
    private JMenuItem redoMenuItem;
    private JMenuItem cutMenuItem;
    private JMenuItem pasteMenuItem;
    private JMenuItem deleteMenuItem;
    private JMenuItem selectAllMenuItem;
    private JMenuItem selectNoneMenuItem;
    
    private JMenuItem cntxCopyMenuItem;
    private JMenuItem cntxUndoMenuItem;
    private JMenuItem cntxRedoMenuItem;
    private JMenuItem cntxCutMenuItem;
    private JMenuItem cntxPasteMenuItem;
    private JMenuItem cntxDeleteMenuItem;
    private JMenuItem cntxPropertiesMenuItem;
    private JMenuItem cntxSendBackMenuItem;
    private JMenuItem cntxSendFrontMenuItem;
    private JMenuItem cntxGroupMenuItem;
    private JMenuItem cntxSplitMenuItem;
    
    private JMenuItem propertiesMenuItem;
    private JMenuItem groupMenuItem;
    private JMenuItem splitGroupMenuItem;
    private JMenuItem sendFrontMenuItem;
    private JMenuItem sendBackMenuItem;
    private JMenuItem alignTopMenuItem;
    private JMenuItem alignBottomMenuItem;
    private JMenuItem alignLeftMenuItem;
    private JMenuItem alignRightMenuItem;
    private JMenuItem alignCenterXMenutItem;
    private JMenuItem alignCenterYMenutItem;
    
    private JCheckBoxMenuItem snapToGridMenuItem;
    private JCheckBoxMenuItem snapToElementsMenuItem;
    private JCheckBoxMenuItem snapToGuideLinesMenuItem;
    private JCheckBoxMenuItem snapTechMenuItem;
    private JCheckBoxMenuItem showGridMenuItem;
    private JCheckBoxMenuItem showGuideLinesMenuItem;
    
    private JPopupMenu contextMenu;
    private JMenuItem librariesMenuItem;
    
    private JPanel masterToolBarPanel;
    private JToolBar fileToolBar;
    private JToolBar editToolBar;
    private JToolBar formatToolBar;
    private JToolBar groupToolBar;
    
    private JButton fileOpenBtn;
    private JButton fileNewBtn;
    private JButton fileSaveBtn;
    private JButton fileSaveAsBtn;

    private JButton editUndoBtn;
    private JButton editRedoBtn;
    private JButton editCopyBtn;
    private JButton editCutBtn;
    private JButton editPasteBtn;
    private JButton editDeleteBtn;
    
    private JComboBox formatFontFaceCombo;
    private JComboBox formatFontSizeCombo;
    private JToggleButton formatBoldBtn;
    private JToggleButton formatItalicBtn;
    private JToggleButton formatUnderlineBtn;
    private JToggleButton formatAlignLeftBtn;
    private JToggleButton formatAlignCenterBtn;
    private JToggleButton formatAlignRightBtn;
    private JComboBox formatLineStyleCombo;
    private JSpinner formatLineWidthSpinner;
    
    private JButton groupGroupBtn;
    private JButton groupSplitBtn;
    private JButton groupSendBackBtn;
    private JButton groupSendFrontBtn;    
    private JButton groupAlignTopBtn;
    private JButton groupAlignLeftBtn;
    private JButton groupAlignRightBtn;
    private JButton groupAlignBottomBtn;
    private JButton groupAlignCenterXBtn;
    private JButton groupAlignCenterYBtn;
    
    private JPanel statusBar;
    private JPanel sidePanel;
    private JPanel libElementsContainer;
    private JLabel mousePositionLabel;
    private JLabel contextInfoLabel;
    private JLabel zoomInfoLabel;
    
    private FileMenuActionFactory fileActionFactory;
    private EditMenuActionFactory editActionFactory;
    private ViewMenuActionFactory viewActionFactory;
    private FormatMenuActionFactory formatActionFactory;
    private InsertMenuActionFactory insertActionFactory;
    
    public MainForm() throws Exception {
        this(null);
    }
    
    public MainForm(String fileName) throws Exception {
        super();
        SplashScreen.setInfo(Lang.get("splash.creatingworkspace"));
        this.setTitle(Lang.get("application.title"));
        this.setSize(900, 700);
        GUIUtils.centerWindow(this);
        
        this.addWindowListener(new WindowAdapter() {
            	public void windowClosing(WindowEvent e) {
            	    closeWindow();
            	}});
        this.getContentPane().setLayout(new BorderLayout());
        
        mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        
        areaBlah = new GraphicArea();
       
        areaBlah.getManager().setSnapToGrid(false);
        areaBlah.getManager().setShowGrid(false);
        //areaBlah.getManager().setPadding(new Point(250, 250));
        //areaBlah.getManager().setZoom(2);
        
        areaBlah.getManager().setPropertyDialogInvoker(new PropertyDialogInvoker(this));
        areaBlah.getManager().setMenuRefreshInvoker(new MenuRefreshInvoker());
        areaBlah.getManager().setContextMenuInvoker(new ContextMenuInvoker(this));
        areaBlah.getManager().setViewChangeInvoker(new ViewChangeInvoker(this));
        areaBlah.getManager().setMouseChangeInvoker(new MouseChangeInvoker(this));
        areaBlah.getManager().setDefaultPropertyChangeInvoker(new DefaultPropertyChangeInvoker(this));
        areaBlah.getManager().setNotFoundElementsInvoker(new NotFoundElementsInvoker(this));
        
        fileActionFactory = new FileMenuActionFactory(areaBlah.getManager(), this);
        editActionFactory = new EditMenuActionFactory(areaBlah.getManager(), this);
        insertActionFactory = new InsertMenuActionFactory(areaBlah.getManager(), this);
        formatActionFactory = new FormatMenuActionFactory(areaBlah.getManager(), this);
        viewActionFactory = new ViewMenuActionFactory(areaBlah.getManager(), this);
        
        SplashScreen.setInfo(Lang.get("splash.creatinggui"));
        createMenu();
        createToolBars();
        createScrollBars();
        
        horizRuler = new Ruler(areaBlah.getManager(), Ruler.HORIZONTAL);
        vertRuler = new Ruler(areaBlah.getManager(), Ruler.VERTICAL);
        
        JPanel topPanel = new JPanel(new BorderLayout());
        JPanel bottomPanel = new JPanel(new BorderLayout());
        
        JPanel spacer1 = new JPanel();
        JPanel spacer2 = new JPanel();
        JPanel spacer3 = new JPanel();
        JPanel spacer4 = new JPanel();
        
        spacer1.setPreferredSize(new Dimension(20, 20));
        spacer2.setPreferredSize(new Dimension(20, 20));
        spacer3.setPreferredSize(new Dimension(20, 20));
        spacer4.setPreferredSize(new Dimension(20, 20));
        
        topPanel.add(spacer1, BorderLayout.WEST);
        topPanel.add(horizRuler, BorderLayout.CENTER);
        topPanel.add(spacer2, BorderLayout.EAST);
        
        bottomPanel.add(spacer3, BorderLayout.WEST);
        bottomPanel.add(horizScrollBar, BorderLayout.CENTER);
        bottomPanel.add(spacer4, BorderLayout.EAST);
        
        mainPanel.add(areaBlah, BorderLayout.CENTER);
        mainPanel.add(vertScrollBar, BorderLayout.EAST);
        mainPanel.add(bottomPanel, BorderLayout.SOUTH);
        mainPanel.add(topPanel, BorderLayout.NORTH);
        mainPanel.add(vertRuler, BorderLayout.WEST);
        
        this.getContentPane().add(mainPanel, BorderLayout.CENTER);
        
        masterToolBarPanel = new JPanel();
        JPanel upperPanel = new JPanel();
        JPanel downerPanel = new JPanel();
        masterToolBarPanel.setLayout(new BorderLayout());
        masterToolBarPanel.add(upperPanel, BorderLayout.NORTH);
        masterToolBarPanel.add(downerPanel, BorderLayout.SOUTH);
        
        upperPanel.setLayout(new BoxLayout(upperPanel, BoxLayout.X_AXIS));
        downerPanel.setLayout(new BoxLayout(downerPanel, BoxLayout.X_AXIS));
        upperPanel.add(fileToolBar);
        upperPanel.add(editToolBar);
        upperPanel.add(groupToolBar);
        downerPanel.add(formatToolBar);
        
        this.getContentPane().add(masterToolBarPanel, BorderLayout.NORTH);
        
        if (fileName != null) {
                areaBlah.getManager().openFile(new File(fileName));
        }
        
        statusBar = new JPanel();
        statusBar.setLayout(new BorderLayout());
        
        mousePositionLabel = new JLabel();
        Dimension dims = new Dimension(150, 24);
        mousePositionLabel.setMinimumSize(dims);
        mousePositionLabel.setPreferredSize(dims);
        mousePositionLabel.setMaximumSize(dims);
        mousePositionLabel.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createEtchedBorder(),
                        BorderFactory.createEmptyBorder(2, 2, 2, 2)));
        
        contextInfoLabel = new JLabel();
        contextInfoLabel.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createEtchedBorder(),
                        BorderFactory.createEmptyBorder(2, 2, 2, 2)));
        
        zoomInfoLabel = new JLabel();
        zoomInfoLabel.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createEtchedBorder(),
                        BorderFactory.createEmptyBorder(2, 2, 2, 2)));
       
        GridLayout gl = new GridLayout(1, 0);
        gl.setHgap(0);
        gl.setVgap(0);
        JPanel groupPanel = new JPanel(gl);
        groupPanel.add(mousePositionLabel);
        groupPanel.add(zoomInfoLabel);
        //groupPanel.setBorder(BorderFactory.createEmptyBorder(-5, -5, -5, -5));
        statusBar.add(groupPanel, BorderLayout.WEST);
        statusBar.add(contextInfoLabel, BorderLayout.CENTER);
        
        this.getContentPane().add(statusBar, BorderLayout.SOUTH);
        
        this.getContentPane().add(createLeftPanel(), BorderLayout.WEST);
        SplashScreen.setInfo(Lang.get("splash.finalizingui"));
        
        areaBlah.setFocusable(true);
        areaBlah.getManager().invokeMenuRefresh();
        areaBlah.getManager().invokeViewChange();
        areaBlah.getManager().invokeDefaultPropertyChange();
        
        addWindowListener(new WindowAdapter() {
            public void windowActivated(WindowEvent e) {
                areaBlah.requestFocusInWindow();
            }
        });
    }
    
    private static final Dimension COLOR_BTN_SIZE = new Dimension(28, 28);
    
    private JPanel createLeftPanel() {
        
        sidePanel = new JPanel();
        sidePanel.setLayout(new BorderLayout());
        
        fgColorBtn = new JButton(Lang.get("tool.color"));
        fgColorBtn.setBackground(areaBlah.getManager().getDefaultProperties().getColorValue(XmlConstants.PROP_COLOR));
        fgColorBtn.setFocusable(false);
        fgColorBtn.setPreferredSize(COLOR_BTN_SIZE);
        fgColorBtn.setMinimumSize(COLOR_BTN_SIZE);
        fgColorBtn.setMaximumSize(COLOR_BTN_SIZE);
        fgColorBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Color cx = JColorChooser.showDialog(getMainForm(), Lang.get("dialog.caption.pickacolor"), 
                        areaBlah.getManager().getDefaultProperties().getColorValue(XmlConstants.PROP_COLOR));
                if (cx != null) {
                    areaBlah.getManager().setDefaulProperty(XmlConstants.PROP_COLOR, cx);
                    fgColorBtn.setBackground(cx);
                }
            }});
        
        bgColorBtn = new JButton(Lang.get("tool.background"));
        bgColorBtn.setBackground(areaBlah.getManager().getDefaultProperties().getColorValue(XmlConstants.PROP_BACKGROUND));
        bgColorBtn.setFocusable(false);
        bgColorBtn.setPreferredSize(COLOR_BTN_SIZE);
        bgColorBtn.setMinimumSize(COLOR_BTN_SIZE);
        bgColorBtn.setMaximumSize(COLOR_BTN_SIZE);
        bgColorBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Color cx = JColorChooser.showDialog(getMainForm(), Lang.get("dialog.caption.pickacolor"), 
                        areaBlah.getManager().getDefaultProperties().getColorValue(XmlConstants.PROP_BACKGROUND));
                if (cx != null) {
                    areaBlah.getManager().setDefaulProperty(XmlConstants.PROP_BACKGROUND, cx);
                    bgColorBtn.setBackground(cx);
                }
            }});

        JToolBar leftPanel = new JToolBar(Lang.get("toolbars.tools"), JToolBar.VERTICAL);
        leftPanel.add(GUIUtils.makeIconBtn("selection", Lang.get("tooltip.selection"), Lang.get("tool.selection"), viewActionFactory.getDefaultContextAction()));
        leftPanel.add(GUIUtils.makeIconBtn("hand", Lang.get("tooltip.hand"), Lang.get("tool.hand"), viewActionFactory.getHandToolAction()));
        leftPanel.add(GUIUtils.makeIconBtn("magnif", Lang.get("tooltip.zoomtool"), Lang.get("tool.zoomtool"), viewActionFactory.getZoomInToolAction()));
        leftPanel.add(GUIUtils.makeIconBtn("line", Lang.get("tooltip.line"), Lang.get("tool.line"), insertActionFactory.getObjectCreateAction(LineElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("polyline", Lang.get("tooltip.polyline"), Lang.get("tool.polyline"), insertActionFactory.getObjectCreateAction(PolyLineElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("rect", Lang.get("tooltip.rect"), Lang.get("tool.rect"), insertActionFactory.getObjectCreateAction(RectElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("oval", Lang.get("tooltip.ellipse"), Lang.get("tool.ellipse"), insertActionFactory.getObjectCreateAction(OvalElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("polygon", Lang.get("tooltip.polygon"), Lang.get("tool.polygon"), insertActionFactory.getObjectCreateAction(PolygonElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("text", Lang.get("tooltip.text"), Lang.get("tool.text"), insertActionFactory.getObjectCreateAction(TextElement.class)));
        leftPanel.add(GUIUtils.makeIconBtn("arrow", Lang.get("tooltip.arrow"), Lang.get("tool.arrow"), insertActionFactory.getObjectCreateAction(ArrowElement.class)));
        leftPanel.add(fgColorBtn);
        leftPanel.add(bgColorBtn);
        
        libElementsContainer = new JPanel();
        libElementsContainer.setLayout(new BorderLayout());
        sidePanel.add(libElementsContainer, BorderLayout.CENTER);
        libElementsContainer.add(new SidePanelFactory(areaBlah.getManager(), this).createPanel(), BorderLayout.CENTER);
        
        sidePanel.add(leftPanel, BorderLayout.WEST);
        
        
        return sidePanel;

    }
    
    private void createScrollBars() {
        vertScrollBar = new JScrollBar(JScrollBar.VERTICAL, 
                -(int)areaBlah.getManager().getPadding().y,
                10, 
                (int)(Manager.MINIMAL_PADDING_Y * areaBlah.getManager().getZoom()),
                (int)(Manager.MAXIMAL_PADDING_Y * areaBlah.getManager().getZoom()));
        vertScrollBar.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                int i = e.getValue();
                Point padding = areaBlah.getManager().getPadding();
                areaBlah.getManager().setPadding(new Point(padding.x, -i));
                areaBlah.getManager().invokeViewChange();
                
            }} );
            
        horizScrollBar = new JScrollBar(JScrollBar.HORIZONTAL, 
                    -(int)areaBlah.getManager().getPadding().x,
                    10, 
                    (int)(Manager.MINIMAL_PADDING_X * areaBlah.getManager().getZoom()),
                    (int)(Manager.MAXIMAL_PADDING_X * areaBlah.getManager().getZoom()));
        horizScrollBar.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                int i = e.getValue();
                Point padding = areaBlah.getManager().getPadding();
                areaBlah.getManager().setPadding(new Point(-i, padding.y));
                areaBlah.getManager().invokeViewChange();
            }} );        
    }
    
    private void createToolBars() {

        fileToolBar = new JToolBar(Lang.get("toolbar.file"), JToolBar.HORIZONTAL);
        editToolBar = new JToolBar(Lang.get("toolbar.edit"), JToolBar.HORIZONTAL);
        formatToolBar = new JToolBar(Lang.get("toolbar.format"), JToolBar.HORIZONTAL);
        groupToolBar = new JToolBar(Lang.get("toolbar.group"), JToolBar.HORIZONTAL);
        
        fileNewBtn = GUIUtils.makeIconBtn("filenew", Lang.get("tooltip.new"), Lang.get("tool.new"), fileActionFactory.getNewAction());
        fileOpenBtn = GUIUtils.makeIconBtn("fileopen", Lang.get("tooltip.open"), Lang.get("tool.open"), fileActionFactory.getOpenAction());
        fileSaveBtn = GUIUtils.makeIconBtn("filesave", Lang.get("tooltip.save"), Lang.get("tool.save"), fileActionFactory.getSaveAction());
        fileSaveAsBtn = GUIUtils.makeIconBtn("filesaveas", Lang.get("tooltip.saveas"), Lang.get("tool.saveas"), fileActionFactory.getSaveAsAction());
        fileToolBar.add(fileNewBtn);
        fileToolBar.add(fileOpenBtn);
        fileToolBar.add(fileSaveBtn);
        fileToolBar.add(fileSaveAsBtn);
        
        editUndoBtn = GUIUtils.makeIconBtn("undo", Lang.get("tooltip.undo"), Lang.get("tool.undo"), editActionFactory.getUndoAction());
        editRedoBtn = GUIUtils.makeIconBtn("redo", Lang.get("tooltip.redo"), Lang.get("tool.redo"), editActionFactory.getRedoAction());
        editCopyBtn = GUIUtils.makeIconBtn("editcopy", Lang.get("tooltip.copy"), Lang.get("tool.copy"), editActionFactory.getCopyAction());
        editCutBtn = GUIUtils.makeIconBtn("editcut", Lang.get("tooltip.cut"), Lang.get("tool.cut"), editActionFactory.getCutAction());
        editPasteBtn = GUIUtils.makeIconBtn("editpaste", Lang.get("tooltip.paste"), Lang.get("tool.paste"), editActionFactory.getPasteAction());
        editDeleteBtn = GUIUtils.makeIconBtn("editdelete", Lang.get("tooltip.delete"), Lang.get("tool.delete"), editActionFactory.getDeleteAction());
        
        editToolBar.add(editUndoBtn);
        editToolBar.add(editRedoBtn);
        editToolBar.add(editCopyBtn);
        editToolBar.add(editCutBtn);
        editToolBar.add(editPasteBtn);
        editToolBar.add(editDeleteBtn);
        
        formatFontFaceCombo = new JComboBox(DefaultArrays.fontList);
        formatFontFaceCombo.addActionListener(formatActionFactory.getSetFontAction(formatFontFaceCombo));
        formatFontFaceCombo.setFocusable(false);
        
        formatFontSizeCombo = new JComboBox(
                new String [] {"8", "10", "11", "12", "14", "16", "18", "20", 
                        "22", "24", "28", "36", "48", "72"});
        formatFontSizeCombo.setPreferredSize(new Dimension(24, 16));
        formatFontSizeCombo.setEditable(true);
        formatFontSizeCombo.setFocusable(false);
        formatFontSizeCombo.addActionListener(formatActionFactory.getSetFontSizeAction(formatFontSizeCombo));
        formatFontSizeCombo.setToolTipText(Lang.get("property.font.size"));
        formatFontSizeCombo.setFocusCycleRoot(false);
        
        formatBoldBtn = new JToggleButton(IconCache.getSystemIcon(XmlConstants.PROP_BOLD), false);
        formatBoldBtn.setToolTipText(Lang.get("property.font.bold"));
        formatBoldBtn.addActionListener(formatActionFactory.getToggleFormatAction(XmlConstants.PROP_BOLD));
        formatBoldBtn.setFocusable(false);
        
        formatItalicBtn = new JToggleButton(IconCache.getSystemIcon("italic"), false);
        formatItalicBtn.setToolTipText(Lang.get("property.font.italic"));
        formatItalicBtn.addActionListener(formatActionFactory.getToggleFormatAction(XmlConstants.PROP_ITALIC));
        formatItalicBtn.setFocusable(false);
        
        formatUnderlineBtn = new JToggleButton(IconCache.getSystemIcon("underline"), false);
        formatUnderlineBtn.setToolTipText(Lang.get("property.font.underline"));
        formatUnderlineBtn.addActionListener(formatActionFactory.getToggleFormatAction(XmlConstants.PROP_UNDERLINE));
        formatUnderlineBtn.setFocusable(false);
        
        formatAlignLeftBtn = new JToggleButton(IconCache.getSystemIcon("alignleft"), false);
        formatAlignLeftBtn.setToolTipText(Lang.get("tooltip.aligntextleft"));
        formatAlignLeftBtn.addActionListener(formatActionFactory.getSetAlignAction(AbstractDevice.ALIGN_LEFT));
        formatAlignLeftBtn.setFocusable(false);
        
        formatAlignCenterBtn = new JToggleButton(IconCache.getSystemIcon("aligncenter"), false);
        formatAlignCenterBtn.setToolTipText(Lang.get("tooltip.aligntextcenter"));
        formatAlignCenterBtn.addActionListener(formatActionFactory.getSetAlignAction(AbstractDevice.ALIGN_CENTER));
        formatAlignCenterBtn.setFocusable(false);
        
        formatAlignRightBtn = new JToggleButton(IconCache.getSystemIcon("alignright"), false);
        formatAlignRightBtn.setToolTipText(Lang.get("tooltip.aligntextright"));
        formatAlignRightBtn.addActionListener(formatActionFactory.getSetAlignAction(AbstractDevice.ALIGN_RIGHT));
        formatAlignRightBtn.setFocusable(false);
        
        formatLineStyleCombo = new JComboBox();
        formatLineStyleCombo.addItem(IconCache.getSystemIcon("lineclear"));
        formatLineStyleCombo.addItem(IconCache.getSystemIcon("linepoints"));
        formatLineStyleCombo.addItem(IconCache.getSystemIcon("linedash"));
        formatLineStyleCombo.addItem(IconCache.getSystemIcon("linedashpoint"));
        formatLineStyleCombo.addActionListener(formatActionFactory.getSetLineStyletAction(formatLineStyleCombo));
        formatLineStyleCombo.setFocusable(false);
        formatLineStyleCombo.setPreferredSize(new Dimension(110, 16));
        
        formatLineWidthSpinner = new JSpinner(new SpinnerNumberModel(1, 0.01, 100, 0.1));
        formatLineWidthSpinner.setFocusable(false);
        formatLineWidthSpinner.setToolTipText(Lang.get("property.linewidth"));
        formatLineWidthSpinner.addChangeListener(formatActionFactory.getSetLineWidthAction(formatLineWidthSpinner));
        formatLineWidthSpinner.setFocusCycleRoot(false);
        
        formatToolBar.add(formatFontFaceCombo);
        formatToolBar.add(formatFontSizeCombo);
        formatToolBar.add(createToolBarSeparator());
        formatToolBar.add(formatBoldBtn);
        formatToolBar.add(formatItalicBtn);
        formatToolBar.add(formatUnderlineBtn);
        formatToolBar.add(createToolBarSeparator());
        formatToolBar.add(formatAlignLeftBtn);
        formatToolBar.add(formatAlignCenterBtn);
        formatToolBar.add(formatAlignRightBtn);
        formatToolBar.add(createToolBarSeparator());
        formatToolBar.add(formatLineStyleCombo);
        formatToolBar.add(formatLineWidthSpinner);
        
        groupGroupBtn = GUIUtils.makeIconBtn("group", Lang.get("menu.format.group"), Lang.get("tool.group"), formatActionFactory.getGroupAction());
        groupSplitBtn = GUIUtils.makeIconBtn("splitgroup", Lang.get("menu.format.split"), Lang.get("tool.split"), formatActionFactory.getSplitGroupAction());
        groupAlignTopBtn = GUIUtils.makeIconBtn("aligntop", Lang.get("menu.format.align.top"), Lang.get("tool.aligntop"), formatActionFactory.getAlignAction(AlignEvent.TOP));
        groupAlignBottomBtn = GUIUtils.makeIconBtn("alignbottom", Lang.get("menu.format.align.bottom"), Lang.get("tool.alignbottom"), formatActionFactory.getAlignAction(AlignEvent.BOTTOM));
        groupAlignLeftBtn = GUIUtils.makeIconBtn("alignleft2", Lang.get("menu.format.align.left"), Lang.get("tool.alignleft"), formatActionFactory.getAlignAction(AlignEvent.LEFT));
        groupAlignRightBtn = GUIUtils.makeIconBtn("alignright2", Lang.get("menu.format.align.right"), Lang.get("tool.alignright"), formatActionFactory.getAlignAction(AlignEvent.RIGHT));
        groupAlignCenterXBtn = GUIUtils.makeIconBtn("aligncenterx", Lang.get("menu.format.align.aligncenterx"), Lang.get("tool.aligncenterx"), formatActionFactory.getAlignAction(AlignEvent.CENTER_X));
        groupAlignCenterYBtn = GUIUtils.makeIconBtn("aligncentery", Lang.get("menu.format.align.aligncentery"), Lang.get("tool.aligncentery"), formatActionFactory.getAlignAction(AlignEvent.CENTER_Y));
        groupSendBackBtn = GUIUtils.makeIconBtn("sendback", Lang.get("menu.format.sendback"), Lang.get("tool.sendback"), formatActionFactory.getSendBackAction());
        groupSendFrontBtn = GUIUtils.makeIconBtn("sendfront", Lang.get("menu.format.bringfront"), Lang.get("tool.bringfront"), formatActionFactory.getSendFrontAction());
        
        groupToolBar.add(groupGroupBtn);
        groupToolBar.add(groupSplitBtn);
        groupToolBar.add(groupSendBackBtn);
        groupToolBar.add(groupSendFrontBtn);
        groupToolBar.add(groupAlignTopBtn);
        groupToolBar.add(groupAlignCenterXBtn);
        groupToolBar.add(groupAlignBottomBtn);
        groupToolBar.add(groupAlignLeftBtn);
        groupToolBar.add(groupAlignCenterYBtn);
        groupToolBar.add(groupAlignRightBtn);
        
    }
    
    private static Dimension separatorSize = new Dimension(5, 16); 
    private JPanel createToolBarSeparator() {
        JPanel separator = new JPanel();
        separator.setMaximumSize(separatorSize);
        separator.setSize(separatorSize);
        return separator;
    }
    
    private MainForm getMainForm() {
        return this;
    }
    
    private void createMenu() {
            
        mainMenu = new JMenuBar();
        
        createInsertMenu(false);
        createPopupMenu();
        
        toolsMenu = new JMenu(Lang.get("menu.tools"));
        
        librariesMenuItem = new JMenuItem(Lang.get("menu.tools.libraries"));
        librariesMenuItem.setIcon(IconCache.getSystemIcon("plain"));
        librariesMenuItem.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                libraryDialog();
            }});
        
        toolsMenu.add(librariesMenuItem);
        
        mainMenu.add(createFileMenu());
        mainMenu.add(createEditMenu());
        mainMenu.add(createViewMenu());
        mainMenu.add(insertMenu);
        mainMenu.add(createFormatMenu());
        mainMenu.add(toolsMenu);
        mainMenu.add(createHelpMenu());
        this.setJMenuBar(mainMenu);
        
    }
    
    public void run() {
        activeWindows++;
        this.setVisible(true);
    }
    
    private JMenu createFileMenu() {
        
        MenuFactory menuFactory = new MenuFactory(Lang.get("menu.file"));
        
        menuFactory.addItem(Lang.get("menu.file.new"), "filenew", fileActionFactory.getNewAction(), KeyEvent.VK_N, CTRL);
        menuFactory.addItem(Lang.get("menu.file.save"), "filesave", fileActionFactory.getSaveAction(), KeyEvent.VK_S, CTRL);
        menuFactory.addItem(Lang.get("menu.file.saveas"), "filesaveas", fileActionFactory.getSaveAsAction(), KeyEvent.VK_S, CTRL + SHIFT);
        menuFactory.addItem(Lang.get("menu.file.open"), "fileopen", fileActionFactory.getOpenAction(), KeyEvent.VK_O, CTRL);
        menuFactory.addSeparator();
        saveIntoLibraryMenuItem = menuFactory.addItem(Lang.get("menu.file.saveintolibrary"), fileActionFactory.getSaveIntoLibraryAction());
        menuFactory.addItem(Lang.get("menu.file.saveaslibrary"), fileActionFactory.getSaveAsNewLibraryAction());
        menuFactory.addSeparator();
        menuFactory.addItem(Lang.get("menu.file.exit"), fileActionFactory.getExitAction(), KeyEvent.VK_X, ALT);
        
        return menuFactory.getMenu();
    }
    
    public void closeWindow() {
        if (areaBlah.getManager().isDocumentChanged()) {
            int result = JOptionPane.showConfirmDialog(this,
                    Lang.get("msg.documentchanged"),
                    Lang.get("msg.caption.caution"), 
                    JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (result == JOptionPane.YES_OPTION)
                new FileMenuActionFactory(areaBlah.getManager(), this).getSaveAction().actionPerformed(null);
        }
        try {            
            areaBlah.getManager().saveLibConfig();
        } catch (Exception e) {
            e.printStackTrace();
            Log.warn(Lang.get("log.unabletosavelibconfig"));
        }
        this.setVisible(false);
        activeWindows--;
        if (activeWindows == 0) System.exit(0);
    }

    private JMenu createEditMenu() {
        MenuFactory menuFactory = new MenuFactory(Lang.get("menu.edit"));
        
        undoMenuItem = menuFactory.addItem(Lang.get("menu.edit.undo"), "undo", editActionFactory.getUndoAction(), KeyEvent.VK_Z, CTRL);
        redoMenuItem = menuFactory.addItem(Lang.get("menu.edit.redo"), "redo", editActionFactory.getRedoAction(), KeyEvent.VK_Y, CTRL);
        menuFactory.addSeparator();
        cutMenuItem = menuFactory.addItem(Lang.get("menu.edit.cut"), "editcut", editActionFactory.getCutAction(), KeyEvent.VK_X, CTRL);
        copyMenuItem = menuFactory.addItem(Lang.get("menu.edit.copy"), "editcopy", editActionFactory.getCopyAction(), KeyEvent.VK_C, CTRL);
        pasteMenuItem = menuFactory.addItem(Lang.get("menu.edit.paste"), "editpaste", editActionFactory.getPasteAction(), KeyEvent.VK_V, CTRL);
        menuFactory.addSeparator();
        deleteMenuItem = menuFactory.addItem(Lang.get("menu.edit.delete"), "editdelete", editActionFactory.getDeleteAction(), KeyEvent.VK_DELETE, 0);
        menuFactory.addSeparator();
        selectAllMenuItem = menuFactory.addItem(Lang.get("menu.edit.selectall"), editActionFactory.getSelectAllAction(), KeyEvent.VK_A, CTRL);
        selectNoneMenuItem = menuFactory.addItem(Lang.get("menu.edit.selectnone"), editActionFactory.getSelectNoneAction(), KeyEvent.VK_A, CTRL + SHIFT);
        
        return menuFactory.getMenu();
    }
    
    private JMenu createFormatMenu() {
        MenuFactory menuFactory = new MenuFactory(Lang.get("menu.format"));
        
        menuFactory.addItem(Lang.get("menu.format.textstyle"), formatActionFactory.getTextPropertiesAction());
        menuFactory.addItem(Lang.get("menu.format.colorstyle"), formatActionFactory.getColorLinesPropertiesAction());
        //menuFactory.addSeparator();
        //menuFactory.addItem(Lang.get("menu.format.document"), formatActionFactory.getDocumentPropAction());
        propertiesMenuItem = menuFactory.addItem(Lang.get("menu.format.properties"), formatActionFactory.getPropertiesAction(), KeyEvent.VK_P, SHIFT);
        
        menuFactory.addSeparator();
        groupMenuItem = menuFactory.addItem(Lang.get("menu.format.group"), "group", formatActionFactory.getGroupAction(), KeyEvent.VK_G, CTRL);
        splitGroupMenuItem = menuFactory.addItem(Lang.get("menu.format.split"), "splitgroup", formatActionFactory.getSplitGroupAction(), KeyEvent.VK_G, CTRL + SHIFT);
        menuFactory.addSeparator();
        
        MenuFactory alignMenuFactory = new MenuFactory(Lang.get("menu.format.align"));
        
        alignTopMenuItem = alignMenuFactory.addItem(Lang.get("menu.format.align.top"), "aligntop", formatActionFactory.getAlignAction(AlignEvent.TOP));
        alignBottomMenuItem = alignMenuFactory.addItem(Lang.get("menu.format.align.bottom"), "alignbottom", formatActionFactory.getAlignAction(AlignEvent.BOTTOM));
        alignCenterXMenutItem = alignMenuFactory.addItem(Lang.get("menu.format.align.centerx"), "aligncenterx", formatActionFactory.getAlignAction(AlignEvent.CENTER_X));
        alignMenuFactory.addSeparator();
        alignLeftMenuItem = alignMenuFactory.addItem(Lang.get("menu.format.align.left"), "alignleft2", formatActionFactory.getAlignAction(AlignEvent.LEFT));
        alignRightMenuItem = alignMenuFactory.addItem(Lang.get("menu.format.align.right"),"alignright2", formatActionFactory.getAlignAction(AlignEvent.RIGHT));
        alignCenterYMenutItem = alignMenuFactory.addItem(Lang.get("menu.format.align.centery"), "aligncentery", formatActionFactory.getAlignAction(AlignEvent.CENTER_Y));
        
        alignMenuFactory.getMenu().setIcon(IconCache.getSystemIcon("alignbase"));
        menuFactory.addMenuFactory(alignMenuFactory);
        menuFactory.addSeparator();
        
        sendBackMenuItem = menuFactory.addItem(Lang.get("menu.format.sendback"), "sendback", formatActionFactory.getSendBackAction());
        sendFrontMenuItem = menuFactory.addItem(Lang.get("menu.format.bringfront"), "sendfront", formatActionFactory.getSendFrontAction());
        

        return menuFactory.getMenu();
    }
    
    private JMenu createViewMenu() {
        
        MenuFactory viewMenuFactory = new MenuFactory(Lang.get("menu.view"));
        MenuFactory zoomMenuFactory = new MenuFactory(Lang.get("menu.view.zoom"));
        
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.zoomin"), viewActionFactory.getZoomInAction(), KeyEvent.VK_PAGE_UP, CTRL);
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.zoomout"), viewActionFactory.getZoomOutAction(), KeyEvent.VK_PAGE_DOWN, CTRL);
        zoomMenuFactory.addSeparator();
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.33"), viewActionFactory.getZoomAction(0.33));
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.50"), viewActionFactory.getZoomAction(0.50));
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.75"), viewActionFactory.getZoomAction(0.75));
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.100"), viewActionFactory.getZoomAction(1), KeyEvent.VK_HOME, CTRL);
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.150"), viewActionFactory.getZoomAction(1.5));
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.200"), viewActionFactory.getZoomAction(2));
        zoomMenuFactory.addSeparator();
        zoomMenuFactory.addItem(Lang.get("menu.view.zoom.zoomother"), viewActionFactory.getZoomOtherAction());
        
        zoomMenuFactory.getMenu().setIcon(IconCache.getSystemIcon("plain"));
        viewMenuFactory.addMenuFactory(zoomMenuFactory);
        viewMenuFactory.addSeparator();
        viewMenuFactory.addItem(Lang.get("menu.view.zoomintool"), "magnif", viewActionFactory.getZoomInToolAction());
        viewMenuFactory.addItem(Lang.get("menu.view.handtool"), "hand", viewActionFactory.getHandToolAction());
        viewMenuFactory.addSeparator();
        viewMenuFactory.addItem(Lang.get("menu.view.centerview"), viewActionFactory.getcenterViewAction(), KeyEvent.VK_HOME, 0);
        viewMenuFactory.addSeparator();
        showGridMenuItem = viewMenuFactory.addCheckItem(Lang.get("menu.view.showgrid"), viewActionFactory.getShowGridAction());
        snapToGridMenuItem = viewMenuFactory.addCheckItem(Lang.get("menu.view.snaptogrid"), viewActionFactory.getSnapToGridAction());
        viewMenuFactory.addItem(Lang.get("menu.view.gridproperties"), viewActionFactory.getGridPropertiesAction());
        viewMenuFactory.addSeparator();
        snapToGuideLinesMenuItem  = viewMenuFactory.addCheckItem(Lang.get("menu.view.snaptoguidelines"), viewActionFactory.getSnapToGuideLines());
        snapToElementsMenuItem  = viewMenuFactory.addCheckItem(Lang.get("menu.view.snaptoelements"), viewActionFactory.getSnapToElementsAction());
        snapTechMenuItem = viewMenuFactory.addCheckItem(Lang.get("menu.view.snaptech"), viewActionFactory.getSnapTechAction());
        
        showGuideLinesMenuItem = viewMenuFactory.addCheckItem(Lang.get("menu.view.showguidelines"), viewActionFactory.getShowGuideLinesAction());
        
        
        return viewMenuFactory.getMenu();
    }
        
    public void createInsertMenu(boolean refresh) {
    
        MenuFactory menuFactory;
        
        if (!refresh) {
            menuFactory = new MenuFactory(Lang.get("menu.insert"));
        } else {
            insertMenu.removeAll();
            menuFactory = new MenuFactory(insertMenu);
        }
        
        insertMenu = menuFactory.getMenu();
        
        menuFactory.addItem(Lang.get("menu.insert.line"), "line", insertActionFactory.getObjectCreateAction(LineElement.class));
        menuFactory.addItem(Lang.get("menu.insert.polyline"), "polyline", insertActionFactory.getObjectCreateAction(PolyLineElement.class));
        menuFactory.addItem(Lang.get("menu.insert.rect"), "rect", insertActionFactory.getObjectCreateAction(RectElement.class));
        menuFactory.addItem(Lang.get("menu.insert.ellipse"), "oval", insertActionFactory.getObjectCreateAction(OvalElement.class));
        menuFactory.addItem(Lang.get("menu.insert.text"), "text", insertActionFactory.getObjectCreateAction(TextElement.class));
        menuFactory.addItem(Lang.get("menu.insert.polygon"), "polygon", insertActionFactory.getObjectCreateAction(PolygonElement.class));
        menuFactory.addItem(Lang.get("menu.insert.arrow"), "arrow", insertActionFactory.getObjectCreateAction(ArrowElement.class));
        
        ArrayList libraries = areaBlah.getManager().getLibraries();
        if (libraries.size() > 0) 
            menuFactory.addSeparator();
        for (int i = 0; i < libraries.size(); i++) {
            final String libName = (String)libraries.get(i);
            MenuFactory subMenuFactory = new MenuFactory(libName);
            subMenuFactory.getMenu().setIcon(IconCache.getSystemIcon("plain"));
            menuFactory.addMenuFactory(subMenuFactory);
            
            ArrayList elements = areaBlah.getManager().getElementsInLibrary(libName);
            for (int j = 0; j < elements.size(); j++) {
                final String eltName = (String)elements.get(j);
                subMenuFactory.addItem(eltName, insertActionFactory.getLibElementCreateAction(libName, eltName));
            }
            
            if (elements.size() == 0) {
                JMenuItem mnuItem = subMenuFactory.addItem(Lang.get("menu.insert.noelements"), null);
                mnuItem.setEnabled(false);
            }
        }    
    }
    
    private JMenu createHelpMenu() {
        
        MenuFactory helpMenu = new MenuFactory(Lang.get("menu.help"));
        helpMenu.addItem(Lang.get("menu.help.intro"), new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                HelpDialog dlg = new HelpDialog("intro");
                dlg.setVisible(true);
            }});
        helpMenu.addSeparator();
        helpMenu.addItem(Lang.get("menu.help.about"), new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                HelpDialog dlg = new HelpDialog("about");
                dlg.setVisible(true);
            }});
        return helpMenu.getMenu();
    }
    
    private void createPopupMenu() {

        contextMenu = new JPopupMenu();
        cntxCopyMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.copy"), "editcopy", editActionFactory.getCopyAction(), KeyEvent.VK_C, CTRL);
        cntxCutMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.cut"), "editcut", editActionFactory.getCutAction(), KeyEvent.VK_X, CTRL);
        cntxPasteMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.paste"), "editpaste", editActionFactory.getPasteAction(), KeyEvent.VK_V, CTRL);
        cntxDeleteMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.delete"), "editdelete", editActionFactory.getDeleteAction(), KeyEvent.VK_DELETE, 0);
        cntxUndoMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.undo"), "undo", editActionFactory.getUndoAction(), KeyEvent.VK_Z, CTRL);
        cntxRedoMenuItem = GUIUtils.createMenuItem(Lang.get("menu.edit.redo"), "redo", editActionFactory.getRedoAction(), KeyEvent.VK_Y, CTRL);
        
        cntxSendBackMenuItem = GUIUtils.createMenuItem(Lang.get("menu.format.sendback"), "sendback", formatActionFactory.getSendBackAction());
        cntxSendFrontMenuItem = GUIUtils.createMenuItem(Lang.get("menu.format.bringfront"), "sendfront", formatActionFactory.getSendFrontAction());
        cntxGroupMenuItem = GUIUtils.createMenuItem(Lang.get("menu.format.group"), "group", formatActionFactory.getGroupAction(), KeyEvent.VK_G, CTRL);
        cntxSplitMenuItem = GUIUtils.createMenuItem(Lang.get("menu.format.split"), "splitgroup", formatActionFactory.getSplitGroupAction(), KeyEvent.VK_G, SHIFT + CTRL);
        cntxPropertiesMenuItem = GUIUtils.createMenuItem(Lang.get("menu.format.properties"), new FormatMenuActionFactory(areaBlah.getManager(), this).getPropertiesAction(), KeyEvent.VK_P, SHIFT);
        
        contextMenu.add(cntxUndoMenuItem);
        contextMenu.add(cntxRedoMenuItem);
        contextMenu.addSeparator();
        contextMenu.add(cntxCopyMenuItem);
        contextMenu.add(cntxCutMenuItem);
        contextMenu.add(cntxPasteMenuItem);
        contextMenu.add(cntxDeleteMenuItem);
        contextMenu.addSeparator();
        contextMenu.add(cntxGroupMenuItem);
        contextMenu.add(cntxSplitMenuItem);
        contextMenu.addSeparator();
        contextMenu.add(cntxSendBackMenuItem);
        contextMenu.add(cntxSendFrontMenuItem);
        contextMenu.addSeparator();
        contextMenu.add(cntxPropertiesMenuItem);
    }
    
    /**
     * Invkoes library management dialog 
     *
     */
    private void libraryDialog() {
        LibrariesForm frm = new LibrariesForm(getMainForm(), areaBlah.getManager());
        frm.setVisible(true);
        refreshLibrariesElements();     
    }
    
    /**
     * Refreshes library elements in menu, etc.
     *
     */
    public void refreshLibrariesElements() {
        createInsertMenu(true);
        areaBlah.getManager().invokeMenuRefresh();
        try {
            libElementsContainer.remove(0);
            libElementsContainer.add(new SidePanelFactory(areaBlah.getManager(), this).createPanel(), BorderLayout.CENTER);
        } catch (Exception e) {
        }
        
    }    
    

    /**
     * Refreshes menu enabled state
     *
     */
    private final class MenuRefreshInvoker extends UIInvoker {

        public Object invoke(Object arg) {
            Manager manager = (Manager)arg;
            Abilities abilities = manager.getAbilities();
            
            boolean canUndo = abilities.isAble("undo");
            boolean canRedo = abilities.isAble("redo");
            boolean canGroupElements = abilities.isAble("groupElements");
            boolean canSplitElements = abilities.isAble("splitElements");
            boolean canAlignSelection = abilities.isAble("alignSelection");
            boolean canSendInAnyDirection = abilities.isAble("sendInAnyDirection");
            boolean canCopy = abilities.isAble("copy");
            boolean canCut = abilities.isAble("cut");
            boolean canPaste = abilities.isAble("paste");
            boolean canDelete = abilities.isAble("delete");
            boolean canShowPropertyDialog = abilities.isAble("showPropertyDialog");

            copyMenuItem.setEnabled(canCopy);
            cntxCopyMenuItem.setEnabled(canCopy);
            editCopyBtn.setEnabled(canCopy);
            
            cutMenuItem.setEnabled(canCut);
            cntxCutMenuItem.setEnabled(canCut);
            editCutBtn.setEnabled(canCut);
            
            deleteMenuItem.setEnabled(canDelete);
            cntxDeleteMenuItem.setEnabled(canDelete);
            editDeleteBtn.setEnabled(canDelete);
            
            propertiesMenuItem.setEnabled(canShowPropertyDialog);
            cntxPropertiesMenuItem.setEnabled(canShowPropertyDialog);
            
            pasteMenuItem.setEnabled(canPaste);
            cntxPasteMenuItem.setEnabled(canPaste);
            editPasteBtn.setEnabled(canPaste);
            
            undoMenuItem.setEnabled(canUndo);
            cntxUndoMenuItem.setEnabled(canUndo);
            editUndoBtn.setEnabled(canUndo);
            
            redoMenuItem.setEnabled(canRedo);
            cntxRedoMenuItem.setEnabled(canRedo);
            editRedoBtn.setEnabled(canRedo);
            
            groupMenuItem.setEnabled(canGroupElements);
            splitGroupMenuItem.setEnabled(canSplitElements);
            cntxGroupMenuItem.setEnabled(canGroupElements);
            cntxSplitMenuItem.setEnabled(canSplitElements);
            groupGroupBtn.setEnabled(canGroupElements);
            groupSplitBtn.setEnabled(canSplitElements);
            
            saveIntoLibraryMenuItem.setEnabled(abilities.isAble("saveIntoLibrary"));
            selectAllMenuItem.setEnabled(abilities.isAble("selectAll"));
            selectNoneMenuItem.setEnabled(abilities.isAble("selectNone"));
            sendFrontMenuItem.setEnabled(canSendInAnyDirection);
            sendBackMenuItem.setEnabled(canSendInAnyDirection);
            cntxSendFrontMenuItem.setEnabled(canSendInAnyDirection);
            cntxSendBackMenuItem.setEnabled(canSendInAnyDirection);
            groupSendBackBtn.setEnabled(canSendInAnyDirection);
            groupSendFrontBtn.setEnabled(canSendInAnyDirection);
            
            alignTopMenuItem.setEnabled(canAlignSelection);
            alignBottomMenuItem.setEnabled(canAlignSelection);
            alignLeftMenuItem.setEnabled(canAlignSelection);
            alignRightMenuItem.setEnabled(canAlignSelection);
            alignCenterXMenutItem.setEnabled(canAlignSelection);
            alignCenterYMenutItem.setEnabled(canAlignSelection);
            
            groupAlignBottomBtn.setEnabled(canAlignSelection);
            groupAlignTopBtn.setEnabled(canAlignSelection);
            groupAlignLeftBtn.setEnabled(canAlignSelection);
            groupAlignRightBtn.setEnabled(canAlignSelection);
            groupAlignCenterXBtn.setEnabled(canAlignSelection);
            groupAlignCenterYBtn.setEnabled(canAlignSelection);
            

            snapToGridMenuItem.setState(manager.isSnapToGrid());
            snapToGuideLinesMenuItem.setState(manager.isSnapToGuideLines());
            snapToElementsMenuItem.setState(manager.isSnapToElements());
            snapTechMenuItem.setState(manager.isSnapTech());
            showGridMenuItem.setState(manager.isShowGrid());
            showGuideLinesMenuItem.setState(manager.isShowGuideLines());

            return null;
        }
    }
    
    /**
     * Invokes property dialog
     *
     */
    private static final class PropertyDialogInvoker extends UIInvoker {

        public PropertyDialogInvoker(MainForm form) {
            super(form);
        }

        public Object invoke(Object arg) {
            PropertyDialog dlg = new PropertyDialog(getParentFrame(), (Properties)arg);
            dlg.runForm();
            return dlg.getChanges();
        }
    }
    
    /**
     * Invokes context menu
     *
     */
    private final class ContextMenuInvoker extends UIInvoker {
        
        public ContextMenuInvoker(MainForm form) {
            super(form);
        }

        public Object invoke(Object arg) {
            java.awt.Point position = (java.awt.Point)arg; 
            contextMenu.show(areaBlah, position.x, position.y);
            return null;
        }
    }
    
    /**
     * Invokes refresh of view
     */
    private final class ViewChangeInvoker extends UIInvoker {
        
        public ViewChangeInvoker(MainForm form) {
            super(form);
        }

        public Object invoke(Object arg) {
            MainForm mform = (MainForm)getParentFrame();
            Manager man = (Manager)arg;
            Point pad = man.getPadding();
            mform.vertScrollBar.setValue(-(int)(pad.y));
            mform.horizScrollBar.setValue(-(int)pad.x);
            
            
            mform.vertScrollBar.setMinimum((int)(Manager.MINIMAL_PADDING_Y * man.getZoom()));
            mform.vertScrollBar.setMaximum((int)(Manager.MAXIMAL_PADDING_Y * man.getZoom()));
             
            mform.horizScrollBar.setMinimum((int)(Manager.MINIMAL_PADDING_X * man.getZoom()));
            mform.horizScrollBar.setMaximum((int)(Manager.MAXIMAL_PADDING_X * man.getZoom()));
            

            mform.horizRuler.repaint();
            mform.vertRuler.repaint();
            
            mform.areaBlah.repaint();
            
            contextInfoLabel.setText(man.getContextInfo());
            zoomInfoLabel.setText(Lang.get("application.statusbar.zoom", round(man.getZoom() * 100)));
            
            return null;   
        }
    }
    
    /**
     * Invokes dialogs after few elements hasnt been found
     *
     */
    private final class NotFoundElementsInvoker extends UIInvoker {
        
        public NotFoundElementsInvoker(MainForm form) {
            super(form);
        }

        /**
         * @return true in case there's no chance to open file
         */
        public Object invoke(Object arg) {
            
            String eltString = GUIUtils.getListOfMissingDependencies((ArrayList)arg);
            
            int result = JOptionPane.showConfirmDialog(getParentFrame(),
                    Lang.get("msg.missingelements", eltString.toString()),
                    Lang.get("msg.caption.problem"),
                    JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE);
            
            if (result == JOptionPane.YES_OPTION) {
                libraryDialog();
            }
            return new Boolean(result == JOptionPane.NO_OPTION);
        }
    }
    
    /**
     * Invokes refresh after change of mouse position
     */
    private final class MouseChangeInvoker extends UIInvoker {
        
        public MouseChangeInvoker(MainForm form) {
            super(form);
        }

        public Object invoke(Object arg) {
            MainForm mform = (MainForm)getParentFrame();
            
            mform.horizRuler.repaint();
            mform.vertRuler.repaint();
            
            Point position = areaBlah.getManager().getMousePosition();
            if (position != null) mousePositionLabel.setText(round(position.x) +  ", " + round(position.y));
            else mousePositionLabel.setText("");
            
            contextInfoLabel.setText(areaBlah.getManager().getContextInfo());
            
            return null;   
        }
        
    }
    
    
    private final class DefaultPropertyChangeInvoker extends UIInvoker {

        public DefaultPropertyChangeInvoker(MainForm form) {
            super(form);
        }
        
        public Object invoke(Object arg) {
            Properties properties = (Properties)arg;
            fgColorBtn.setBackground(properties.getColorValue(XmlConstants.PROP_COLOR));
            bgColorBtn.setBackground(properties.getColorValue(XmlConstants.PROP_BACKGROUND));
            formatBoldBtn.setSelected(properties.getBooleanValue(XmlConstants.PROP_BOLD));
            formatItalicBtn.setSelected(properties.getBooleanValue(XmlConstants.PROP_ITALIC));
            formatUnderlineBtn.setSelected(properties.getBooleanValue(XmlConstants.PROP_UNDERLINE));
            
            formatAlignLeftBtn.setSelected(properties.getIntValue(XmlConstants.PROP_ALIGN) == AbstractDevice.ALIGN_LEFT);
            formatAlignCenterBtn.setSelected(properties.getIntValue(XmlConstants.PROP_ALIGN) == AbstractDevice.ALIGN_CENTER);
            formatAlignRightBtn.setSelected(properties.getIntValue(XmlConstants.PROP_ALIGN) == AbstractDevice.ALIGN_RIGHT);
            
            int lineIndex = properties.getIntValue(XmlConstants.PROP_LINE_STYLE) - 1;
            if (lineIndex != formatLineStyleCombo.getSelectedIndex())
                formatLineStyleCombo.setSelectedIndex(lineIndex);
            
            
            String fontSize = Integer.toString(properties.getIntValue(XmlConstants.PROP_FONT_SIZE)); 
            if (!fontSize.equals(formatFontSizeCombo.getSelectedItem()))
                formatFontSizeCombo.setSelectedItem(fontSize);
                
            formatLineWidthSpinner.setValue(properties.getProperty(XmlConstants.PROP_LINE_WIDTH).getValue());
            
            String fontFace = properties.getStringValue(XmlConstants.PROP_FONT_FACE);
            for (int i = 0; i < DefaultArrays.fontList.length; i++)
                if (DefaultArrays.fontList[i].equals(fontFace)) {
                    if (i != formatFontFaceCombo.getSelectedIndex())
                        formatFontFaceCombo.setSelectedIndex(i);
                    break;
                }
            
            return null;
        }
        
    }   
    
    private String round(double value) {
        return Double.toString(Math.rint((value * DECIMAL_CORRECTION)) / DECIMAL_CORRECTION);
    }

    public File getLastUsedPath() {
        return lastUsedPath;
    }

    public void setLastUsedPath(File lastUsedPath) {
        if (lastUsedPath.isFile()) lastUsedPath = new File(lastUsedPath.getAbsolutePath());
        this.lastUsedPath = lastUsedPath;
    }
}