/*
 * Created on Jul 2, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;

import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.system.GuideLine;
import diagnose.utils.DPIUtils;

/**
 * Represents simple ruler
 */
public class Ruler extends JComponent {

    private static final long serialVersionUID = 5872807763211884422L;
    
    public static final int VERTICAL = 1;
    public static final int HORIZONTAL = 2;
    
    private Manager manager;
    
    private GuideLine guideLine = null;
    
    private int orientation;
    
    private Point guideLinePosition;
    
    public Ruler(Manager manager, int orientation) {
        super();
        this.orientation = orientation;
        this.manager = manager;
        this.addMouseMotionListener(new MouseMotionEvents());
        this.addMouseListener(new MouseEvents());
        if (orientation == HORIZONTAL) setPreferredSize(new Dimension(0, 20));
        else setPreferredSize(new Dimension(20, 0));
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D gr = (Graphics2D)g;
        
        int height = getHeight();
        int width = getWidth();
        
        gr.setColor(getBackground());
        gr.fillRect(0, 0, width, height);

        gr.setColor(getForeground());
        gr.setFont(new Font("SansSerif", Font.PLAIN, 10));
        
        
        if (orientation == HORIZONTAL) gr.drawLine(0, height - 1, width, height - 1);
        else gr.drawLine(width - 1, 0, width - 1, height);
        
        diagnose.utils.geometry.Point padding = manager.getPadding();

        double stepMM = Math.round(getStep(manager.getZoom())); // step in milimeters 
        double step = DPIUtils.milimetersToPixels(stepMM) * manager.getZoom();
        
        
        if (orientation == HORIZONTAL) {
            double paddingStart = -(Math.floor(padding.x / manager.getZoom() / stepMM) * stepMM);
            double firstIdent = Math.ceil(DPIUtils.milimetersToPixels(padding.x)) % step;
            
            int i = (firstIdent > 0 ? -1 : 0);
            while (i * step + firstIdent < width) {
                
                int pos = (int)(i * step + firstIdent);
      
                // draws big step
                gr.drawString(Integer.toString((int)(paddingStart + i * stepMM)), pos + 2, height - 8);
                gr.drawLine(pos, height, pos, height - 10);
                i++;
                
                // draws ten small sub steps
                double subStep = step / 10;
                for (int j = 1; j < 10; j++) {
                    int xPos = (int)(pos + subStep * j);
                    g.drawLine(xPos, height, xPos, height - 5);
                    if (j == 5) g.drawLine(xPos, height, xPos, height - 8);
                }
            }
        
        } else { /////////// VERTICAL //////////////
            
            double paddingStart = -(Math.floor(padding.y / manager.getZoom() / stepMM) * stepMM);
            double firstIdent = Math.ceil(DPIUtils.milimetersToPixels(padding.y)) % step;
            
            int i = (firstIdent > 0 ? -1 : 0);
            while (i * step + firstIdent < height) {
                
                int pos = (int)(i * step + firstIdent);
                
                
                // draws big step
                gr.rotate(-Math.PI / 2, pos, 0);

                String text = Integer.toString((int)(paddingStart + i * stepMM));
                gr.drawString(text, 0, -pos + 10);
                gr.rotate(Math.PI / 2, pos, 0);
                gr.drawLine(width, pos, width - 10, pos);
                i++;
                
                // draws ten small sub steps
                double subStep = step / 10;
                for (int j = 1; j < 10; j++) {
                    int yPos = (int)(pos + subStep * j);
                    g.drawLine(width, yPos, width - 5, yPos);
                    if (j == 5) g.drawLine(width, yPos, width - 8, yPos); 
                }
            }
        }
        
        gr.setColor(Color.BLUE);
        if ((orientation == HORIZONTAL) && (guideLine != null)) gr.drawLine(0, guideLinePosition.y, width, guideLinePosition.y);
        if ((orientation == VERTICAL) && (guideLine != null)) gr.drawLine(guideLinePosition.x, 0, guideLinePosition.x, height);
        
        diagnose.utils.geometry.Point position = manager.getMousePosition();
        if (position != null) {
            gr.setColor(Color.RED);
            position = (diagnose.utils.geometry.Point)position.clone();
            
            position.multiply(manager.getZoom());
            position.x += manager.getPadding().x;
            position.y += manager.getPadding().y;
            
            position = DPIUtils.milimetersToPixels(position);
            
            
            if (orientation == HORIZONTAL) gr.drawLine((int)position.x, 0, (int)position.x, height);
            else g.drawLine(0, (int)position.y, width, (int)position.y);
        }
    }    

    
    private final class MouseMotionEvents implements MouseMotionListener {

        public void mouseDragged(MouseEvent e) {
            if (e.getButton() == 0) {
                guideLinePosition = e.getPoint();
                
                diagnose.utils.geometry.Point areaPoint = convertPositionToDrawUnits(guideLinePosition, manager);
                
                boolean isVisible = false;
                if ((orientation == HORIZONTAL) && (guideLinePosition.y > getHeight())) isVisible = true;
                if ((orientation == HORIZONTAL) && (guideLinePosition.y <= getHeight())) isVisible = false;
                if ((orientation == VERTICAL) && (guideLinePosition.x > getWidth())) isVisible = true;
                if ((orientation == VERTICAL) && (guideLinePosition.x <= getWidth())) isVisible = false;
                
                if (isVisible && (!manager.hasGuideLine(guideLine)))
                    manager.getDocument().addGuideLine(guideLine);
                
                if (!isVisible && (manager.hasGuideLine(guideLine)))
                    manager.getDocument().removeGuideLine(guideLine);
                
                guideLine.setControlPoint(0, areaPoint);
                
                
                manager.repaint();
                repaint();
            }
        }

        public void mouseMoved(MouseEvent e) {
        }        
    }
    
    private final class MouseEvents implements MouseListener {

        public void mouseClicked(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
            guideLinePosition = e.getPoint();
            diagnose.utils.geometry.Point areaPoint = convertPositionToDrawUnits(guideLinePosition, manager);
            guideLine = new GuideLine(orientation == HORIZONTAL ? GuideLine.HORIZONTAL : GuideLine.VERTICAL, areaPoint);
            
            manager.addGuideLine(guideLine);
            manager.repaint();
            
        }

        public void mouseReleased(MouseEvent e) {
            if ((orientation == HORIZONTAL) && (guideLinePosition.y < getHeight())) guideLine = null;
            if ((orientation == VERTICAL) && (guideLinePosition.x < getWidth())) guideLine = null;
            repaint();
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {

        }
        
    }

    public int getOrientation() {
        return orientation;
    }

    public void setOrientation(int orientation) {
        this.orientation = orientation;
    } 
    
    /**
     * @return "ideal" value of distence for the biggest step
     * in respect to zoom
     */
    protected static double getStep(double zoom) {
        double pv = (1 / zoom) * 100;
        long logCorrection = Math.round(Math.log10(pv));
        double logCorrection2 = Math.pow(10, logCorrection);
        double step = Math.ceil(pv / logCorrection2) * logCorrection2;
        
        return step / 10;
    }
    
    /**
     * Generates view correction between GUI and itnernal logic
     */
    protected diagnose.utils.geometry.Point convertPositionToDrawUnits(Point position, Manager manager) {
        
        Point px = (Point)position.clone(); 
        px.x -= getWidth();
        px.y -= getHeight();
        
        diagnose.utils.geometry.Point result = DPIUtils.pixelsToMilimeters(px);
        diagnose.utils.geometry.Point padding = (diagnose.utils.geometry.Point)manager.getPadding().clone();
        padding.multiply(-1);
        result.translate(padding);
        result.multiply(1 / manager.getZoom());
        return result;
    }
}