/*
 * Created on Jun 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import diagnose.utils.GUIUtils;
import diagnose.utils.Lang;

/**
 * Creates form for library selection
 */
public class SelectLibraryForm extends JDialog {

    private static final long serialVersionUID = 3063752553049021360L;
    
    protected static final int OK = 1;
    protected static final int CANCEL = 2;
    
    protected JComboBox selectionCombo;
    protected int finalState;
    
    /**
     * @return name of selected library otherwise null
     * @param libraries - arraylist of library names
     * 
     */
    public static String showForm(ArrayList libraries) {
        SelectLibraryForm form = new SelectLibraryForm(libraries.toArray());
        form.setVisible(true);
        String resultValue = form.getSelectedValue();
        
        return resultValue;
    }
    
    protected SelectLibraryForm(Object [] libraries) {

        this.setTitle(Lang.get("dialog.sellib.caption"));
        this.getContentPane().setLayout(new BorderLayout());
        this.setSize(200, 80);
        this.setResizable(false);
        this.setModal(true);
       
        GridLayout layout = new GridLayout(2, 2);
        layout.setHgap(8);
        layout.setVgap(8);
        JPanel mainPanel = new JPanel(layout);
         
        mainPanel.add(new JLabel(Lang.get("dialog.sellib.library")));
        selectionCombo = new JComboBox(libraries);
        mainPanel.add(selectionCombo);
        
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
			    finalState = CANCEL;
			    setVisible(false);
			}});
        
        JButton okBtn = new JButton(Lang.get("dialog.button.ok"));
        okBtn.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                finalState = OK;
                setVisible(false);
            } });
        
        JButton cancelBtn = new JButton(Lang.get("dialog.button.cancel"));
        cancelBtn.addActionListener(new CancelActionListener());
        
        mainPanel.add(okBtn);
        mainPanel.add(cancelBtn);
        
        JPanel contentPanel = new JPanel(new BorderLayout());
        contentPanel.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
        contentPanel.add(mainPanel, BorderLayout.CENTER);
        contentPanel.add(mainPanel, BorderLayout.SOUTH);
        this.getContentPane().add(contentPanel);
        this.pack();
        
        GUIUtils.centerWindow(this);
        GUIUtils.setEscapeListener(this, new CancelActionListener());
    }
    
    public String getSelectedValue() {
        return (finalState == OK ? (String)selectionCombo.getSelectedItem() : null);
    }
    
    private final class CancelActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            finalState = CANCEL;
            setVisible(false);
        }
    }
}