/*
 * Created on Jun 28, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import diagnose.graphicengine.Export;
import diagnose.graphicengine.Manager;
import diagnose.graphicengine.elements.ComposedElement;
import diagnose.graphicengine.libraries.Library;
import diagnose.gui.DefaultFileFilter;
import diagnose.gui.MainForm;
import diagnose.gui.SelectLibraryForm;
import diagnose.utils.CursorFactory;
import diagnose.utils.Lang;
import diagnose.utils.serialization.XmlConstants;

/**
 * Creates actions for file menu
 */
public class FileMenuActionFactory extends MenuActionFactory {

    public FileMenuActionFactory(Manager manager, MainForm mainFrame) {
        super(manager, mainFrame);
    }
    
    public ActionListener getNewAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                MainForm mfrm;
                try {
                    mfrm = new MainForm();
                    mfrm.run();
                } catch (Exception e1) {
                    e1.printStackTrace();
                } 
            }};
    }
    
    public ActionListener getSaveAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String path = getManager().getDocumentPath();
                if (path != null)
                    try {
                        getManager().save();
                    } catch (IOException e1) {
                        JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.saveerror") , Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                    }
                else getSaveAsAction().actionPerformed(null);
            }
        };
    }

    
    public ActionListener getSaveAsAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(getMainForm().getLastUsedPath());
                fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                
                DefaultFileFilter dff = new DefaultFileFilter("dff", Lang.get("format.dff"));
                fc.addChoosableFileFilter(dff);
                fc.addChoosableFileFilter(new DefaultFileFilter("svg", Lang.get("format.svg")));
                fc.addChoosableFileFilter(new DefaultFileFilter("png", Lang.get("format.png")));
                fc.addChoosableFileFilter(new DefaultFileFilter(new String [] {"jpeg", "jpg"}, Lang.get("format.jpeg")));
                fc.addChoosableFileFilter(new DefaultFileFilter(new String [] {"tiff", "tif"}, Lang.get("format.tiff")));
                fc.addChoosableFileFilter(new DefaultFileFilter("pdf", Lang.get("format.pdf")));
                fc.setFileFilter(dff);
                
                int returnVal = fc.showSaveDialog(getMainForm());
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                   getMainForm().setCursor(CursorFactory.getWaitCursor());
                   File file = fc.getSelectedFile();
                   getMainForm().setLastUsedPath(file);
                   getMainForm().repaint();
                   try {
                       if (export(file, ".svg", Export.SVG)) exportWarn(); 
                       else if (export(file, ".png", Export.PNG)) exportWarn();
                       else if (export(file, ".jpg", Export.JPEG)) exportWarn();
                       else if (export(file, ".jpeg", Export.JPEG)) exportWarn();
                       else if (export(file, ".tiff", Export.TIFF)) exportWarn();
                       else if (export(file, ".tif", Export.TIFF)) exportWarn();
                       else if (export(file, ".pdf", Export.PDF)) exportWarn();
                       else getManager().saveToFile(file.getAbsoluteFile());
                       
                   } catch (Exception ex) {
                       ex.printStackTrace();
                       JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.saveerror") , Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                   }

                   getMainForm().setCursor(CursorFactory.getDefaultCursor());
                   getManager().setDefaultContext();
                }
            }};
    }
    
    private void exportWarn() {
        JOptionPane.showMessageDialog(getMainForm(), 
                Lang.get("msg.exportinformationlost"), 
                Lang.get("msg.caption.caution"),
                JOptionPane.INFORMATION_MESSAGE);
    }
    
    private boolean export(File file, String extension, int format) throws Exception {
        if (file.getAbsoluteFile().toString().toLowerCase().endsWith(extension)) {
            getManager().export(file.getAbsoluteFile(), format);
            return true;
        } else {
            return false;
        }
    }
    
    public ActionListener getOpenAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(getMainForm().getLastUsedPath());
                DefaultFileFilter dff = new DefaultFileFilter("dff", Lang.get("format.dff"));
                fc.addChoosableFileFilter(dff);
                fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                int returnVal = fc.showOpenDialog(getMainForm());
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File file = fc.getSelectedFile();
                    getMainForm().setLastUsedPath(file);
                    try {
                        MainForm frm = new MainForm(file.getAbsolutePath());
                        frm.run();
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.unabletoopen", file.getAbsolutePath()), 
                                Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                    }
                }
            }};
    }
    
    public ActionListener getSaveIntoLibraryAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getManager().emptyDocument()) {
                    JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.emptydocument"), Lang.get("msg.caption.problem"), JOptionPane.WARNING_MESSAGE);
                    return;
                }
                String libName = SelectLibraryForm.showForm(getManager().getLibraries());
                if (libName != null) {
                    String elementName = JOptionPane.showInputDialog(getMainForm(), Lang.get("msg.input.elementname"));
                    if (elementName != null) {
                        int result = JOptionPane.YES_OPTION;
                        if (elementName.trim().length() == 0) result = JOptionPane.NO_OPTION; 
                        if (getManager().containsLibraryElement(libName, elementName)) {
                            result = JOptionPane.showConfirmDialog(getMainForm(), 
                                    Lang.get("msg.elementalreadyinlib", elementName),
                                    Lang.get("msg.caption.caution"),
                                    JOptionPane.YES_NO_OPTION);
                        }
                        if (result != JOptionPane.YES_OPTION) {
                            JOptionPane.showMessageDialog(getMainForm(),
                                    Lang.get("msg.elementnotsaved"),
                                    Lang.get("msg.caption.caution"),
                                    JOptionPane.INFORMATION_MESSAGE);
                        }
                        if (result == JOptionPane.YES_OPTION) {
                            int onlySelection = JOptionPane.NO_OPTION;
                            if (!getManager().getSelection().isEmpty())
                                onlySelection = JOptionPane.showConfirmDialog(getMainForm(), Lang.get("msg.selectionaslibrary"), Lang.get("msg.caption.caution"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                            Library lib = getManager().getLibrarian().getLibrary(libName);
                            lib.putElement(elementName, ((onlySelection == JOptionPane.YES_OPTION) ? (ComposedElement)getManager().getSelection() : (ComposedElement)getManager().getDocument()));
                            getManager().saveLibraries();
                            //getManager().saveIntoLibrary(libName, elementName);
                            getMainForm().refreshLibrariesElements();
                        }
                    }
                }
            }};
    }
    
    public ActionListener getSaveAsNewLibraryAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getManager().emptyDocument()) {
                    JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.emptydocument"), Lang.get("msg.caption.problem"), JOptionPane.WARNING_MESSAGE);
                    return;
                }
                Library lib = getManager().createEmptyLibrary();
                while (getManager().getLibraries().contains(lib.getName())) {
                    int result = JOptionPane.showConfirmDialog(
                            getMainForm(),
                            Lang.get("msg.libraryinsystem", lib.getName()),
                            Lang.get("msg.caption.caution"),
                            JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
                    
                    if (result == JOptionPane.CANCEL_OPTION) return;
                    if (result == JOptionPane.YES_OPTION) break;
                    if (result == JOptionPane.NO_OPTION) getManager().invokeLibraryPropertiesDialog(lib);
                }
                
                JFileChooser fc = new JFileChooser();
                fc.setCurrentDirectory(getMainForm().getLastUsedPath());
                fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                DefaultFileFilter dlf = new DefaultFileFilter("dlf", Lang.get("format.dlf"));
                fc.addChoosableFileFilter(dlf);
                int returnVal = fc.showSaveDialog(getMainForm());
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File file = fc.getSelectedFile();
                    getMainForm().setLastUsedPath(file);
                    
                   try {
                       String elementName = JOptionPane.showInputDialog(getMainForm(), Lang.get("msg.input.elementname"));
                       if ((elementName == null) || (elementName.trim().length() == 0)) {
                           JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.elementsaverror"), Lang.get("msg.caption.caution"), JOptionPane.WARNING_MESSAGE);
                       }
                       else {
                           lib.getProperties().getProperty(XmlConstants.PROP_URL).setValue(file.getAbsolutePath());
                           int onlySelection = JOptionPane.NO_OPTION;
                           if (!getManager().getSelection().isEmpty())
                               onlySelection = JOptionPane.showConfirmDialog(getMainForm(), Lang.get("msg.selectionaslibrary"), Lang.get("msg.caption.caution"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                           lib.putElement(elementName, ((onlySelection == JOptionPane.YES_OPTION) ? (ComposedElement)getManager().getSelection() : (ComposedElement)getManager().getDocument()));
                           getManager().putLibrary(lib);
                           getManager().saveLibraries();
                       }
                   } catch (Exception ex) {
                       ex.printStackTrace();
                       JOptionPane.showMessageDialog(getMainForm(), Lang.get("msg.librarysaverror"), Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                   }
                   getMainForm().refreshLibrariesElements();
                   return;
            }}};
    }
    
    public ActionListener getExitAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getMainForm().closeWindow();
            }};
    }
}
