/*
 * Created on Jun 28, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import diagnose.graphicengine.Manager;
import diagnose.gui.MainForm;
import diagnose.utils.Lang;
import diagnose.utils.Log;
import diagnose.utils.properties.Property;
import diagnose.utils.serialization.XmlConstants;

/**
 * Generates action listeners for formating actions
 */
public class FormatMenuActionFactory extends MenuActionFactory {

    public FormatMenuActionFactory(Manager manager, MainForm mainForm) {
        super(manager, mainForm);
    }

    /**
     * Document properties
     */
    public ActionListener getDocumentPropAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().invokeDocumentPropertiesDialog();
            }};
    } 
   
            
    public ActionListener getGroupAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().groupSelection();
            }};
    }
    
    public ActionListener getSplitGroupAction() {
        return new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            getManager().splitSelection();
        }};
    }

    public ActionListener getSendFrontAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().sendSelectionFront();
            }};
    }
    
    public ActionListener getSendBackAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().sendSelectionBack();
            }};
    }
    
    public ActionListener getPropertiesAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().invokeSelectionPropertiesDialog();
               
            }};
        }
    
    public ActionListener getAlignAction(final int direction) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                getManager().alignSelection(direction);
            }};
    }
    
    public ActionListener getTextPropertiesAction() {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                getManager().invokeFontStyleDialog();
            }};
    }
    
    public ActionListener getColorLinesPropertiesAction() {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                getManager().invokeLineStyleDialog();
            }};
    }
    
    public ActionListener getToggleFormatAction(final String propertyName) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Property property = getManager().getDefaultProperties().getProperty(propertyName);
                boolean value = ((Boolean)property.getValue()).booleanValue();
                getManager().setDefaulProperty(propertyName, new Boolean(!value));
            }};
    }
    
    public ActionListener getSetAlignAction(final int align) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {                
                getManager().setDefaulProperty(XmlConstants.PROP_ALIGN, new Integer(align));
            }};        
    }
    
    public ActionListener getSetFontAction(final JComboBox combo) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {                
                getManager().setDefaulProperty(XmlConstants.PROP_FONT_FACE, combo.getSelectedItem().toString());
            }};
    }
    
    public ActionListener getSetLineStyletAction(final JComboBox combo) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                getManager().setDefaulProperty(XmlConstants.PROP_LINE_STYLE, new Integer(combo.getSelectedIndex() + 1));
            }};
    }
    
    public ActionListener getSetFontSizeAction(final JComboBox combo) {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String value = combo.getSelectedItem().toString();
                try {
                    int val = Integer.parseInt(value);
                    getManager().setDefaulProperty(XmlConstants.PROP_FONT_SIZE, new Integer(val));
                } catch (Exception ex) {
                    Log.warn(Lang.get("invalid.int", value));
                }
            }};
    }
    
    public ChangeListener getSetLineWidthAction(final JSpinner spinner) {
        return new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                String value = spinner.getValue().toString();
                try {
                    double val = Double.parseDouble(value);
                    getManager().setDefaulProperty(XmlConstants.PROP_LINE_WIDTH, new Double(val));
                } catch (Exception ex) {
                    Log.warn(Lang.get("invalid.float", value));
                }
            }};
    }    
}