/*
 * Created on Jun 28, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.menu;

import java.awt.event.ActionListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import diagnose.utils.GUIUtils;
import diagnose.utils.IconCache;

/**
 * Support for MainForm to generate menus
 */
public class MenuFactory {
    
    private JMenu menu;
    
    public MenuFactory(String menuName) {
        super();
        menu = new JMenu(menuName);
    }
    
    /**
     * Takes over creation using existing menu
     */

    public MenuFactory(JMenu menu) {
        super();
        this.menu = menu;
    }

    /**
     * @return create menu
     */
    public JMenu getMenu() {
        return menu;
    }
    
    /**
     * Creates new menu item and adds into menu 
     */
    public JMenuItem addItem(String caption, ActionListener actionListener) {
        JMenuItem item = GUIUtils.createMenuItem(caption, actionListener);
        menu.add(item);
        return item;
    }
    
    /**
     * Creates new menu item and adds into menu 
     */
    public JCheckBoxMenuItem addCheckItem(String caption, ActionListener actionListener) {
        JCheckBoxMenuItem item = new JCheckBoxMenuItem(caption);
        item.addActionListener(actionListener);
        item.setIcon(IconCache.getSystemIcon("semiplain"));
        menu.add(item);
        return item;
    }
    
    /**
     * Creates new menu item and adds into menu 
     */
    public JMenuItem addItem(String caption, String iconName, ActionListener actionListener) {
        JMenuItem item = GUIUtils.createMenuItem(caption, iconName, actionListener);
        menu.add(item);
        return item;
    }

    /**
     * Creates new menu item and adds into menu 
     */
    public JMenuItem addItem(String caption, ActionListener actionListener, int key, int modifiers) {
        JMenuItem item = GUIUtils.createMenuItem(caption, actionListener, key, modifiers);
        menu.add(item);
        return item;
    }
    
    /**
     * Creates new menu item and adds into menu 
     */
    public JMenuItem addItem(String caption, String iconName, ActionListener actionListener, int key, int modifiers) {
        JMenuItem item = GUIUtils.createMenuItem(caption, iconName, actionListener, key, modifiers);
        menu.add(item);
        return item;
    }
    
    public void addSeparator() {
        menu.addSeparator();
    }
    
    public void addMenuFactory(MenuFactory menuFactory) {
        menu.add(menuFactory.getMenu());
    }
}
