/*
 * Created on Jun 28, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JOptionPane;

import diagnose.graphicengine.Manager;
import diagnose.gui.MainForm;
import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;

/**
 * Creates actions for menu view
 */
public class ViewMenuActionFactory extends MenuActionFactory {

    public ViewMenuActionFactory(Manager manager, MainForm mainForm) {
        super(manager, mainForm);
    }
    
    public ActionListener getZoomInAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setZoom(getManager().getZoom() + 0.1);
                getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getZoomInToolAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setZoomInContext();
                getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getZoomOutAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setZoom(getManager().getZoom() - 0.1);
                getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getZoomAction(final double zoom) {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setZoom(zoom);
                getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getHandToolAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setPaddingContext();
            }};
    }
    
    public ActionListener getDefaultContextAction() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getManager().setDefaultContext();
            }};
    }
        
    public ActionListener getcenterViewAction() {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
               getManager().setPadding(new Point(0, 0));
               getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getZoomOtherAction() {
        return new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String value = JOptionPane.showInputDialog(getMainForm(), Lang.get("dialog.zoom.zoom"), new Integer((int)(getManager().getZoom() * 100)));
                if (value == null) return;
                double val = 100;
                try {
                    val = Double.parseDouble(value);
                } catch (NumberFormatException ex) {
                    JOptionPane.showMessageDialog(getMainForm(), Lang.get("dialog.zoom.badvalue", value), Lang.get("msg.caption.problem"), JOptionPane.ERROR_MESSAGE);
                    return;
                }
                
                val /= 100;
                if (val < Manager.MIN_ZOOM) {
                    JOptionPane.showMessageDialog(getMainForm(), Lang.get("dialog.zoom.valuetoolow", value, Double.toString(Manager.MIN_ZOOM * 100)), Lang.get("msg.caption.info"), JOptionPane.INFORMATION_MESSAGE);
                    val = Manager.MIN_ZOOM;
                }
                
                if (val > Manager.MAX_ZOOM) {
                    JOptionPane.showMessageDialog(getMainForm(), Lang.get("dialog.zoom.valuetoohigh", value, Double.toString(Manager.MAX_ZOOM * 100)), Lang.get("msg.caption.info"), JOptionPane.INFORMATION_MESSAGE);
                    val = Manager.MAX_ZOOM;
                }
                
                getManager().setZoom(val);
                getManager().invokeViewChange();
            }};
    }
    
    public ActionListener getSnapToGridAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setSnapToGrid(!getManager().isSnapToGrid());
            }};
    }
    
    public ActionListener getSnapToGuideLines() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setSnapToGuideLines(!getManager().isSnapToGuideLines());
            }};
    }
    
    public ActionListener getSnapToElementsAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setSnapToElements(!getManager().isSnapToElements());
            }};
    }
    
    public ActionListener getSnapTechAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setSnapTech(!getManager().isSnapTech());
            }};
    }
    
    public ActionListener getShowGridAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setShowGrid(!getManager().isShowGrid());
            }};
    }
    
    public ActionListener getShowGuideLinesAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().setShowGuideLines(!getManager().isShowGuideLines());
            }};
    }
    
    public ActionListener getGridPropertiesAction() {
        return new ActionListener () {

            public void actionPerformed(ActionEvent e) {
                getManager().invokeGridPropertiesDialog();
            }};
    }
}
