/*
 * Created on Apr 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.properties;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.properties.PropertyDescription;

/**
 *
 */
public class BoolPropertyBox extends PropertyBox {

    private JCheckBox chkBox;
    
    /**
     * @param description
     * @param property
     */
    public BoolPropertyBox(PropertyDescription description, Property property, String invalidMessage) {
        super(description, property, invalidMessage);
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#validate()
     */
    public boolean validate() {
        return true;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#getComponent()
     */
    public JComponent getComponent() {
        chkBox = ComponentFactory.createCheckBox(getDescription().getDescription());
        chkBox.addActionListener(new DefaultChangeListener(chkBox));
        
        Boolean value = (Boolean)getProperty().getValue();
        if (value != null) chkBox.setSelected(value.booleanValue());
        else chkBox.setForeground(SHARED_UNCHANGED_COLOR);
        
        chkBox.setEnabled(!getDescription().isReadOnly());
        
        return chkBox;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#saveValues()
     */
    public PropertyChangeLog saveValues() {
        return (isChanged() ? new PropertyChangeLog(getProperty(), new Boolean(chkBox.isSelected())) : null);
    }
    
    private class DefaultChangeListener implements ActionListener {
        
        private Color oldColor;
        private JComponent labelComponent;
        
        public DefaultChangeListener(JComponent labelComponent) {
            this.labelComponent = labelComponent;
            oldColor = labelComponent.getForeground();
        }

        public void actionPerformed(ActionEvent e) {
            setChanged(true);
            labelComponent.setForeground(oldColor);
        }
    }
}
