/*
 * Created on Apr 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.properties;

import java.awt.Component;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.properties.PropertyDescription;
import diagnose.utils.properties.SelectPropertyDescription;

/**
 *
 */
public class IntPropertyBox extends PropertyBox {
    
    private JComponent valueField;

    /**
     * @param description
     * @param property
     */
    public IntPropertyBox(PropertyDescription description, Property property, String invalidMessage) {
        super(description, property, invalidMessage);
    }
    
    public JComponent getComponent() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        JLabel lbl = ComponentFactory.createLabel(getDescription().getDescription());
        panel.add(lbl);
        
        
        
        Integer value = (Integer)getProperty().getValue();
        
        if (!(getDescription() instanceof SelectPropertyDescription)) {
            valueField = new JTextField((value == null ? "" : value.toString()));
            ((JTextField)valueField).getDocument().addUndoableEditListener(new DefaultTextFieldChangeListener(lbl));
            valueField.setMaximumSize(NUMFIELD_MAXIMUM_SIZE);
            valueField.setPreferredSize(NUMFIELD_PREFERED_SIZE);
            
        } else { /////////////////////// COMBO ///////////////////
            
            SelectPropertyDescription desc = (SelectPropertyDescription)getDescription();
            valueField = createComboBox(desc);
            int index = (value != null ? getValueIndex(desc, getProperty().getValue()) : -1);
            ((JComboBox)valueField).setSelectedIndex(index);
            ((JComboBox)valueField).addItemListener(new DefaultComboChangeListener(lbl));
        }
        
        if (value == null)
            lbl.setForeground(SHARED_UNCHANGED_COLOR);
        
        lbl.setAlignmentX(Component.LEFT_ALIGNMENT);
        valueField.setAlignmentX(Component.LEFT_ALIGNMENT);
        valueField.setEnabled(!getDescription().isReadOnly());
        panel.add(valueField);
        return panel;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#validate()
     */
    public boolean validate() {
        if (!isChanged()) return true;
        try {
            if (valueField instanceof JTextField)
                Integer.valueOf(((JTextField)valueField).getText());
            
            if (valueField instanceof JComboBox)
                Integer.valueOf(getComboBoxValue((SelectPropertyDescription)getDescription(), (JComboBox)valueField));
            
            return true;
        } catch (NumberFormatException e) {
            return false;
        }    
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#saveValues()
     */
    public PropertyChangeLog saveValues() {
        
        if (isChanged()) {
            Integer value = null;
            if (valueField instanceof JTextField)
                value = Integer.valueOf(((JTextField)valueField).getText());
            
            if (valueField instanceof JComboBox)
                value = Integer.valueOf(getComboBoxValue((SelectPropertyDescription)getDescription(), (JComboBox)valueField));
            return new PropertyChangeLog(getProperty(), value);
        }
            
        return null;
    }
}
