/*
 * Created on Apr 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.properties;

import java.awt.Color;
import java.awt.Component;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import diagnose.utils.Lang;
import diagnose.utils.geometry.Point;
import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.properties.PropertyDescription;

/**
 *
 */
public class PointPropertyBox extends PropertyBox {

    private static final int DECIMAL_CORRECTION = 100;
    
    private JTextField xField;
    private JTextField yField;
    private JLabel titleLabel;
    private JLabel xLabel;
    private JLabel yLabel;
    
    /**
     * @param property
     */
    public PointPropertyBox(PropertyDescription description, Property property, String invalidMessage) {
        super(description, property, invalidMessage);
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#validate()
     */
    public boolean validate() {
        if (!isChanged()) return true;
        try {
            Double.valueOf(xField.getText());
            Double.valueOf(yField.getText());
            return true;
        } catch (NumberFormatException e) {
            return false;
        }    
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#getComponent()
     */
    public JComponent getComponent() {
        
        String presentedX;
        String presentedY;
        
        Point value = (Point)getProperty().getValue();
        
        if (value != null) {
            value = (Point)value.clone();
            Point px = (Point)getProperty().getValue();
            
            px.x = Math.rint((px.x * DECIMAL_CORRECTION)) / DECIMAL_CORRECTION;
            px.y = Math.rint((px.y * DECIMAL_CORRECTION)) / DECIMAL_CORRECTION;
        
            presentedX = Double.toString(px.x);
            presentedY = Double.toString(px.y);
        } else {
            presentedX = "";
            presentedY = "";
        }
        
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        titleLabel = ComponentFactory.createLabel(getDescription().getDescription());
        xLabel = ComponentFactory.createLabel(Lang.get("dialog.pointproperty.x"));
        yLabel = ComponentFactory.createLabel(Lang.get("dialog.pointproperty.y"));
        
        panel.add(titleLabel);
        panel.add(xLabel);
        xField = new JTextField(presentedX);
        xField.setMaximumSize(NUMFIELD_MAXIMUM_SIZE);
        xField.setPreferredSize(NUMFIELD_PREFERED_SIZE);
        
        panel.add(xField);
        
        panel.add(yLabel);
        yField = new JTextField(presentedY);
        yField.setMaximumSize(NUMFIELD_MAXIMUM_SIZE);
        yField.setPreferredSize(NUMFIELD_PREFERED_SIZE);
        panel.add(yField);
        
        xField.getDocument().addUndoableEditListener(new DefaultPointChangeListener());
        yField.getDocument().addUndoableEditListener(new DefaultPointChangeListener());
        
        xField.setEnabled(!getDescription().isReadOnly());
        yField.setEnabled(!getDescription().isReadOnly());
        
        titleLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
        xLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
        yField.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        if (value == null) {
            titleLabel.setForeground(SHARED_UNCHANGED_COLOR);
            xLabel.setForeground(SHARED_UNCHANGED_COLOR);
            yLabel.setForeground(SHARED_UNCHANGED_COLOR);
        }
        
        return panel;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#saveValues()
     */
    public PropertyChangeLog saveValues() {
        if (isChanged()) {
            double xVal, yVal;
            xVal = Double.parseDouble(xField.getText());   
            yVal = Double.parseDouble(yField.getText());
            
            return new PropertyChangeLog(getProperty(), new Point(xVal, yVal));
        }
        
        return null;
    }
    
    protected class DefaultPointChangeListener implements UndoableEditListener {
        
        private Color oldColor1;
        private Color oldColor2;
        private Color oldColor3;
        
        public DefaultPointChangeListener() {
            oldColor1 = titleLabel.getForeground();
            oldColor2 = xLabel.getForeground();
            oldColor3 = yLabel.getForeground();
        }

        public void undoableEditHappened(UndoableEditEvent e) {
            setChanged(true);
            titleLabel.setForeground(oldColor1);
            xLabel.setForeground(oldColor2);
            yLabel.setForeground(oldColor3);
        }
    }
}
