/*
 * Created on Apr 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.properties;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import diagnose.utils.IconCache;
import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.properties.PropertyDescription;
import diagnose.utils.properties.SelectPropertyDescription;

/**
 * Abstract class for inserting of properties into property dialogs
 */
public abstract class PropertyBox {

    private Property property;
    private PropertyDescription description;
    private String invalidMessage;
    private boolean changed = false;
    
    public static final int DECIMAL_POINTS = 3;
    public static Color SHARED_UNCHANGED_COLOR = Color.BLUE;
    
    protected static final Dimension NUMFIELD_PREFERED_SIZE = new Dimension(80, 24);
    protected static final Dimension NUMFIELD_MAXIMUM_SIZE = new Dimension(80, 24);
    
    public PropertyBox(PropertyDescription description, Property property, String invalidMessage) {
        super();
        this.property = property;
        this.description = description;
        this.invalidMessage = invalidMessage;
    }

    public PropertyDescription getDescription() {
        return description;
    }
    
    public void setDescription(PropertyDescription description) {
        this.description = description;
    }
    
    public Property getProperty() {
        return property;
    }
    
    public void setProperty(Property property) {
        this.property = property;
    }
    
    public String getInvalidMessage() {
        return invalidMessage;
    }
    
    public void setInvalidMessage(String invalidMessage) {
        this.invalidMessage = invalidMessage;
    }
    
    /**
     * @return true if the value in field is valid
     */
    public abstract boolean validate();
    
    /**
     * @return component which can be inserted into property dialog
     */
    public abstract JComponent getComponent();
    
    /**
     * Saves values in to properties
     */
    public abstract PropertyChangeLog saveValues();
    
    
    /**
     * Creates combo box for select style property
     */
    protected static JComboBox createComboBox(SelectPropertyDescription desc) {
        Object [] items = new Object [desc.getItems().size()];
        
        for (int i = 0; i < items.length; i++) {
            if (desc.getItems().get(i).toString().startsWith("@")) {
                String url = desc.getItems().get(i).toString().substring(1);
                items[i] = IconCache.getIcon(url); 
            } else items[i] = desc.getItems().get(i);
        }
        
        JComboBox combo = new JComboBox(items);
        combo.setMaximumSize(new Dimension(200, 24));
        
        return combo; 
    }

    /**
     * @return value according to selection of combobox (created using
     * createComboBox) and value specified in property description
     */
    protected static String getComboBoxValue(SelectPropertyDescription desc, JComboBox combo) {
        int itemIndex = combo.getSelectedIndex();
        return ((SelectPropertyDescription.SelectionItem)(desc.getItems().get(itemIndex))).value;        
    }
    
    /**
     * @return selection index for value
     */
    protected static int getValueIndex(SelectPropertyDescription desc, Object value) {
        if (value == null) return -1;
        for (int i = 0; i < desc.getItems().size(); i++) {
            String itemValue = ((SelectPropertyDescription.SelectionItem)(desc.getItems().get(i))).value;
            if (itemValue.equals(value.toString())) return i;
        }
        return -1;
    }

    protected boolean isChanged() {
        return changed;
    }

    protected void setChanged(boolean changed) {
        this.changed = changed;
    }
    
    protected class DefaultTextFieldChangeListener implements UndoableEditListener {
        
        private Color oldColor;
        private JComponent labelComponent;
        
        public DefaultTextFieldChangeListener(JComponent labelComponent) {
            this.labelComponent = labelComponent;
            oldColor = labelComponent.getForeground();
        }

        public void undoableEditHappened(UndoableEditEvent e) {
            setChanged(true);
            labelComponent.setForeground(oldColor);
        }
    }
    
   protected class DefaultComboChangeListener implements ItemListener {
        
        private Color oldColor;
        private JComponent labelComponent;
        
        public DefaultComboChangeListener(JComponent labelComponent) {
            this.labelComponent = labelComponent;
            oldColor = labelComponent.getForeground();
        }

        public void itemStateChanged(ItemEvent e) {
            setChanged(true);
            labelComponent.setForeground(oldColor);
        }
    }
}
