/*
 * Created on Apr 29, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.gui.properties;

import java.awt.Component;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import diagnose.utils.properties.Property;
import diagnose.utils.properties.PropertyChangeLog;
import diagnose.utils.properties.PropertyDescription;
import diagnose.utils.properties.SelectPropertyDescription;

/**
 *
 */
public class StringPropertyBox extends PropertyBox {

    private boolean multiline;
    private JComponent textField;

    /**
     * @param description
     * @param property
     */
    public StringPropertyBox(PropertyDescription description, Property property, String invalidMessage, boolean multiline) {
        super(description, property, invalidMessage);
        this.multiline = multiline;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#validate()
     */
    public boolean validate() {
        return true;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#getComponent()
     */
    public JComponent getComponent() {
        JPanel panel = new JPanel();
        JLabel descriptionLabel = ComponentFactory.createLabel(getDescription().getDescription());
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        String value = (String)getProperty().getValue();
        
        panel.add(descriptionLabel);

        if (!(getDescription() instanceof SelectPropertyDescription)) {
            if (multiline) {
                textField = new JTextArea(value == null ? "" : value, 5, 40);
                panel.add(new JScrollPane(textField));
            } else {
                textField = new JTextField(value == null ? "" : value);
                panel.add(textField);
            }
            
            ((JTextComponent)textField).getDocument().addUndoableEditListener(new DefaultTextFieldChangeListener(descriptionLabel));
            
        } else { ////////////////// COMBO //////////////////////////
            SelectPropertyDescription desc = (SelectPropertyDescription)getDescription();
            textField = createComboBox(desc);
            int index = (value != null ? getValueIndex(desc, getProperty().getValue()) : -1);
            ((JComboBox)textField).setSelectedIndex(index);
            setChanged(false);
            ((JComboBox)textField).addItemListener(new DefaultComboChangeListener(descriptionLabel));
            panel.add(textField);
        }
        
        if (value == null)
            descriptionLabel.setForeground(SHARED_UNCHANGED_COLOR);
        textField.setEnabled(!getDescription().isReadOnly());
        
        descriptionLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
        textField.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        
        return panel;
    }

    /* (non-Javadoc)
     * @see diagnose.gui.properties.PropertyBox#saveValues()
     */
    public PropertyChangeLog saveValues() {
        if (textField instanceof JTextComponent)
            return (isChanged() ? new PropertyChangeLog(getProperty(), ((JTextComponent)textField).getText()) : null);
        
        // it's a combo !
        return (isChanged() ? new PropertyChangeLog(getProperty(), 
                getComboBoxValue((SelectPropertyDescription)getDescription(), (JComboBox)textField)) : null);
    }
}
