/*
 * Created on Jun 5, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.RootPaneContainer;

import diagnose.graphicengine.libraries.LibElementIdentifier;

/**
 * Utils for GUI
 */
public final class GUIUtils {
    
    public static void centerWindow(Window win) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension winSize = win.getSize();
        
        if (winSize.height > screenSize.height)
            winSize.height = screenSize.height;
        
        if (winSize.width > screenSize.width)
            winSize.width = screenSize.width;
        
        win.setLocation((screenSize.height - winSize.height) / 2,
                (screenSize.width - winSize.width) / 2);
    }
    
    public static void setEscapeListener(RootPaneContainer win, ActionListener listener) {
        win.getRootPane().registerKeyboardAction(
                listener,
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW);
    }
    
    public static JMenuItem createMenuItem(String caption, ActionListener actionListener) {
        JMenuItem item = new JMenuItem(caption);
        item.addActionListener(actionListener);
        item.setIcon(IconCache.getSystemIcon("plain"));
        return item;
    }
    
    public static JMenuItem createMenuItem(String caption, String iconName, ActionListener actionListener) {
        JMenuItem item = new JMenuItem(caption);
        item.addActionListener(actionListener);
        item.setIcon(IconCache.getSystemIcon(iconName));
        return item;
    }

    public static JMenuItem createMenuItem(String caption, ActionListener actionListener, int key, int modifiers) {
        JMenuItem item = createMenuItem(caption, actionListener);
        item.setAccelerator(KeyStroke.getKeyStroke(key, modifiers));
        item.setIcon(IconCache.getSystemIcon("plain"));
        return item;
    }
    
    public static JMenuItem createMenuItem(String caption, String iconName, ActionListener actionListener, int key, int modifiers) {
        JMenuItem item = createMenuItem(caption, actionListener);
        item.setAccelerator(KeyStroke.getKeyStroke(key, modifiers));
        item.setIcon(IconCache.getSystemIcon(iconName));
        return item;
    }
    
    public static JButton makeIconBtn(String imageName,
            String toolTipText,
            String altText,
            ActionListener actionListener) {
        
        Icon ico = IconCache.getSystemIcon(imageName);

        //Create and initialize the button.
        JButton button = new JButton();
        button.setFocusable(false);
        button.setToolTipText(toolTipText);
        button.addActionListener(actionListener);

        if (ico != null) button.setIcon(ico);
        else button.setText(altText);
            

        return button;
    }
    
    /**
     * Format list of dependencies into user readable form
     */
    public static String getListOfMissingDependencies(ArrayList deps) {
        ArrayList elements = removeDuplicities(deps);
        StringBuilder eltString = new StringBuilder();
        for (int i = 0; i < elements.size(); i++) {
            LibElementIdentifier id = (LibElementIdentifier)elements.get(i);
            eltString.append(Lang.get("msg.elementfromlibrary", id.getElementName(), id.getLibraryName()) + "\n");
        }
        
        return eltString.toString();
    }
    
    private static ArrayList removeDuplicities(ArrayList elements) {
        ArrayList result = new ArrayList();
        for (int i = 0; i < elements.size(); i++) {
            boolean found = false;
            for (int j = 0; j < result.size(); j++) {
                if (elements.get(i).equals(result.get(j))) {
                    found = true;
                    break;
                }
            }
            if (!found) result.add(elements.get(i));
        }
        return result;
    }
    

}
