/*
 * Created on Jul 27, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;

import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import diagnose.utils.serialization.XmlConstants;

/**
 * Provides localizated strings
 */
public final class Lang {

    private static HashMap langSet = new HashMap();
    
    static {
        URL langUrl = Lang.class.getResource("/resources/cz.xml");
        load(langUrl);
    }
    
    public static String get(String key) {
        return (String)langSet.get(key);
    }
    
    /**
     * @return formated message with replaced parameters
     */
    public static String get(String key, String [] params) {
        return MessageFormat.format(get(key), params);
    }

    /**
     * @return formated message with replaced parameters
     */
    public static String get(String key, String param1) {
        return get(key, new String[] { param1 });
    }
    
    /**
     * @return formated message with replaced parameters
     */
    public static String get(String key, String param1, String param2) {
        return get(key, new String[] { param1, param2 });
    }
    
    /**
     * Loads language set
     */
    public static void load(URL path) {
        try {
            
            InputStream ins = path.openStream();
            Reader rd = new InputStreamReader(ins);
            //FileReader rd = new FileReader(fileName);
            SAXBuilder builder = new SAXBuilder();
            org.jdom.Document dc = builder.build(rd);
            
            List items = dc.getRootElement().getChildren();
            for (int i = 0; i < items.size(); i++) {
                Element elt = (Element)items.get(i);
                
                String itemName = elt.getAttribute(XmlConstants.LANG_ITEM_NAME).getValue();
                String itemValue = elt.getContent(0).getValue();
                
                langSet.put(itemName, itemValue);
            }
            
        } catch (Exception e) {
            Log.error("Unable to load localization file: " + path);
            e.printStackTrace();
        }
    }
    
    

}
