/*
 * Created on Apr 10, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.geometry;

import java.util.ArrayList;


/**
 * Represents rectangular area
 */
public class Rect implements Cloneable {

    private Point position;
    private double width;
    private double height;

    public Rect(double x, double y, double width, double height) {
        super();
        this.position = new Point(x, y);
        this.width = width;
        this.height = height;
    }
    
    public Rect(Point position, double width, double height) {
        super();
        this.position = position;
        this.width = width;
        this.height = height;
    }
    
    /**
     * Creates rectangular area - mark out by array of points
     */
    public Rect(ArrayList points) {
        
        double minX = Double.POSITIVE_INFINITY;
        double minY = Double.POSITIVE_INFINITY;
        double maxX = Double.NEGATIVE_INFINITY;
        double maxY = Double.NEGATIVE_INFINITY;
        
        for (int i = 0; i < points.size(); i++) {
            Point p = (Point)points.get(i);
            if (p.x < minX) minX = p.x;
            if (p.y < minY) minY = p.y;
            if (p.x > maxX) maxX = p.x;
            if (p.y > maxY) maxY = p.y;
        }
        this.position = new Point(minX, minY);
        this.width = Math.abs(maxX - minX);
        this.height = Math.abs(maxY - minY);
    }
    
    public Rect(Point p1, Point p2) {
        double minX = Math.min(p1.x, p2.x);
        double minY = Math.min(p1.y, p2.y);
        double maxX = Math.max(p1.x, p2.x);
        double maxY = Math.max(p1.y, p2.y);
        
        this.position = new Point(minX, minY);
        this.width = Math.abs(maxX - minX);
        this.height = Math.abs(maxY - minY);
    }
    
    /**
     * @return true if current rectangle contains whole rectangle <code>r</code>
     */
    public boolean contains(Rect r) {
        Point p1 = (Point)position.clone();
        Point p2 = (Point)r.getPosition().clone();
        
        p1.translate(width, height);
        p2.translate(r.getWidth(), r.getHeight());
        
        return ((r.getPosition().x >= position.x) &&
                (r.getPosition().y >= position.y) &&
                (p2.x <= p1.x) &&
                (p2.y <= p1.y));
        
    }
    
    /**
     * @return true if current rectangle contains point <code>p</code>
     */
    public boolean contains(Point p) {
        return ((p.x >= position.x) && (p.y >= position.y) && 
                (p.x <= position.x + width) && (p.y <= position.y + height));        
    }
    
    public double getHeight() {
        return height;
    }
    
    public void setHeight(double height) {
        this.height = height;
    }
    
    public Point getPosition() {
        return position;
    }
    
    public void setPosition(Point position) {
        this.position = position;
    }
    
    public double getWidth() {
        return width;
    }
    
    public void setWidth(double width) {
        this.width = width;
    }
    
    /**
     * @return opposite point for the position
     */
    public Point getOppositePoint() {
        return new Point(position.x + width, position.y + height);
    }
    
    public void translate(double x, double y) {
        getPosition().translate(x, y);
    }
    
    public void translate(Point direction) {
        getPosition().translate(direction);
    }
    
    public String toString() {
        return "[ Rect: " + position.x + "; " + position.y + "; " + width + "; " + height + " ]"; 
    }
    
    public Object clone() {
        Rect result = new Rect((Point)getPosition().clone(), getWidth(), getHeight());
        return result;
    }
}