/*
 * Created on Feb 11, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import org.jdom.Element;

import diagnose.utils.geometry.Point;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XmlConstants;

/**
 *
 */
public class PointProperty extends Property {
    
    public PointProperty() {
        super();
        this.value = new Point();
    }
    
    public PointProperty(String name, Point value) {
        super(name);
        this.value = value;
    }
    
    public Element serialize() {
        Element e = super.serialize();
        if (e != null) {
            Point p = (Point)getValue();
            e.setAttribute(XmlConstants.POINT_X, Double.toString(p.x));
            e.setAttribute(XmlConstants.POINT_Y, Double.toString(p.y));
        }
        return e;
    }
    
    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        super.deserialize(elt, deserializer);
        value = new Point(
                    Double.parseDouble(elt.getAttributeValue(XmlConstants.POINT_X)),
                    Double.parseDouble(elt.getAttributeValue(XmlConstants.POINT_Y)));
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.properties.Property#cloneValue()
     */
    protected Object cloneValue() {
        return ((Point)value).clone();
    }

    /* (non-Javadoc)
     * @see diagnose.graphicengine.XMLSeriazable#getElementType()
     */
    public String getElementType() {
        return XmlConstants.ELEMENT_POINT_PROPERTY;
    }
}