/*
 * Created on Feb 6, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.jdom.Element;

import diagnose.utils.geometry.Point;
import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.Serializer;
import diagnose.utils.serialization.XMLSeriazable;
import diagnose.utils.serialization.XmlConstants;

/**
 * Holds all elements properties
 */
public class Properties implements Cloneable, XMLSeriazable {
   
    private HashMap properties = new HashMap();
    private PropertyDescriptor propertyDescriptor = null;
    
    public void setProperty(Property p) {
        properties.put(p.getName(), p);
    }
    
    public void removeProperty(String name) {
        properties.remove(name);
    }
    
    public Property getProperty(String name) {
        return (Property)properties.get(name);
    }
    
    public boolean hasProperty(String name) {
        return (properties.get(name) != null);
    }
    
    public Object clone() {
        Properties result = new Properties();
        result.propertyDescriptor = this.propertyDescriptor;
        Iterator it = properties.keySet().iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            Property p = getProperty(name);
            result.setProperty((Property)p.clone());
        }
        
        return result;
    }

    public Element serialize() {
        if (properties.size() == 0) return null;
        return Serializer.serializeCollection(getElementType(), properties.values());
    }
    
    public Set getPropertyName() {
        return properties.keySet();
    }

    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        for (int i = 0; i < elt.getChildren().size(); i++)
            setProperty((Property)deserializer.deserialize((Element)elt.getChildren().get(i)));            
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_PROPERTIES;
    }
    
    public int getIntValue(String name) {
        return ((Integer)getProperty(name).getValue()).intValue();
    }
    
    public double getDoubleValue(String name) {
        return ((Double)getProperty(name).getValue()).doubleValue();
    }
    
    public boolean getBooleanValue(String name) {
        return ((Boolean)getProperty(name).getValue()).booleanValue();
    }
    
    public Point getPointValue(String name) {
        return (Point)getProperty(name).getValue();
    }
    
    public String getStringValue(String name) {
        return (String)getProperty(name).getValue();
    }
    
    public Color getColorValue(String name) {
        return (Color)getProperty(name).getValue();
    }
    
    public PropertyDescription getPropertyDescription(String name) {
        if (propertyDescriptor != null) return propertyDescriptor.getDescription(name);
        return null;
    }
    
    public PropertyDescriptor getPropertyDescriptor() {
        return propertyDescriptor;
    }
    
    public void setPropertyDescriptor(PropertyDescriptor propertyDescriptor) {
        this.propertyDescriptor = propertyDescriptor;
    }
    
    /**
     * @return list of properties with specified accessibility
     */
    public ArrayList getPropertiesNames() {
        ArrayList result = new ArrayList();
        Iterator e = properties.keySet().iterator();
        while (e.hasNext())
            result.add(getProperty((String)e.next()).getName());
        
        return result;
    }
}
