/*
 * Created on Feb 6, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import org.jdom.Element;

import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XMLSeriazable;
import diagnose.utils.serialization.XmlConstants;

/**
 * Common class that holds informations about elements features   
 */
public abstract class Property implements Cloneable, XMLSeriazable {
    
    public static final int STANDARD = 0x01;
    public static final int VIRTUAL = 0x02;
    
    private String name;
    private int modifiers;
    protected Object value;
    private PropertyChangedListener propertyChangedListener = null;
    private PropertyReadListener propertyReadListener = null;
    
    public Property() {
        super();
        modifiers = STANDARD;
    }
    
    public Property(String name) {
        super();
        this.name = name;
    }

    public Property(String name, Object value) {
        super();
        this.name = name;
        this.value = value;
    }



    public String getName() {
        return name;
    }
    
    public int getModifiers() {
        return modifiers;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public void setModifiers(int accessibility) {
        this.modifiers = accessibility;
    }
    
    public Object clone() {
        try {
	        Property result = (Property)this.getClass().newInstance();
	        result.setName(getName());
	        result.setModifiers(getModifiers());
	        if ((modifiers & VIRTUAL) == 0) result.value = this.cloneValue();
	        result.propertyChangedListener = this.propertyChangedListener;
	        result.propertyReadListener = this.propertyReadListener;
	        return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }    
   
   public Element serialize() {
       if ((modifiers & VIRTUAL) > 0) return null;
       Element e = builder.element(getElementType());
       e.setAttribute(XmlConstants.NAME, getName());
       return e;
   }   
   
	public Object getValue() {
	    if (propertyReadListener == null) return value;
	    return propertyReadListener.getValue();
	}
	
	public void setValue(Object value) {
	    Object old = this.value;
	    this.value = value;
	    if (propertyChangedListener != null) {
	        this.value = propertyChangedListener.propertyChanged(old, value);
	    }
	}
    
    public void setValue(Object value, boolean ignoreListener) {
        Object old = this.value;
        this.value = value;
        if (!ignoreListener && (propertyChangedListener != null)) {
            this.value = propertyChangedListener.propertyChanged(old, value);
        }
    }
	
    public PropertyChangedListener getPropertyChangedListener() {
        return propertyChangedListener;
    }
    
    public void setPropertyChangedListener(
            PropertyChangedListener propertyChangedListener) {
        this.propertyChangedListener = propertyChangedListener;
    }
    
    public PropertyReadListener getPropertyReadListener() {
        return propertyReadListener;
    }
    
    public void setPropertyReadListener(
            PropertyReadListener propertyReadListener) {
        this.propertyReadListener = propertyReadListener;
    }
    
   public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
       setName(elt.getAttributeValue(XmlConstants.NAME));
   }
   
   public String toString() {
       return value.toString();
   }
   
   public boolean equals(Object obj) {
       if (!(obj instanceof Property)) return false;
       Property p = (Property)obj;
       return (p.name.equals(this.name) && (p.getValue().equals(this.getValue())));
       
   }
   
   /**
    * This function have to return carbon copy of a value
    */
   protected abstract Object cloneValue();
}