/*
 * Created on Jun 18, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

/**
 * Holds information about property change
 */
public class PropertyChangeLog {
    
    private Property property;
    private Object oldValue;
    private Object newValue;
    
    public PropertyChangeLog(Property property, Object newValue) {
        super();
        this.property = property;
        this.oldValue = property.getValue();
        this.newValue = newValue;
    }
    
    public Object getNewValue() {
        return newValue;
    }
    
    public Object getOldValue() {
        return oldValue;
    }
    
    public Property getProperty() {
        return property;
    }
    
    public void applyChanges() {
        property.setValue(newValue);
    }
    
    public void revertChanges() {
        property.setValue(oldValue);
    }
    
    /**
     * @return true if the value of property changed
     */
    public boolean hasChanged() {
        return !getOldValue().equals(getNewValue());
    }
}
