/*
 * Created on Apr 28, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import org.jdom.Element;

import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XMLSeriazable;
import diagnose.utils.serialization.XmlConstants;

/**
 * Describes properties in forms
 */
public class PropertyDescription implements XMLSeriazable {

    private String propertyName;
    private String description;
    private String category;
    private boolean readOnly = false;
    private int position;
    
    public PropertyDescription() {
        super();
    }    
    
    public PropertyDescription(String propertyName, String description, String category, int position) {
        super();
        this.propertyName = propertyName;
        this.description = description;
        this.category = category;
        this.position = position;
    }
    
    public PropertyDescription(String propertyName, String description, String category, int position, boolean readOnly) {
        super();
        this.propertyName = propertyName;
        this.description = description;
        this.category = category;
        this.readOnly = readOnly;
        this.position = position;
    }
    
    public String getCategory() {
        return category;
    }
    
    public void setCategory(String category) {
        this.category = category;
    }
    
    public String getDescription() {
        return description;
    }
    
    public void setDescription(String description) {
        this.description = description;
    }
    
    public boolean isReadOnly() {
        return readOnly;
    }
    
    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }
    
    public int getPosition() {
        return position;
    }
    
    public void setPosition(int position) {
        this.position = position;
    }
    
    public String getPropertyName() {
        return propertyName;
    }

    public void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }

    public Element serialize() {
        Element result = builder.element(getElementType());
        result.setAttribute(XmlConstants.PROPDESC_NAME, getPropertyName());
        result.setAttribute(XmlConstants.PROPDESC_POSITION, Integer.toString(getPosition()));
        result.setAttribute(XmlConstants.PROPDESC_DESCRIPTION, getDescription());
        result.setAttribute(XmlConstants.PROPDESC_CATEGORY, getCategory());
        result.setAttribute(XmlConstants.PROPDESC_READONLY, isReadOnly() ? XmlConstants.TRUE : XmlConstants.FALSE);
        return result;
    }

    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        setPropertyName(elt.getAttributeValue(XmlConstants.PROPDESC_NAME));
        setPosition(Integer.parseInt(elt.getAttributeValue(XmlConstants.PROPDESC_POSITION)));
        setDescription(elt.getAttributeValue(XmlConstants.PROPDESC_DESCRIPTION));
        setCategory(elt.getAttributeValue(XmlConstants.PROPDESC_CATEGORY));
        setReadOnly(elt.getAttribute(XmlConstants.PROPDESC_READONLY).toString().equalsIgnoreCase(XmlConstants.TRUE) ? true : false);
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_PROPERTY_DESCRIPTION;
    }
    
    public boolean equals(Object obj) {
        if (!(obj instanceof PropertyDescription)) return false;
        PropertyDescription desc = (PropertyDescription)obj;
        return (desc.propertyName.equals(this.propertyName) 
                && (desc.category.equals(this.category)));
    }
}