/*
 * Created on Jul 15, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;
import org.jdom.Text;

import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XmlConstants;

/**
 * Allows select select from one of specified items
 */
public class SelectPropertyDescription extends PropertyDescription {

    private ArrayList items = new ArrayList();
    
    public SelectPropertyDescription() {
        super();
    }

    public SelectPropertyDescription(String propertyName,
            String description, String category, int position) {
        super(propertyName, description, category, position);
    }

    public SelectPropertyDescription(String propertyName,
            String description, String category, int position, boolean readOnly) {
        super(propertyName, description, category, position, readOnly);
    }
    
    public void addItem(String value, String text) {
        items.add(new SelectionItem(value, text));
    }

    public ArrayList getItems() {
        return items;
    }

    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        super.deserialize(elt, deserializer);
        
        items = new ArrayList();
        
        List newItems = elt.getChildren(XmlConstants.ITEM);
        for (int i = 0; i < newItems.size(); i++) {
            Element itemElt = (Element)newItems.get(i);
            
            SelectionItem selVal = new SelectionItem(
                    itemElt.getAttributeValue(XmlConstants.VALUE),
                    ((Text)itemElt.getContent().get(0)).getText());
            
            items.add(selVal);
        }
    }

    public Element serialize() {
        Element result = super.serialize();
        for (int i = 0; i < items.size(); i++) {
            Element itemElt = builder.element(XmlConstants.ITEM);
            
            SelectionItem selVal = (SelectionItem)items.get(i);
            itemElt.setAttribute(XmlConstants.VALUE, selVal.value);
            itemElt.addContent(builder.text(selVal.text));
            
            result.addContent(itemElt);
        }
        return result;
    }
    
    public String getElementType() {
        return XmlConstants.ELEMENT_SELECT_PROPERTY_DESCRIPTOR;
    }
    
    public static class SelectionItem {
        
        public String value;
        public String text;
        
        public SelectionItem(String value, String text) {
            super();
            this.text = text;
            this.value = value;
        }
        
        public String toString() {
            return text;
        }
    }
}