/*
 * Created on Apr 21, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.properties;

import org.jdom.CDATA;
import org.jdom.Content;
import org.jdom.Element;
import org.jdom.Text;

import diagnose.utils.serialization.DeserializationException;
import diagnose.utils.serialization.Deserializer;
import diagnose.utils.serialization.XmlConstants;

/**
 * Property for representing text information
 */
public class StringProperty extends Property {

    public StringProperty() {
        super();
        this.value = "";
    }

    public StringProperty(String name, String value) {
        super(name);
        this.value = value;
    }

    public void deserialize(Element elt, Deserializer deserializer) throws DeserializationException {
        super.deserialize(elt, deserializer);
        
        if (elt.getContent().size() > 0) { 
            Content cnt = (Content)elt.getContent().get(0);
            if (cnt instanceof CDATA) value = ((CDATA)cnt).getText();
            else if (cnt instanceof Text) value = ((Text)cnt).getText();
            return;
        }
        value = elt.getAttributeValue(XmlConstants.VALUE);
        if (value == null) value = "";
    }
    
    public Element serialize() {
        Element e = super.serialize();
        if (e != null) e.addContent(builder.text((String)value));
        return e;
    }
    
    protected Object cloneValue() {
        return new String((String)value);
    }

    public String getElementType() {
        return XmlConstants.ELEMENT_STRING_PROPERTY;
    }
}