/*
 * Created on Apr 15, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.serialization;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import diagnose.utils.Lang;
import diagnose.utils.Log;


/**
 * Class for desirializing saved files
 */
public class Deserializer {
    
    private HashMap elementClasses = new HashMap();
    
    /**
     * Opens document from file
     * @param file
     * @return document
     * @throws JDOMException
     * @throws IOException
     * @throws DeserializationException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    public Object deserialeze(File file) 
    		throws JDOMException, IOException, DeserializationException, InstantiationException, IllegalAccessException {
        
        FileReader rd = new FileReader(file);
        SAXBuilder builder = new SAXBuilder();
        org.jdom.Document dc = builder.build(rd);
        
        return deserialize(dc.getRootElement());
    }
    
    /**
     * @return deserializes JDOM element into given object
     * @throws DeserializationException
     */
    public Object deserialize(Element elt) throws DeserializationException {
        String objName = elt.getName();
       
        try {
            Class cls = (Class)elementClasses.get(objName);
            XMLSeriazable obj = (XMLSeriazable)cls.newInstance();
            obj.deserialize(elt, this);
            return obj;
        } catch (Exception e) {
            e.printStackTrace();
            throw new DeserializationException(Lang.get("log.unabletocreateinstance" ,objName));
        }
    }
    
    /**
     * Registers element class for deserialization. This is important
     * for opening files with such elements.
     */
    public void registerElementClass(Class cls) {
        
        try {
            Method met = cls.getMethod("getElementType", null);
            String name = (String)met.invoke(cls.newInstance(), null);
            elementClasses.put(name, cls);
        } catch (Exception e) {
            e.printStackTrace();
            Log.error(Lang.get("log.unabletoregisterclass" ,cls.toString()));
        }
    }
}