/*
 * Created on Apr 30, 2005
 *
 * (c) Petr Krajca <pkrajca@iol.cz>
 */

package diagnose.utils.serialization;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.Iterator;

import org.jdom.DefaultJDOMFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Unified intrface for serializing objects
 */
public class Serializer {
    
    private static final DefaultJDOMFactory builder = new DefaultJDOMFactory();
    
    public static Document serialize(XMLSeriazable obj) {
        return builder.document(obj.serialize());
    }
    
    public static void serialize(XMLSeriazable obj, Writer wr) throws IOException {
        XMLOutputter otp = new XMLOutputter(Format.getPrettyFormat());
        otp.output(serialize(obj), wr);
    }
    
    public static void serialize(XMLSeriazable obj, File f) throws IOException {
        FileWriter wr = new FileWriter(f);
        serialize(obj, wr);
        wr.close();
    }
    
    /**
     * @return serializad elements from collection (suppose that all elements support
     * XMLSerializable interface)
     */
    public static Element serializeCollection(String elementName, Collection collection) {
        Iterator it = collection.iterator();
        Element elt = builder.element(elementName);
        while (it.hasNext()) {
            XMLSeriazable item = (XMLSeriazable)it.next();
            Element e = item.serialize();
            if (e != null) elt.addContent(e);
        }
        
        return elt;    
    }
}
